@@ -20,6 +20,16 @@ This method creates a new base request instance. It also validates the
 session/user credentials, as appropriate for the run mode.  Finally, it sets up
 the database connections for the user.
 
+=item verify_csrf()
+
+This method verifies the C<csrf_token> value in request parameters (held in
+C<$self->{csrf_token}>) against the value in the session object.  When one is
+not defined or they are not equal, this function returns a PSGI triplet to be
+used as the response resulting in a 400 -- Bad Request.
+
+When the CSRF token matches, C<undef> is returned indicating processing is to
+continue.
+
 =item open_form()
 
 This sets a $self->{form_id} to be used in later form validation (anti-XSRF
@@ -50,6 +60,9 @@ If an index is specified, the merged keys are given a form of
 Returns the script and query string part of the URL of the GET request,
 without the script path, or undef.
 
+Returns a URL-decoded string to prevent double-encoding when the URL
+is round-tripped.x
+
 =cut
 
 =item upload([$filename])
@@ -131,6 +144,41 @@ Month info in hashref format in 01 => January format
 
 =back
 
+=item enabled_languages()
+
+Returns arrayref of hashes with the following keys:
+
+=over
+
+=item value
+
+The code of the language as per the CLDR
+
+=item text
+
+The name of the language, translated into the user's selected language
+
+=back
+
+=item enabled_countries()
+
+Returns arrayref of hashes with the following keys:
+
+=over
+
+=item id
+
+The internal identifier for the country
+
+=item short_name
+
+The 2-leter iso code of the country
+
+=item name
+
+The country's full name translated into the user's selected language
+
+=back
 
 =item report_renderer_ui
 
@@ -138,7 +186,7 @@ Returns a code reference to render a report on the UI - pass as the
 named argument 'renderer' to the C<LedgerSMB::Report->render> method.
 
   my $report = LedgerSMB::Report
-  $report->render($request, renderer => $request->report_renderer_ui);
+  $report->render( renderer => $request->report_renderer_ui);
 
 
 =item report_renderer_doc
@@ -147,7 +195,7 @@ Returns a code reference to render a report as a document - pass as the
 named argument 'renderer' to the C<LedgerSMB::Report->render> method.
 
   my $report = LedgerSMB::Report
-  $report->render($request, renderer => $request->report_renderer_doc);
+  $report->render( renderer => $request->report_renderer_doc);
 
 
 =item render_report($report)
@@ -187,29 +235,36 @@ use strict;
 use warnings;
 
 use Carp;
+use DateTime::Format::Duration::ISO8601;
 use Encode qw(perlio_ok);
 use HTTP::Headers::Fast;
-use HTTP::Status qw( HTTP_OK ) ;
-use Log::Log4perl;
+use HTTP::Status qw( HTTP_OK HTTP_BAD_REQUEST );
+use List::Util qw( pairgrep );
+use Locale::CLDR;
+use Locales unicode => 1;
+use Log::Any;
+use Math::BigFloat;
+use Math::BigInt;
 use PGObject;
 use Plack;
+use URI;
+use URI::Escape;
 
-use LedgerSMB::Sysconfig;
 use LedgerSMB::App_State;
 use LedgerSMB::Locale;
 use LedgerSMB::User;
 use LedgerSMB::Company_Config;
+use LedgerSMB::PSGI::Util qw( template_response );
 use LedgerSMB::Setting;
 use LedgerSMB::Template;
-use LedgerSMB::Template::UI;
 
-our $VERSION = '1.8.0';
-
-my $logger = Log::Log4perl->get_logger('LedgerSMB');
+our $VERSION = '1.11.11';
 
+my $logger = Log::Any->get_logger(category => 'LedgerSMB');
+my $expiration_parser = DateTime::Format::Duration::ISO8601->new;
 
 sub new {
-    my ($class, $request) = @_;
+    my ($class, $request, $wire) = @_;
     my $self = {};
     bless $self, $class;
 
@@ -232,13 +287,35 @@ sub new {
     $self->{_logout} = $request->env->{'lsmb.invalidate_session_cb'};
     $self->{_setting} = $request->env->{'lsmb.setting'};
     $self->{_req} = $request;
+    $self->{_wire} = $wire;
 
-    $self->_process_args($request->parameters);
+    my $q = $self->{query_string} // '';
+    $self->{_uri} = URI->new(
+        $request->env->{'lsmb.script'} . ($q ? "?$q" : ''),
+        $request->request_uri
+        );
+
+    # Initialize ourselves from parameters in $self->{_req}
+    $self->_process_args;
     $self->_set_default_locale();
 
     return $self;
 }
 
+
+sub verify_csrf {
+    my ($self) = @_;
+    my $got = $self->{csrf_token};
+    my $want = $self->{_req}->env->{'lsmb.session'}->{csrf_token};
+    if (not ($got and $want and $got eq $want)) {
+        $logger->debug( "CSRF have '$got'; want '$want'" );
+        return [ HTTP_BAD_REQUEST,
+                 [ 'Content-Type' => 'text/plain; charset=ascii' ],
+                 [ 'Bad request: CSRF token failure' ] ];
+    }
+    return undef;
+}
+
 sub open_form {
     my ($self) = @_;
     my @vars = $self->call_procedure(procname => 'form_open',
@@ -271,11 +348,12 @@ sub initialize_with_db {
         $sth = $self->{dbh}->prepare('SELECT user__check_my_expiration()')
             or die $self->{dbh}->errstr;
         $sth->execute or die $sth->errstr;
-        ($self->{pw_expires})  = $sth->fetchrow_array;
+        my ($pw_expires) = $sth->fetchrow_array;
+        $self->{pw_expires} = $expiration_parser->parse_duration($pw_expires);
     }
 
     $self->{_company_config} =
-        LedgerSMB::Company_Config::initialize($self);
+        LedgerSMB::Company_Config::initialize($self->{dbh});
 
     $self->get_user_info;
 
@@ -292,16 +370,19 @@ sub initialize_with_db {
 
 sub get_user_info {
     my ($self) = @_;
-    $LedgerSMB::App_State::User =
+    LedgerSMB::App_State::set_User(
         $self->{_user} =
-        LedgerSMB::User->fetch_config($self);
+        LedgerSMB::User->fetch_config($self));
     return $self->{_user}->{language} ||= 'en';
 }
 
 sub _set_default_locale {
     my ($self) = @_;
 
-    my $lang = $LedgerSMB::Sysconfig::language;
+    my $lang = $self->{_wire}->get( 'default_locale' )
+        ->from_header( $self->{_req}->header( 'Accept-Language' ) );
+
+    $self->{_user}->{language} = $lang;
     $self->{_locale}=LedgerSMB::Locale->get_handle($lang);
     $self->error( __FILE__ . ':' . __LINE__
                   . ": Locale ($lang) not loaded: $!\n" )
@@ -311,22 +392,27 @@ sub _set_default_locale {
 }
 
 sub _process_args {
-    my ($self, $args) = @_;
-
-    for my $key (keys %$args){
-        my @values = grep { defined $_ && $_ ne '' } $args->get_all($key);
-        next if ! @values;
-
-        $self->{$key} = (@values == 1) ? $values[0] : \@values;
-    }
-    return;
-}
-
-sub get_relative_url {
     my ($self) = @_;
 
-    return $self->{script} .
-        ($self->{query_string} ? "?$self->{query_string}" : '');
+    # Prefer body parameters over query string parameters
+    # Normally, they shouldn't both be present, but there's at least a bug in
+    # Safari 15 which submits query parameters, even when told not to.
+    #
+    # $self->{_req}->parameters values query and body parameters equally, causing
+    # them to be collected into arrays when both are specified. This way, we
+    # prefer one over the other instead.
+    for my $args ($self->{_req}->query_parameters, $self->{_req}->body_parameters) {
+        for my $key (keys %$args){
+            my @values = grep { defined $_ && $_ ne '' } $args->get_all($key);
+            next if ! @values;
+
+            my $value = (@values == 1) ? $values[0] : \@values;
+            next if $value eq '_!lsmb!empty!_';
+
+            $self->{$key} = $value;
+        }
+    }
+    return;
 }
 
 sub upload {
@@ -382,11 +468,17 @@ sub upload {
             $encoding = 'UTF-32BE';
             $bom_length = 4;
         }
+        else { # no BOM
+            $encoding = 'UTF-8';
+            $bom_length = 0;
+        }
         sysseek $fh, 0, 0;
     }
 
-    if ($bom_length) {
+    if ($encoding) {
         binmode $fh, ':encoding(' . $encoding . ')';
+    }
+    if ($bom_length) {
         read($fh, my $unused, 1); # read the bom character
     }
 
@@ -396,7 +488,7 @@ sub upload {
 sub call_procedure {
     my $self = shift;
     my %args = @_;
-    $args{funcschema} ||= $LedgerSMB::Sysconfig::db_namespace;
+    $args{funcschema} ||= $self->{_wire}->get( 'db' )->schema;
     $args{funcname} ||= $args{procname};
     $args{dbh} = $self->{dbh};
     $args{args} ||= [];
@@ -541,13 +633,59 @@ sub all_years {
                            } @years ] };
 }
 
+sub enabled_languages {
+    my ($self) = @_;
+
+    my $l = Locales->new( $self->{_user}->{language} );
+    return [
+        map {
+            +{
+                value => $_->{code},
+                text => ucfirst($l->get_language_from_code($_->{code})
+                                // $_->{description})
+            }
+        } $self->call_procedure(funcname => 'person__list_languages')
+        ];
+}
+
+sub enabled_countries {
+    my ($self) = @_;
+
+    local $Math::BigInt::upgrade = undef;
+    local $Math::BigFloat::downgrade = undef;
+    my $regions = Locale::CLDR->new($self->{_user}->{language})->all_regions;
+    return [
+        map {
+            +{
+                $_->%*,
+                name => $regions->{$_->{short_name}} // $_->{name}
+            }
+        } $self->call_procedure(funcname => 'location_list_country')
+        ];
+}
 
 sub report_renderer_ui {
   my ($request) = @_;
-  my $ui = LedgerSMB::Template::UI->new_UI;
+  my $ui = $request->{_wire}->get('ui');
+  my $uri = $request->{_uri}->clone;
+  if (not pairgrep { $a eq 'company' } $uri->query_form) {
+      $uri->query_form(
+          $uri->query_form,
+          company => $request->{company},
+          );
+  }
 
   return sub {
       my ($template_name, $report, $vars, $cvars) = @_;
+      $vars->{REPORT_LINK} = $uri->as_string;
+      $vars->{SCRIPT} = $request->{script};
+      $vars->{SETTINGS} = {
+          papersize    => 'letter', # default paper size when not configured
+          (%{$request->{_company_config} // {}},)
+      };
+      $vars->{SETTINGS}->{company_name} ||= $request->{company};
+      $vars->{HIDDENS} = $request->{hiddens};
+      $vars->{FORM_ID} = $request->{form_id};
 
       return $ui->render($request, "Reports/$template_name", $vars, $cvars);
   };
@@ -555,45 +693,24 @@ sub report_renderer_ui {
 
 sub report_renderer_doc {
     my ($request) = @_;
+    my $renderer =
+        $request->{_wire}->get( 'output_formatter' )->report_doc_renderer(
+            $request->{_dbh},
+            uc($request->{format}) || 'HTML',
+            {
+                SETTINGS => {
+                     # default paper size when not configured
+                    papersize    => 'letter',
+                    (%{$request->{_company_config} // {}},)
+                }
+            });
 
     return sub {
         my ($template_name, $report, $vars, $cvars) = @_;
-        my $template = LedgerSMB::Template->new( # printed document
-            template => $template_name,
-            user     => $request->{_user},
-            path     => 'DB',
-            dbh      => $request->{dbh},
-            format   => uc($request->{format} || 'HTML'),
-            output_options => {
-                filename => $report->output_name($request),
-            },
-            );
 
-        $template->render($vars, $cvars);
-
-        my $charset = '';
-        $charset = '; charset=utf-8'
-                          if $template->{mimetype} =~ m!^text/!;
-
-        # $request->{mimetype} set by format
-        my $headers = [
-            'Content-Type' => "$template->{mimetype}$charset",
-            ];
-
-        # Use the same Content-Disposition criteria as _http_output()
-        my $name =
-            $template->{output_options}->{filename} . '.' . lc($request->{format});
-        if ($name) {
-            $name =~ s#^.*/##;
-            push @$headers,
-                ( 'Content-Disposition' =>
-                  qq{attachment; filename="$name"} );
-        }
-
-        my $body = $template->{output};
-        utf8::encode($body) if utf8::is_utf8($body); ## no critic
-
-        return [ HTTP_OK, $headers, [ $body ] ];
+        return template_response(
+            $renderer->( $template_name, $report, $vars, $cvars ),
+            disposition => 'attach' );
     };
 }
 
@@ -610,7 +727,7 @@ sub render_report {
         # render as UI element
         $renderer = $request->report_renderer_ui;
     }
-    return $report->render($request, renderer => $renderer);
+    return $report->render( renderer => $renderer);
 }
 
 =head1 LICENSE AND COPYRIGHT
