@@ -9,11 +9,14 @@ LedgerSMB::Database - APIs for database creation and management.
 
 This module wraps both DBI and the PostgreSQL commandline tools.
 
-  my $db = LedgerSMB::Database->new({
-       dbname => 'mycompany',
-       username => 'foo',
-       password => 'foospassword'
-  });
+  my $db = LedgerSMB::Database->new(
+       connect_data => {
+          user     => 'foo',
+          dbname   => 'mycompany',
+          password => 'foospassword',
+       },
+       schema   => 'non_public',
+  );
 
   $db->load_modules('LOADORDER');
 
@@ -36,10 +39,10 @@ use DBD::Pg;
 use DBI;
 use File::Spec;
 use File::Temp;
-use Log::Log4perl;
+use PGObject::Util::DBAdmin 'v1.6.1';
+
 use Moose;
 use namespace::autoclean;
-
 extends 'PGObject::Util::DBAdmin';
 
 use LedgerSMB::Database::Loadorder;
@@ -47,14 +50,28 @@ use LedgerSMB::Database::Loadorder;
 
 our $VERSION = '1.2';
 
-my $logger = Log::Log4perl->get_logger('LedgerSMB::Database');
 
 
+around BUILDARGS => sub {
+    my $orig  = shift;
+    my $class = shift;
+    my %args  = @_;
+
+    if ($args{schema} && $args{connect_data}) {
+        $args{connect_data}->{options} //= '';
+
+        my $enc_schema = ($args{schema} =~ s/['\\]/$1$1/gr);
+        $args{connect_data}->{options} .= qq{-c search_path=$enc_schema};
+    }
+    elsif ($args{schema}) {
+        die q{Missing 'connect_data' arg; expected to set default schema};
+    }
+    return $class->$orig( %args );
+};
+
 =head1 PROPERTIES
 
-=over
-
-=item source_dir
+=head2 source_dir
 
 Indicates the path to the directory which holds the 'Pg-database.sql' file
 and the associated changes, charts and gifi files.
@@ -66,10 +83,26 @@ to be the root of the LedgerSMB source tree.
 
 has source_dir => (is => 'ro', default => './sql');
 
-=back
+=head2 default_connect_options
 
 =cut
 
+has default_connect_options => (is => 'rw',
+                                default => sub {
+                                    {
+                                        PrintError          => 0,
+                                        AutoCommit          => 0,
+                                        AutoInactiveDestroy => 1,
+                                        pg_enable_utf8      => 1,
+                                        pg_server_prepare   => 0,
+                                    }
+                                });
+
+=head2 schema
+
+=cut
+
+has schema => (is => 'ro', default => 'public');
 
 =head1 METHODS
 
@@ -189,6 +222,7 @@ though the fullversion may give you an idea of what the actual version is run.
 
 sub _stringify_db_ver {
     my ($ver) = @_;
+    $ver //= 0;
     return join('.',
                 reverse
                 map {
@@ -208,14 +242,15 @@ sub _set_system_info {
     my %utf8_mode_desc = (
         '-1' => 'Auto-detect',
         '0'  => 'Never',
-        '1'  => 'Always'
+        '1'  => 'Always',
+        ''   => '<unspecified>', # happens when testing with DBD::Mock
         );
     $rv->{system_info} = {
         'PostgreSQL (client)' => _stringify_db_ver($dbh->{pg_lib_version}),
         'PostgreSQL (server)' => _stringify_db_ver($dbh->{pg_server_version}),
         'DBD::Pg (version)' => $DBD::Pg::VERSION->stringify,
         'DBI (version)' => $DBI::VERSION,
-         'DBD::Pg UTF-8 mode' => $utf8_mode_desc{$dbh->{pg_enable_utf8}},
+         'DBD::Pg UTF-8 mode' => $utf8_mode_desc{$dbh->{pg_enable_utf8} // ''},
         'PostgreSQL server encoding' => $server_encoding,
         'PostgreSQL client encoding' => $client_encoding,
     };
@@ -223,8 +258,10 @@ sub _set_system_info {
     return;
 }
 
+
 sub get_info {
     my $self = shift @_;
+    my $authdb = shift @_;
     my $retval = { # defaults
          appname => undef,
          version => undef,
@@ -234,12 +271,16 @@ sub get_info {
     local $@ = undef;
 
     my $dbh = eval { $self->connect({PrintError => 0, AutoCommit => 0}) };
-    if (!$dbh){ # Could not connect, try to validate existance by connecting
-                # to postgres and checking
-        $dbh = $self->new($self->export, (dbname => 'postgres'))
+    if ( !$dbh ) {
+        # Could not connect, try to validate existance by connecting
+        # to postgres and checking
+
+        $self->logger->debug(
+            "Can't connect to database; falling through to $authdb"
+            );
+        $dbh = $self->new($self->export, (dbname => $authdb))
             ->connect({PrintError=>0});
         return $retval unless $dbh;
-        $logger->debug("DBI->connect dbh=$dbh");
         _set_system_info($dbh, $retval);
 
         # don't assign to App_State::DBH, since we're a fallback connection,
@@ -262,15 +303,14 @@ sub get_info {
         $dbh->disconnect();
 
         return $retval;
-   } else { # Got a db handle... try to find the version and app by a few
-            # different means
-       $logger->debug("DBI->connect dbh=$dbh");
-
+    }
+    else {
+        # Got a db handle... try to find the version and app by a few
+        # different means
        $retval->{status} = 'exists';
        _set_system_info($dbh, $retval);
 
-       my $sth;
-       $sth = $dbh->prepare('SELECT SESSION_USER');
+       my $sth = $dbh->prepare('SELECT SESSION_USER');
        $sth->execute;
        $retval->{username} = $sth->fetchrow_array();
        $sth->finish();
@@ -290,30 +330,29 @@ sub get_info {
              }
        );
        $sth->execute();
-       my ($have_version_column) =
-       $sth->fetchrow_array();
+       my ($have_version_column) = $sth->fetchrow_array();
        $sth->finish();
 
        if ($have_version_column) {
-       # Legacy SL and LSMB
-       $sth = $dbh->prepare(
-           'SELECT version FROM defaults'
-           );
-       #avoid DBD::Pg::st fetchrow_hashref failed: no statement executing
-       my $rv=$sth->execute();
-       if(defined($rv))
-       {
-           if (my $ref = $sth->fetchrow_hashref('NAME_lc')) {
-           if ($ref->{version}){
-               $retval->{appname} = 'ledgersmb';
-               $retval->{version} = 'legacy';
-               $retval->{full_version} = $ref->{version};
+           # Legacy SL and LSMB
+           $self->logger->trace( 'Legacy SL or LSMB database' );
+           $sth = $dbh->prepare(
+               'SELECT version FROM defaults'
+               );
+           #avoid DBD::Pg::st fetchrow_hashref failed: no statement executing
+           my $rv=$sth->execute();
+           if (defined($rv)) {
+               if (my $ref = $sth->fetchrow_hashref('NAME_lc')) {
+                   if ($ref->{version}){
+                       $retval->{appname} = 'ledgersmb';
+                       $retval->{version} = 'legacy';
+                       $retval->{full_version} = $ref->{version};
 
-               $dbh->rollback();
-               return $retval;
+                       $dbh->rollback();
+                       return $retval;
+                   }
+               }
            }
-           }
-       }
        }
        $dbh->rollback;
        # LedgerSMB 1.2 and above
@@ -339,11 +378,14 @@ sub get_info {
        $sth = $dbh->prepare('SELECT fldvalue FROM defaults WHERE fldname = ?');
        $sth->execute('version');
        if (my $ref = $sth->fetchrow_hashref('NAME_lc')){
+            $self->logger->trace( 'SL' );
             $retval->{appname} = 'sql-ledger';
             $retval->{full_version} = $ref->{fldvalue};
             $retval->{version} = $ref->{fldvalue};
             $retval->{version} =~ s/(\d+\.\d+).*/$1/g;
        } else {
+            my $schema = $self->schema;
+            $self->logger->debug( "SL; possibly no version set; $schema" );
             $retval->{appname} = 'unknown';
        }
        $dbh->rollback;
@@ -351,6 +393,70 @@ sub get_info {
     return $retval;
 }
 
+=head2 connect($options)
+
+Calls C<PGObject::Util::DBAdmin>'s C<connect> method with C<$options>,
+merged with C<default_options> attribute's value.
+
+Upon succesfull return, sets the connection's C<client_min_messages>
+to C<warning>.
+
+This routine claims the C<private_LedgerSMB> slot in the database handle
+for storage of LedgerSMB internal data.
+
+=cut
+
+sub connect {
+    my $self    = shift;
+    my $options = shift // {};
+
+    my $dbh = $self->SUPER::connect(
+        {
+            ( %{ $self->default_connect_options }, %{ $options }, )
+        })
+        or return undef;
+
+    $dbh->{private_LedgerSMB} = {
+        schema    =>  $self->schema,
+    };
+    $dbh->do(q{set client_min_messages = 'warning'});
+
+    return $dbh;
+}
+
+=head2 require_version($dbh, $version)
+
+Checks for a setting called 'ignore_version' and returns immediately if this is
+set and true.
+
+Otherwise, requires a specific version (exactly).  Dies if doesn't match.
+
+The ignore_version setting is intended to be temporarily set during
+zero-downtime upgrades.
+
+=cut
+
+sub require_version {
+    my ($self, $dbh, $expected_version) = @_;
+
+
+    my $settings = $dbh->selectall_hashref(
+        q{SELECT * FROM defaults
+           WHERE setting_key IN ('ignore_version', 'version')},
+        'setting_key')
+        or die $dbh->errstr;
+
+    return if ($settings->{ignore_version}
+               and $settings->{ignore_version}->{value});
+
+    if ($expected_version eq $settings->{version}->{value}) {
+        return '';
+    }
+    else {
+        return $settings->{version}->{value};
+    }
+}
+
 =head2 $db->copy('new_name')
 
 Copies the existing database to a new name.
@@ -359,14 +465,10 @@ Copies the existing database to a new name.
 
 sub copy {
     my ($self, $new_name) = @_;
-    my $rc = $self->new($self->export, (dbname => $new_name)
-        )->create(copy_of => $self->dbname);
+    my $new_db = $self->new($self->export, (dbname => $new_name));
+    my $rc = $new_db->create(copy_of => $self->dbname);
 
-    (__PACKAGE__->new(
-         dbname    => $new_name,
-         username  => $self->username,
-         password  => $self->password,
-     ))->connect->do(
+    $new_db->connect({ AutoCommit => 1 })->do(
         q|SELECT setting__set('role_prefix',
                               coalesce((setting_get('role_prefix')).value,?))|,
         undef, 'lsmb_' . $self->dbname . '__');
@@ -387,19 +489,27 @@ tag in the LOADORDER file will be applied (the main use-case being migrations).
 sub load_base_schema {
     my ($self, %args) = @_;
 
+    my $dbh = $self->connect({ AutoCommit => 1 });
+    $dbh->do(
+        q|CREATE SCHEMA IF NOT EXISTS | . $dbh->quote_identifier($self->schema)
+        )
+        or die $dbh->errstr;
+    $self->logger->infof('Created schema "%s"', $self->schema);
     $self->run_file(
         file       => "$self->{source_dir}/Pg-database.sql",
+        vars       => {
+            lsmb_schema => $self->schema,
+        },
     );
-    my $dbh = $self->connect({ AutoCommit => 1 });
     my $sth = $dbh->prepare(
         q|select true
             from pg_class cls
             join pg_namespace nsp
                  on nsp.oid = cls.relnamespace
            where cls.relname = 'defaults'
-                 and nsp.nspname = 'public'
+                 and nsp.nspname = ?
              |);
-    $sth->execute();
+    $sth->execute($self->schema);
     my ($success) = $sth->fetchrow_array();
     $sth->finish();
 
@@ -427,6 +537,33 @@ Returns true when succesful, dies upon error.
 
 =cut
 
+sub _load_module {
+    my ($self, $dbh, $module) = @_;
+
+    $dbh->do(q{delete from defaults where setting_key = 'module_load_ok'})
+        or die $dbh->errstr;
+    $dbh->do(q{insert into defaults (setting_key, value)
+             values ('module_load_ok', 'no') })
+        or die $dbh->errstr;
+
+    my ($success, $stdout, $stderr) = $self->run_file_with_logs(
+        file => "$self->{source_dir}/modules/$module",
+        vars => {
+            lsmb_schema => $self->schema,
+        },
+        );
+    $success or die $stderr;
+
+    my $sth = $dbh->prepare(q{select value from defaults
+                            where setting_key = 'module_load_ok'});
+    $sth->execute;
+    my ($value) = $sth->fetchrow_array();
+    $sth->finish;
+    die "Module $module failed to load"
+        if not $value or $value ne 'yes';
+}
+
+
 sub load_modules {
     my ($self, $loadorder, $args) = @_;
 
@@ -440,57 +577,27 @@ sub load_modules {
         $mod =~ s/(\s+|#.*)//g;
         next unless $mod;
 
-        $dbh->do(q{delete from defaults where setting_key = 'module_load_ok'})
-            or die $dbh->errstr;
-        $dbh->do(q{insert into defaults (setting_key, value)
-                    values ('module_load_ok', 'no') })
-            or die $dbh->errstr;
-
-        my ($success, $stdout, $stderr) = $self->run_file_with_logs(
-            file => "$self->{source_dir}/modules/$mod",
-        );
-        $success or die $stderr;
-
-        my $sth = $dbh->prepare(q{select value from defaults
-                                   where setting_key = 'module_load_ok'});
-        $sth->execute;
-        my ($value) = $sth->fetchrow_array();
-        $sth->finish;
-        die "Module $mod failed to load"
-            if not $value or $value ne 'yes';
+        $self->_load_module($dbh, $mod);
     }
     close $fh or die "Cannot close $filename";
     return 1;
 }
 
-=head2 $db->load_coa({ country => '2-char-country-code',
-                       chart => 'name-of-chart',
-                       gifi => 'name-of-gifi',
+=head2 $db->load_sic({ country => '2-char-country-code',
                        sic => 'name-of-sic' })
 
-Loads the chart of accounts (and possibly GIFI/SIC tables) as specified in
-the chart of accounts file name given for the given 2-char (iso) country code.
+Loads the SIC table as specified in the chart of accounts file name given
+for the given 2-char (iso) country code.
 
 =cut
 
-sub load_coa {
+sub load_sic {
     my ($self, $args) = @_;
 
-    $self->run_file (
-        file         => "$self->{source_dir}/coa/$args->{country}/chart/$args->{chart}",
-        );
-
-    $args->{gifi} //= $args->{chart};
-    if (defined $args->{gifi}
-        && -f "$self->{source_dir}/coa/$args->{country}/gifi/$args->{gifi}"){
-        $self->run_file(
-            file        => "$self->{source_dir}/coa/$args->{country}/gifi/$args->{gifi}",
-            );
-    }
     if (defined $args->{sic}
         && -f "$self->{source_dir}/coa/$args->{country}/sic/$args->{sic}"){
         $self->run_file(
-            file        => "$self->{source_dir}/coa/$args->{country}/sic/$args->{sic}",
+            file => "$self->{source_dir}/coa/$args->{country}/sic/$args->{sic}",
             );
     }
     return;
@@ -507,12 +614,16 @@ Returns true when successful, dies on error.
 
 sub create_and_load {
     my ($self, $args) = @_;
+    $self->logger->info('Creating database');
     $self->create;
+    $self->logger->info('Loading schema into ' . $self->schema);
     $self->load_base_schema(
         log_stdout     => $args->{log},
         errlog  => $args->{errlog},
         );
+    $self->logger->info('Applying schema changes');
     $self->apply_changes();
+    $self->logger->info('Loading LedgerSMB application database modules');
     return $self->load_modules('LOADORDER', {
     log     => $args->{log},
     errlog  => $args->{errlog},
@@ -540,12 +651,13 @@ sub upgrade_modules {
                 })
         or die 'Modules failed to be loaded.';
 
-    my $dbh = $self->connect({PrintError=>0});
+    my $dbh = $self->connect({PrintError=>0, AutoCommit=>1});
     my $sth = $dbh->prepare(
           q{UPDATE defaults SET value = ? WHERE setting_key = 'version'}
     );
     $sth->execute($version)
         or die 'Version not updated.';
+    $dbh->disconnect;
 
     return 1;
 }
@@ -605,7 +717,7 @@ Returns a hashref of table names to rows.  The following tables are counted:
 
 =cut
 
-my @tables = qw(ar ap gl users entity_credit_account entity acc_trans oe);
+my @tables = qw(ar ap gl users entity_credit_account entity acc_trans oe parts orderitems invoice);
 
 sub stats {
     my ($self) = @_;
@@ -625,6 +737,7 @@ sub stats {
        { table => $_, query => $qt };
     } @tables;
 
+    $dbh->disconnect;
     return $results;
 }
 
