@@ -26,10 +26,9 @@ This module doesn't specify any (public) methods.
 
 use Moose::Role;
 use namespace::autoclean;
-with 'PGObject::Simple::Role' => { -excludes => [qw(_get_dbh _get_schema _get_prefix)], };
+with 'LedgerSMB::PGObject::Role' => { -excludes => [ '_get_dbh' ], };
 
 use LedgerSMB::App_State;
-use LedgerSMB::Sysconfig;
 
 # nulls come back from the db as undefs.
 # we have not put this in the main PGObject module because
@@ -50,8 +49,6 @@ around BUILDARGS => sub {
 };
 
 sub _get_dbh { return LedgerSMB::App_State::DBH() }
-sub _get_schema { return LedgerSMB::Sysconfig::db_namespace() }
-sub _get_prefix { return '' } # can be overridden
 
 
 =head1 LICENSE AND COPYRIGHT
