@@ -53,16 +53,15 @@ use HTTP::Status qw( HTTP_OK );
 use List::Util qw/any sum/;
 
 use LedgerSMB::Batch;
+use LedgerSMB::Business_Unit;
 use LedgerSMB::DBObject::Payment;
-use LedgerSMB::Magic qw( MAX_DAYS_IN_MONTH EC_VENDOR );
+use LedgerSMB::Magic qw( MAX_DAYS_IN_MONTH  BRU_DEPARTMENT  EC_VENDOR );
 use LedgerSMB::Num2text;
 use LedgerSMB::PGDate;
 use LedgerSMB::PGNumber;
 use LedgerSMB::Report::Invoices::Payments;
 use LedgerSMB::Request::Helper::ParameterMap;
-use LedgerSMB::Sysconfig;
 use LedgerSMB::Template;
-use LedgerSMB::Template::UI;
 
 
 # CT:  A few notes for future refactoring of this code:
@@ -103,7 +102,7 @@ sub payments {
     my $payment = LedgerSMB::DBObject::Payment->new(%$payment_data);
     $payment->get_metadata();
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'payments/payments_filter',
                              { request => $request,
                                        payment => $payment });
@@ -141,7 +140,7 @@ sub get_search_criteria {
     my $payment = LedgerSMB::DBObject::Payment->new(%$payment_data);
     $payment->get_metadata();
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render(
         $request,
         'Reports/filters/payments',
@@ -159,7 +158,7 @@ is saved.  For receipts, this just redirects to bulk_post currently.
 
 my $bulk_post_map = input_map(
     [ qr/^(?<fld>id|source|memo|paid)_(?<cid>\d+)$/ => '@contacts<cid>:%<fld>' ],
-    [ qr/^contact_label_(?<cid>\d+)$/ => '@contacts<cid>:%pay_to>' ],
+    [ qr/^contact_label_(?<cid>\d+)$/ => '@contacts<cid>:%pay_to' ],
     [ qr/^(?<fld>invoice_date|invnumber|due|payment|invoice|net)_(?<cid>\d+)_(?<invrow>\d+)$/
       => '@contacts<cid>:@invoices<invrow>:%<fld>' ],
     [ qr/(?<fld>cash_accno|ar_ap_accno)$/ => '%<fld>' ],
@@ -197,11 +196,6 @@ sub pre_bulk_post_report {
             name => $request->{_locale}->text('Source'),
             type => 'text',
         },
-        {
-            col_id => 'memo',
-            name => $request->{_locale}->text('Memo'),
-            type => 'text',
-        },
         {
             col_id => 'debits',
             name => $request->{_locale}->text('Debits'),
@@ -226,9 +220,14 @@ sub pre_bulk_post_report {
     # The user interface sets the 'id' field true-ish when the customer
     # is selected for inclusion in the bulk payment
     @{$data->{contacts}} = grep { $_->{id} } @{$data->{contacts}};
+    my %accounts = map {
+        $_->{accno} => $_
+    } $request->call_procedure(funcname => 'chart_list_all');
+
     for my $crow (@{$data->{contacts}}) {
         $crow->{accno} = $data->{ar_ap_accno};
-        $crow->{transdate} = $request->{payment_date};
+        $crow->{acc_description} = $accounts{$crow->{accno}}->{description};
+        $crow->{transdate} = $data->{payment_date};
         $crow->{amount} =
             sum map {
                 ($crow->{paid} eq 'some')
@@ -238,7 +237,7 @@ sub pre_bulk_post_report {
         $crow->{amount} *= -1
                     if ($request->{account_class} == EC_VENDOR);
         $crow->{debits} = ($crow->{amount} < 0) ? ($crow->{amount} * -1) : 0;
-        $crow->{credits} = ($crow->{amount} > 0) ? $crow->{amount} : 0;;
+        $crow->{credits} = ($crow->{amount} > 0) ? $crow->{amount} : 0;
     }
     my $rows = $data->{contacts};
 
@@ -248,10 +247,10 @@ sub pre_bulk_post_report {
 
     # Cash summary
     my $ref = {
-       accno     => $request->{cash_accno},
-       transdate => $request->{date_paid},
-       source    => $request->{_locale}->text('Total'),
-       amount    => $total,
+        accno           => $request->{cash_accno},
+        acc_description => $accounts{$request->{cash_accno}}->{description},
+        transdate       => $data->{payment_date},
+        amount          => $total,
     };
     $ref->{amount} *= -1;
 
@@ -272,22 +271,25 @@ sub pre_bulk_post_report {
 
     my $buttons = [{
         text  => $request->{_locale}->text('Save Batch'),
-        name  => 'action',
+        name  => '__action',
         value => 'post_payments_bulk',
         class => 'submit',
     }];
     delete $request->{$_}
-       for qw(action dbh);
-    $request->{hiddens} = { %$request }; # prevent circular reference
-    delete $request->{form_id};
-    my $template = LedgerSMB::Template::UI->new_UI;
-    return $template->render($request, 'Reports/display_report',
-                             {
-                                 request => $request,
-                                 columns => $cols,
-                                 rows    => $rows,
-                                 buttons => $buttons,
-                             });
+       for qw(__action dbh);
+    my $template = $request->{_wire}->get('ui');
+    return $template->render(
+        $request,
+        'Reports/display_report',
+        {
+            SCRIPT  => $request->{script},
+            HIDDENS => { %$request },
+            FORM_ID => $request->{form_id},
+            name    => $request->{_locale}->text('Payment batch'),
+            columns => $cols,
+            rows    => $rows,
+            buttons => $buttons,
+        });
 }
 
 
@@ -379,13 +381,35 @@ successful.
 
 sub post_payments_bulk {
     my ($request) = @_;
-    my $payment =  LedgerSMB::DBObject::Payment->new(%$request);
     if ($request->close_form){
+        my $payment =  LedgerSMB::DBObject::Payment->new(
+            dbh           => $request->{dbh},
+            ar_ap_accno   => $request->{ar_ap_accno},
+            account_class => $request->{account_class},
+            batch_id      => $request->{batch_id},
+            cash_accno    => $request->{cash_accno},
+            currency      => $request->{currency},
+            exchangerate  => $request->{exchangerate},
+            payment_date  => LedgerSMB::PGDate->from_input(
+                $request->{datepaid}),
+            );
         my $data = $bulk_post_map->($request);
+        $data->{contacts} = [ grep { $_->{id} } @{$data->{contacts}} ];
+        for my $contact (@{$data->{contacts}}) {
+            for my $invoice (@{$contact->{invoices}}) {
+                if ($contact->{paid} eq 'all') {
+                    $invoice->{payment} = $invoice->{net};
+                }
+
+                $invoice->{payment} =
+                    LedgerSMB::PGNumber->from_input($invoice->{payment});
+            }
+        }
+
         $payment->post_bulk($data);
     } else {
-        $payment->{notice} =
-           $payment->{_locale}->text('Data not saved.  Please try again.');
+        $request->{notice} =
+           $request->{_locale}->text('Data not saved.  Please try again.');
         return display_payments($request);
     }
 
@@ -418,7 +442,12 @@ sub print {
     }
 
     $payment->{format_amount} =
-        sub {return LedgerSMB::PGNumber->from_input(@_)->to_output(money => 1); };
+        sub {
+            my $args = shift;
+            return LedgerSMB::PGNumber
+                ->from_input($args->{amount})
+                ->to_output(%$args);
+    };
 
     my $data = $bulk_post_map->($request);
     if ($data->{multiple}){
@@ -441,7 +470,8 @@ sub print {
             my $inv_count;
             my $check_max_invoices = $request->setting->get(
                          'check_max_invoices'
-            );
+                );
+            $check_max_invoices ||= scalar(@{$contact->{invoices}});
             if ($check_max_invoices > scalar(@{$contact->{invoices}})) {
                 $inv_count = scalar(@{$contact->{invoices}});
             } else {
@@ -449,6 +479,7 @@ sub print {
             }
 
             for my $invoice (@{$contact->{invoices}}) {
+                last if scalar(@{$check->{invoices}}) > $inv_count;
                 if ($contact->{paid} eq 'some'){
                     $invoice->{paid} = LedgerSMB::PGNumber
                         ->from_input($invoice->{payment});
@@ -458,13 +489,14 @@ sub print {
                 } else {
                     $request->error('Invalid Payment Amount Option');
                 }
+                next unless defined $invoice->{paid};
+
                 $check->{amount} += $invoice->{paid};
                 $invoice->{paid} = $invoice->{paid}->to_output(
                     format => '1000.00',
                     money => 1
                 );
-                push @{$check->{invoices}}, $invoice
-                    if scalar(@{$check->{invoices}}) <= $inv_count;
+                push @{$check->{invoices}}, $invoice;
             }
             my $amt = $check->{amount}->copy;
             $amt->bfloor();
@@ -479,12 +511,13 @@ sub print {
         $template = LedgerSMB::Template->new( # printed document
             user => $payment->{_user},
             template => 'check_multiple',
-            format => uc $payment->{'format'},
             path => 'DB',
             dbh  => $request->{dbh},
             output_options => {
                filename => 'printed-checks',
             },
+            format_plugin   =>
+                 $request->{_wire}->get( 'output_formatter' )->get( uc $payment->{format} ),
             );
         $template->render(
             {
@@ -515,6 +548,8 @@ Displays the bulk payment screen with current data
 =cut
 
 sub update_payments {
+    my ($request) = @_;
+    $request->{action} = 'update_payments';
     return display_payments(@_);
 }
 
@@ -527,7 +562,7 @@ C<$request> is a L<LedgerSMB> object reference.
 Required request parameters:
 
   * dbh
-  * action
+  * __action
   * account_class [1|2]
   * batch_id
   * batch_date
@@ -587,7 +622,7 @@ sub display_payments {
 
         for my $invoice (@{$_->{invoices}}){
             my $req_payment_data = $req_contact_data->{$invoice->{id}};
-            if (($payment->{action} ne 'update_payments')
+            if (($payment->{__action} ne 'update_payments')
                   or (defined $request->{"id_$_->{contact_id}"})){
                 $request->{"paid_$_->{contact_id}"} = ''
                     unless defined $request->{"paid_$_->{contact_id}"};
@@ -605,7 +640,7 @@ sub display_payments {
             }
 
             my $fld = "payment_$_->{id}_$invoice->{id}";
-            if ('display_payments' eq $request->{action}) {
+            if ('display_payments' eq $request->{__action}) {
                 $payment->{$fld} = $invoice->{due};
             }
             else {
@@ -654,22 +689,17 @@ sub display_payments {
     }
     $payment->{grand_total} = $payment->{grand_total}->to_output(money  => 1);
     @{$payment->{media_options}} = (
-            {text  => $request->{_locale}->text('Screen'),
-             value => 'screen'});
-    for (keys %LedgerSMB::Sysconfig::printer){
-         push @{$payment->{media_options}},
-              {text  => $_,
-               value => $_};
-    }
-    if ($LedgerSMB::Sysconfig::latex){
-        @{$payment->{format_options}} = (
-              {text => 'PDF',        value => 'PDF'},
-              {text => 'Postscript', value => 'Postscript'},
-        );
-        $payment->{can_print} = 1;
-    }
+        { text  => $request->{_locale}->text('Screen'),
+          value => 'screen' },
+        $request->{_wire}->get( 'printers' )->as_options );
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    $payment->{format_options} = [
+        map { { text => $_, value => lc $_ } }
+        $request->{_wire}->get( 'output_formatter' )->get_formats
+        ];
+    $payment->{can_print} = scalar @{$payment->{format_options}};
+
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'payments/payments_detail',
                              { request => $request,
                                        payment => $payment });
@@ -697,9 +727,12 @@ sub payment {
 
     }
     # Lets set the data in a hash for the template system. :)
+    my @departments = LedgerSMB::Business_Unit
+        ->list(BRU_DEPARTMENT, undef, undef, LedgerSMB::PGDate->today());
     my $select = {
         script => 'payment.pl',
         stylesheet => $request->{_user}->{stylesheet},
+        departments => \@departments,
         login    => { name  => 'login',
                       value => $request->{_user}->{login}   },
         curr => {
@@ -728,14 +761,15 @@ sub payment {
             name  => 'type',
             value => $request->{type}
         },
+        # accessed in the form as 'action'
         action => {
-            name => 'action',
+            name => '__action',
             value => 'payment1_5',
             text => $request->{_locale}->text('Continue'),
         }
     };
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'payments/payment1', $select);
 }
 
@@ -791,12 +825,13 @@ sub payment1_5 {
                                value    => $dbPayment->{account_class}},
             type         => {  name  => 'type',
                                value => $request->{type}},
-            action       => {  name => 'action',
+            # accessed in the form as 'action'
+            action       => {  name => '__action',
                                value => 'payment2',
                                text =>  $request->{_locale}->text('Continue')}
         };
 
-        my $template = LedgerSMB::Template::UI->new_UI;
+        my $template = $request->{_wire}->get('ui');
         return $template->render($request, 'payments/payment1_5', $select);
     }
 }
@@ -1032,7 +1067,7 @@ sub payment2 {
             $uri_module='??';
         }
         #my $uri = $Payment->{account_class} == 1 ? 'ap' : 'ar';
-        my $uri = $uri_module . '.pl?action=edit&id='
+        my $uri = $uri_module . '.pl?__action=edit&id='
             . $invoice->{invoice_id} . '&login=' . $request->{login};
         my $invoice_id = $invoice->{invoice_id};
         my $invoice_amt = $invoice->{amount};
@@ -1054,11 +1089,12 @@ sub payment2 {
             optional          =>  $request->{"optional_pay_$invoice_id"},
             selected_account  =>  $request->{"account_$invoice_id"},
             selected_source   =>  $request->{"source_$invoice_id"},
+            description       =>  $invoice->{description},
             memo              =>  {
                 name  => "memo_invoice_$invoice_id",
                 value => $request->{"memo_invoice_$invoice_id"}
             },#END HASH
-            orig_topay_fx     => ($request->{"topay_fx_$invoice->{invoice_id}"} // LedgerSMB::PGNumber->new($due_fx)->to_output(money => 1)),
+            orig_topay_fx     => LedgerSMB::PGNumber->new($due_fx)->to_output(money => 1),
             topay_fx          =>  {
                 name  => "topay_fx_$invoice_id",
                 value => ($request->{"topay_fx_$invoice_id"}
@@ -1111,23 +1147,11 @@ sub payment2 {
         }
     }
     # We need to set the available media and format from printing
-    my @media_options;
-    push  @media_options, {value => 1, text => 'Screen'};
-    if ($#{LedgerSMB::Sysconfig::printer}) {
-        for (keys %{LedgerSMB::Sysconfig::printer}) {
-            push  @media_options, {value => 1, text => $_};
-        }
-    }
-    push  @media_options, {value => 1, text => 'e-mail'};
+    my @media_options =
+        ( {value => 1, text => 'Screen'},
+          $request->{_wire}->get( 'printers' )->as_options,
+          {value => 1, text => 'e-mail'});
 
-    #$request->error("@media_options");
-    my @format_options;
-    push @format_options, {value => 1, text => 'HTML'};
-    if (${LedgerSMB::Sysconfig::latex}) {
-        push  @format_options,
-        {value => 2, text => 'PDF' },
-        {value => 3, text => 'POSTSCRIPT' };
-    }
     # LETS BUILD THE SELECTION FOR THE UI
     # Notice that the first data inside this selection is the firs_load, this
     # will help payment2.html to know wether it is being called for the first time
@@ -1162,7 +1186,7 @@ sub payment2 {
         column_headers => \@column_headers,
         rows        =>  \@invoice_data,
         topay_subtotal => LedgerSMB::PGNumber->new(
-             (sum map {LedgerSMB::PGNumber->from_input($_->{topay} // 0)} @invoice_data) // 0
+             (sum map {LedgerSMB::PGNumber->from_input($_->{topay_fx}->{value} // 0)} @invoice_data) // 0
         )->to_output(money => 1),
         topay_state   => \@topay_state,
         vendorcustomer => {
@@ -1179,9 +1203,6 @@ sub payment2 {
                 {text => $vc_options[0]->{country}},
                 ]
         },
-        format => {
-            name => 'FORMAT',
-            options => \@format_options },
         media => {
             name => 'MEDIA',
             options => \@media_options  },
@@ -1195,14 +1216,14 @@ sub payment2 {
         )->to_output(money => 1),
         payment_total => LedgerSMB::PGNumber->new(
               (sum map {LedgerSMB::PGNumber->from_input($_->{amount} // 0)} @overpayment)
-            + (sum map {LedgerSMB::PGNumber->from_input($_->{topay}  // 0)} @invoice_data)
+            + (sum map {LedgerSMB::PGNumber->from_input($_->{topay_fx}->{value}  // 0)} @invoice_data)
             + LedgerSMB::PGNumber->from_input(0) # never end up with undef
         )->to_output(money => 1),
     };
 
     $select->{selected_account} = $vc_options[0]->{cash_account_id}
       unless defined $select->{selected_account};
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'payments/payment2', $select);
 }
 
@@ -1280,6 +1301,11 @@ sub post_payment {
     @array_options = $Payment->get_entity_credit_account();
     my $discount_account_id = $array_options[0]->{discount};
     @array_options = $Payment->get_open_invoices();
+
+    die q|Applicable discounts can't be posted due missing discount account in credit account|
+        if ((not defined $discount_account_id)
+            and (any { $_->{discount_tc} != 0 } @array_options));
+
     for my $ref (0 .. $#array_options) {
         if ($request->{"checkbox_$array_options[$ref]->{invoice_id}"}
             && ($request->{"topay_fx_$array_options[$ref]->{invoice_id}"})) {
@@ -1301,10 +1327,13 @@ sub post_payment {
             #
             my $sign = "$array_options[$ref]->{due_fx}" <=> 0;
             my $decimals = $request->{_company_config}->{decimal_places};
-            if ( $sign * LedgerSMB::PGNumber->from_input($array_options[$ref]->{due_fx})->bround($decimals)
-                 <
-                 $sign * LedgerSMB::PGNumber->from_input($request_topay_fx_bigfloat)->bround($decimals)
-                ){
+            my $_due_fx =
+                LedgerSMB::PGNumber->from_input($array_options[$ref]->{due_fx})
+                ->bfround(-$decimals); # round right of decimal sep
+            my $_to_pay_fx =
+                LedgerSMB::PGNumber->from_input($request_topay_fx_bigfloat)
+                ->bfround(-$decimals); # round right of decimal sep
+            if ( $sign * $_due_fx < $sign * $_to_pay_fx ){
                 # We need to store all the overpayments
                 # so we can use it on a new payment2 screen
                 $unhandled_overpayment += $request_topay_fx_bigfloat
@@ -1314,7 +1343,8 @@ sub post_payment {
             if ($temporary_discount != 0) {
                 push @amount, $temporary_discount;
                 push @cash_account_id, $discount_account_id;
-                push @source, $locale->text('Applied discount');
+                push @source, undef;
+                push @memo, $locale->text('Applied discount');
                 push @transaction_id, $array_options[$ref]->{invoice_id};
             }
 
@@ -1391,7 +1421,7 @@ sub post_payment {
     $Payment->post_payment();
     if ($request->{continue_to_calling_sub}) {
         $request->{payment_id} = $Payment->{payment_id};
-        return;
+        return $request->{payment_id};
     }
     else {
         # Our work here is done, ask for more payments.
@@ -1440,7 +1470,9 @@ sub print_payment {
       locale   => $Payment->{_locale},
       path     => 'DB',
       template => 'printPayment',
-      format => 'HTML' );
+      format_plugin   =>
+         $request->{_wire}->get( 'output_formatter' )->get( 'HTML' ),
+    );
     $template->render(
         {
             DBNAME => $request->{company},
@@ -1449,7 +1481,8 @@ sub print_payment {
     return
         [ 200,
           [ 'Content-Disposition' =>
-            'attachment; filename="printPayment.html"' ],
+            'attachment; filename="printPayment.html"',
+            'Content-Type' => 'text/html; charset=utf-8' ],
           [ $template->{output} ] ];
 }
 
@@ -1513,12 +1546,13 @@ sub use_overpayment {
 
     $ui->{curr} = \@currOptions;
     $ui->{entities} =  \@entities;
+    # accessed in the form as 'action'
     $ui->{action}   =  {
-        name => 'action',
+        name => '__action',
         value => 'use_overpayment2',
         text => $locale->text('Continue')
     };
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'payments/use_overpayment1', $ui);
 }
 
@@ -1690,7 +1724,7 @@ sub use_overpayment2 {
             }
             #lets make the href for the invoice
             my $uri = $Payment->{account_class} == 1 ? 'ap' : 'ar';
-            $uri .= '.pl?action=edit&id='
+            $uri .= '.pl?__action=edit&id='
                 . $Payment->{"invoice_id_$count"} . '&login='
                 . $request->{login};
 
@@ -1769,7 +1803,7 @@ sub use_overpayment2 {
 
                 #lets make the href for the invoice
                 my $uri = $Payment->{account_class} == 1 ? 'ap' : 'ar';
-                $uri .= '.pl?action=edit&id='
+                $uri .= '.pl?__action=edit&id='
                     . $avble_invoices[$ref]->{invoice_id}
                 . '&login=' . $request->{login};
 
@@ -1842,7 +1876,7 @@ sub use_overpayment2 {
 
     $ui->{hiddens} = \@hiddens;
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'payments/use_overpayment2', $ui);
 }
 
@@ -1970,9 +2004,9 @@ sub post_overpayment {
         if ($list_key->{"optional_discount_$count"} && $list_key->{"amount_$count"} == $list_key->{"due_$count"}) {
             push @{$list_key->{array_amount}}, $list_key->{"discount_$count"};
             push @{$list_key->{array_cash_account_id}}, $list_key->{"vc_discount_accno_$count"};
-            push @{$list_key->{array_source}}, $locale->text('Applied discount by an overpayment');
+            push @{$list_key->{array_source}}, undef;
             push @{$list_key->{array_transaction_id}}, $list_key->{"invoice_id_$count"};
-            push @{$list_key->{array_memo}}, undef;
+            push @{$list_key->{array_memo}}, $locale->text('Applied discount by an overpayment');
             push @{$list_key->{ovp_payment_id}}, undef;
         }
 
@@ -2048,17 +2082,4 @@ sub post_overpayment {
 
 =cut
 
-{
-    local ($!, $@) = (undef, undef);
-    my $do_ = 'scripts/custom/payment.pl';
-    if ( -e $do_ ) {
-        unless ( do $do_ ) {
-            if ($! or $@) {
-                warn "\nFailed to execute $do_ ($!): $@\n";
-                die (  "Status: 500 Internal server error(payment.pm)\n\n" );
-            }
-        }
-    }
-};
-
 1;
