@@ -24,37 +24,41 @@ management tasks.
 
 use strict;
 use warnings;
+use version;
 
+use Carp;
 use Digest::MD5 qw(md5_hex);
 use Encode;
-use HTTP::Status qw( HTTP_OK HTTP_UNAUTHORIZED );
-use Log::Log4perl;
+use File::Spec;
+use HTML::Escape;
+use HTTP::Status qw( HTTP_OK HTTP_INTERNAL_SERVER_ERROR HTTP_UNAUTHORIZED );
+use Log::Any;
 use MIME::Base64;
 use Scope::Guard;
-use Try::Tiny;
+use Feature::Compat::Try;
 
 use LedgerSMB;
 use LedgerSMB::App_State;
+use LedgerSMB::Company;
 use LedgerSMB::Database;
 use LedgerSMB::Database::Config;
-use LedgerSMB::DBObject::Admin;
-use LedgerSMB::DBObject::User;
+use LedgerSMB::Database::ConsistencyChecks;
 use LedgerSMB::Entity::User;
 use LedgerSMB::Entity::Person::Employee;
-use LedgerSMB::Locale;
+use LedgerSMB::I18N;
 use LedgerSMB::Magic qw( EC_EMPLOYEE HTTP_454 PERL_TIME_EPOCH );
 use LedgerSMB::Mailer;
 use LedgerSMB::PGDate;
 use LedgerSMB::PSGI::Util;
 use LedgerSMB::Setting;
 use LedgerSMB::Setup::SchemaChecks qw( html_formatter_context );
-use LedgerSMB::Sysconfig;
-use LedgerSMB::Template::UI;
 use LedgerSMB::Template::DB;
 use LedgerSMB::Database::Upgrade;
+use LedgerSMB::User;
 
 
-my $logger = Log::Log4perl->get_logger('LedgerSMB::Scripts::setup');
+
+my $logger = Log::Any->get_logger(category => 'LedgerSMB::Scripts::setup');
 my $CURRENT_MINOR_VERSION;
 if ( $LedgerSMB::VERSION =~ /(\d+\.\d+)./ ) {
     $CURRENT_MINOR_VERSION = $1;
@@ -84,9 +88,8 @@ sub authenticate {
 
 
 sub __default {
-
     my ($request) = @_;
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/credentials', $request);
 }
 
@@ -110,11 +113,11 @@ sub _get_database {
         if ( $request->{database} && $request->{database} =~ /postgres|template0|template1/);
 
     return (undef,
-            LedgerSMB::Database->new(
-                username => $creds->{login},
+            $request->{_wire}->get( 'db' )->instance(
+                user     => $creds->{login},
                 password => $creds->{password},
-                  dbname => $request->{database},
-    ));
+                dbname   => $request->{database},
+            ));
 }
 
 
@@ -218,6 +221,21 @@ sub get_dispatch_table {
         message => $request->{_locale}->text('LedgerSMB 1.8 db found.'),
         operation => $request->{_locale}->text('Rebuild/Upgrade?'),
         next_action => 'rebuild_modules' },
+      { appname => 'ledgersmb',
+        version => '1.9',
+        message => $request->{_locale}->text('LedgerSMB 1.9 db found.'),
+        operation => $request->{_locale}->text('Rebuild/Upgrade?'),
+        next_action => 'rebuild_modules' },
+      { appname => 'ledgersmb',
+        version => '1.10',
+        message => $request->{_locale}->text('LedgerSMB 1.10 db found.'),
+        operation => $request->{_locale}->text('Rebuild/Upgrade?'),
+        next_action => 'rebuild_modules' },
+      { appname => 'ledgersmb',
+        version => '1.11',
+        message => $request->{_locale}->text('LedgerSMB 1.11 db found.'),
+        operation => $request->{_locale}->text('Rebuild/Upgrade?'),
+        next_action => 'rebuild_modules' },
       { appname => 'ledgersmb',
         version => undef,
         message => $request->{_locale}->text('Unsupported LedgerSMB version detected.'),
@@ -241,9 +259,25 @@ sub login {
     my ($reauth, $database) = _get_database($request);
     return $reauth if $reauth;
 
-    my $server_info = $database->server_version;
+    my $template = $request->{_wire}->get('ui');
+    my $settings = $request->{_wire}->get( 'setup_settings' );
+    my $auth_db = ($settings and $settings->{auth_db}) // 'postgres';
+    $database->{connect_data}->{options} = "-c search_path=$database->{schema},public";
 
-    my $version_info = $database->get_info();
+    my $version_info = $database->get_info($auth_db);
+
+    my $server_version     = version->parse(
+        $version_info->{system_info}->{'PostgreSQL (server)'}
+        );
+    my $server_min_version = version->parse('10.0.0');
+
+    return $template->render($request,
+                             'setup/mismatch',
+                             {
+                                 found    => "$server_version",
+                                 required => "$server_min_version",
+                             })
+        if $server_version < $server_min_version;
 
     ($reauth) = _init_db($request);
     return $reauth if $reauth;
@@ -292,7 +326,6 @@ sub login {
             }
         }
     }
-    my $template = LedgerSMB::Template::UI->new_UI;
     return $template->render($request, 'setup/confirm_operation', $request);
 }
 
@@ -306,14 +339,16 @@ sub list_databases {
     my ($reauth, $database) = _get_database($request);
     return $reauth if $reauth;
 
-    my @results = $database->list_dbs;
+    my @results = $database->list_dbs(
+        $request->{_wire}->get('setup_settings')->{admin_db}
+        );
     $request->{dbs} = [];
     # Ideally we would extend DBAdmin->list_dbs to accept an argument containing a list of databases to exclude using a method similar to that shown at https://git.framasoft.org/framasoft/OCB/commit/7a6e94edd83e9e73e56d2d148e3238618
     # also, we should add a new function DBAdmin->list_dbs_this_user which only returns db's the currently auth'd user has access to. Once again the framasoft.org link shows a method of doing this
     # for now we simply use a fixed regex. It will cover many if not most use cases.
     @{$request->{dbs}} = map {+{ row_id => $_, db  => $_ }} grep { ! m/^(postgres|template0|template1)$/ } @results ;
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/list_databases', $request);
 }
 
@@ -327,14 +362,12 @@ sub list_users {
     my ($reauth) = _init_db($request);
     return $reauth if $reauth;
 
-    my $user = LedgerSMB::DBObject::User->new();
-    $user->set_dbh($request->{dbh});
-    my $users = $user->get_all_users;
+    my @users = LedgerSMB::User->get_all_users($request);
     $request->{users} = [];
-    for my $u (@$users) {
-        push @{$request->{users}}, {row_id => $u->{id}, name => $u->{username} };
+    for my $user (@users) {
+        push @{$request->{users}}, {row_id => $user->{id}, name => $user->{username} };
     }
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/list_users', $request);
 }
 
@@ -346,6 +379,9 @@ Copies db to the name of $request->{new_name}
 
 sub copy_db {
     my ($request) = @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _get_database($request);
     return $reauth if $reauth;
 
@@ -364,6 +400,9 @@ Backs up a full db
 
 sub backup_db {
     my $request = shift @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     $request->{backup} = 'db';
     return _begin_backup($request);
 }
@@ -376,6 +415,9 @@ Backs up roles only (for all db's)
 
 sub backup_roles {
     my $request = shift @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     $request->{backup} = 'roles';
     return _begin_backup($request);
 }
@@ -383,8 +425,12 @@ sub backup_roles {
 # Private method, basically just passes the inputs on to the next screen.
 sub _begin_backup {
     my $request = shift @_;
-    $request->{can_email} = defined $LedgerSMB::Sysconfig::backup_email_from;
-    my $template = LedgerSMB::Template::UI->new_UI;
+    $request->{can_email} = eval {
+        # when accessing an undefined service, an exception is thrown;
+        # suppress the exception: all we want to know is if there is a value
+        $request->{_wire}->get( 'miscellaneous/backup_email_from' );
+    };
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/begin_backup', $request);
 };
 
@@ -397,6 +443,9 @@ Runs the backup.  If backup_type is set to email, emails the
 
 sub run_backup {
     my $request = shift @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _get_database($request);
     return $reauth if $reauth;
 
@@ -424,10 +473,12 @@ sub run_backup {
     if ($request->{backup_type} eq 'email') {
 
         my $mail = LedgerSMB::Mailer->new(
-            from     => $LedgerSMB::Sysconfig::backup_email_from,
-            to       => $request->{email},
-            subject  => 'Email of Backup',
-            message  => 'The Backup is Attached',
+            transport => $request->{_wire}->get( 'mail' )->{transport},
+            from      => $request->{_wire}->get( 'miscellaneous/backup_email_from' ),
+
+            to        => $request->{email},
+            subject   => 'Email of Backup',
+            message   => 'The Backup is Attached',
         );
         $mail->attach(
             mimetype => $mimetype,
@@ -436,7 +487,7 @@ sub run_backup {
         );
         $mail->send;
         unlink $backupfile;
-        my $template = LedgerSMB::Template::UI->new_UI;
+        my $template = $request->{_wire}->get('ui');
         return $template->render($request, 'setup/complete', $request);
     }
     elsif ($request->{backup_type} eq 'browser') {
@@ -467,12 +518,40 @@ sub run_backup {
     }
 }
 
+=item consistency
+
+=cut
+
+sub consistency {
+    my ($request) = @_;
+    my ($reauth, $database) = _get_database($request);
+    return $reauth if $reauth;
+
+    my $dbh = $database->connect({PrintError => 0, AutoCommit => 0});
+    my $paths = find_checks($request->{_wire}->get( 'paths/sql' ) );
+    my $checks = load_checks( $paths );
+    my $results = run_checks( $dbh, $checks );
+
+    return $request->{_wire}->get('ui')->render(
+        $request,
+        'setup/consistency_results',
+        {
+            database => $request->{database},
+            login    => $request->{login},
+            results  => $results
+        });
+}
+
+
 =item revert_migration
 
 =cut
 
 sub revert_migration {
     my ($request) = @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _get_database($request);
     return $reauth if $reauth;
 
@@ -489,7 +568,7 @@ sub revert_migration {
     $dbh->do("ALTER SCHEMA $src_schema RENAME TO public");
     $dbh->commit();
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
 
     return $template->render($request, 'setup/complete_migration_revert',
                              $request);
@@ -506,8 +585,10 @@ sub template_screen {
     my ($request, $entrypoint) = @_;
     $request->{template_dirs} =
         [ map { +{ text => $_, value => $_ } }
-          keys %{ LedgerSMB::Database::Config->new->templates } ];
-    return LedgerSMB::Template::UI->new_UI
+          sort keys %{ LedgerSMB::Database::Config->new(
+                           templates_dir => $request->{_wire}->get( 'paths/templates' ),
+                           )->templates } ];
+    return $request->{_wire}->get('ui')
         ->render($request, 'setup/template_info',
                  { %$request, templates_action => $entrypoint });
 }
@@ -522,7 +603,9 @@ and not the user creation screen.
 
 sub _save_templates {
     my ($request, $entrypoint) = @_;
-    my $templates = LedgerSMB::Database::Config->new->templates;
+    my $templates = LedgerSMB::Database::Config->new(
+        templates_dir => $request->{_wire}->get( 'paths/templates' ),
+        )->templates;
 
     return template_screen($request, $entrypoint)
         if not exists $templates->{$request->{template_dir}};
@@ -543,6 +626,9 @@ sub _save_templates {
 sub load_templates {
     my ($request) = @_;
 
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     return (_save_templates($request, 'load_templates')
             or login($request));
 }
@@ -617,7 +703,7 @@ sub _dispatch_upgrade_workflow {
         return __PACKAGE__->can($next)->($request);
     }
 
-    die "Upgrade workflow error: no next step for '$step_name'";
+    croak "Upgrade workflow error: no next step for '$step_name'";
 }
 
 sub _select_coa {
@@ -634,16 +720,64 @@ sub _process_and_run_upgrade_script {
     my ($reauth, $database) = _init_db($request);
     return $reauth if $reauth;
 
+    my $hdr = $request->{_req}->header( 'Accept-Language' );
+    my $lang = $request->{_wire}->get( 'default_locale' )->from_header( $hdr );
+
     my $upgrade = LedgerSMB::Database::Upgrade->new(
         database => $database,
         type     => $type,
+        language => $lang
         );
-    $upgrade->run_upgrade_script(
-        {
-            %{$request}{qw( default_country default_ap default_ar
-                            slschema lsmbschema )}
-        });
-    $upgrade->run_post_upgrade_steps;
+    try {
+        my $info = $database->get_info();
+        $upgrade->run_upgrade_script(
+            {
+                sl_version => version->parse($info->{full_version}),
+                %{$request}{qw( default_country default_ap default_ar
+                                slschema lsmbschema lsmbversion)}
+            });
+        $upgrade->run_post_upgrade_steps;
+    }
+    catch ($e) {
+        my $error_text = escape_html( $e );
+        local $/ = undef;
+        my $stdout = '';
+        if ( open( my $out, '<:encoding(UTF-8)', $upgrade->logfiles->{out} ) ) {
+            $stdout = escape_html( <$out> );
+        }
+        else {
+            $logger->warn(
+                "Unable to open psql upgrade script STDOUT logfile: $!"
+                );
+        }
+
+        my $stderr = '';
+        if ( open( my $err, '<:encoding(UTF-8)', $upgrade->logfiles->{err} ) ) {
+            $stderr = escape_html( <$err> );
+        }
+        else {
+            $logger->warn(
+                "Unable to open psql upgrade script STDERR logfile: $!"
+                );
+        }
+
+        return [ HTTP_INTERNAL_SERVER_ERROR,
+                 [ 'Content-Type' => 'text/html; charset=UTF-8' ],
+                 [ <<~EMBEDDED_HTML ] ];
+        <html>
+          <body>
+            <h1>Error!</h1>
+            <p><b>$error_text</b></p>
+
+            <h3>STDERR</h3>
+            <pre style="max-height:30em;overflow:scroll">$stderr</pre>
+
+            <h3>STDOUT</h3>
+            <pre style="max-height:30em;overflow:scroll">$stdout</pre>
+          </body>
+        </html>
+        EMBEDDED_HTML
+    };
 
     return;
 }
@@ -748,21 +882,30 @@ sub _load_templates {
 
 sub upgrade {
     my ($request) = @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _init_db($request);
     return $reauth if $reauth;
 
-    my $dbinfo = $database->get_info();
+    my $settings = $request->{_wire}->get( 'setup_settings' );
+    my $auth_db = ($settings and $settings->{auth_db}) // 'postgres';
+    my $dbinfo = $database->get_info($auth_db);
     my $upgrade_type = "$dbinfo->{appname}/$dbinfo->{version}";
     my $locale = $request->{_locale};
 
     $request->{dbh}->{AutoCommit} = 0;
 
+    my $hdr = $request->{_req}->header( 'Accept-Language' );
+    my $lang = $request->{_wire}->get( 'default_locale' )->from_header( $hdr );
     my $upgrade = LedgerSMB::Database::Upgrade->new(
         database => $database,
-        type => $upgrade_type,
+        type     => $upgrade_type,
+        language => $lang
         );
 
     my $rv;
+    $logger->debug( "Running upgrade tests for '$upgrade_type'" );
     $upgrade->run_tests(
         sub {
             my ($check, $dbh, $sth) = @_;
@@ -779,7 +922,7 @@ sub upgrade {
                                           $upgrade_run_step{$upgrade_type});
     }
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
 
     my $step = $upgrade_run_step{$upgrade_type};
     my $upgrade_action = $upgrade_next_steps{$step};
@@ -790,8 +933,10 @@ sub upgrade {
 
     for my $key (keys %$required_vars) {
         my $val = $required_vars->{$key};
-        $request->{$key} = (@$val > 1) ? [ {}, @$val ]
-            : ($val->[0] ? $val->[0]->{value} : 'null');
+        $request->{$key} = (ref($val) eq 'ARRAY')
+            ? ((@$val > 1) ? [ {}, @$val ]
+                          : ($val->[0] ? $val->[0]->{value} : 'null'))
+            : $val;
     }
     $request->{lsmbversion} = $CURRENT_MINOR_VERSION;
     return $template->render($request, 'setup/upgrade_info', $request);
@@ -870,7 +1015,7 @@ verify_check => md5_hex($check->test_query),
         if ( $buttons{$_->{label}} && $_->{cond}) {
             push @$enabled_buttons, {
                  type => 'submit',
-                 name => 'action',
+                 name => '__action',
                 value => $_->{value},
               tooltip => { id => 'action-' . $_->{value},
                            msg => $check->{tooltips}->{$_->{label}}
@@ -884,7 +1029,7 @@ verify_check => md5_hex($check->test_query),
         }
     }
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/migration_step', {
            form               => $request,
            headers            => [
@@ -895,7 +1040,7 @@ verify_check => md5_hex($check->test_query),
            rows               => $rows,
            buttons            => $enabled_buttons,
            hiddens            => $hiddens,
-           include_stylesheet => 'setup.css',
+           include_stylesheet => 'system/setup.css',
     });
 }
 
@@ -909,16 +1054,25 @@ script.
 sub fix_tests {
     my ($request) = @_;
 
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _init_db($request);
     return $reauth if $reauth;
 
-    my $dbinfo = $database->get_info();
+    my $settings = $request->{_wire}->get( 'setup_settings' );
+    my $auth_db = ($settings and $settings->{auth_db}) // 'postgres';
+    my $dbinfo = $database->get_info($auth_db);
+    my $upgrade_type = "$dbinfo->{appname}/$dbinfo->{version}";
     my $dbh = $request->{dbh};
     $dbh->{AutoCommit} = 0;
 
+    my $hdr = $request->{_req}->header( 'Accept-Language' );
+    my $lang = $request->{_wire}->get( 'default_locale' )->from_header( $hdr );
     my $upgrade = LedgerSMB::Database::Upgrade->new(
         database => $database,
-        type => '.../...',
+        type     => $upgrade_type,
+        language => $lang
         );
 
     my $check = $upgrade->applicable_test_by_name($request->{check});
@@ -926,7 +1080,7 @@ sub fix_tests {
         . 'found no applicable tests for given identifier'
         unless $check;
 
-    die "Inconsistent state fixing date for $request->{check}: "
+    die "Inconsistent state fixing data for $request->{check}: "
         . 'found different test by the same name while fixing data'
         if $request->{verify_check} ne md5_hex($check->test_query);
 
@@ -935,6 +1089,7 @@ sub fix_tests {
         my %row_data;
         for my $key (@{$check->columns}) {
             $row_data{$key} = $request->{"${key}_$count"};
+            $logger->trace( "Setting row $count field $key to $row_data{$key}" );
         }
         @row_data{@{$check->id_columns}} =
             map {
@@ -957,10 +1112,15 @@ sub fix_tests {
 sub create_db {
     my ($request) = @_;
 
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _get_database($request);
     return $reauth if $reauth;
 
-    my $version_info = $database->get_info;
+    my $settings = $request->{_wire}->get( 'setup_settings' );
+    my $auth_db = ($settings and $settings->{auth_db}) // 'postgres';
+    my $version_info = $database->get_info($auth_db);
     $request->{login_name} = $version_info->{username};
     if ($version_info->{status} ne 'does not exist') {
         $request->{message} = $request->{_locale}->text(
@@ -969,7 +1129,7 @@ sub create_db {
             $request->{_locale}->text('Login?');
         $request->{next_action} = 'login';
 
-        my $template = LedgerSMB::Template::UI->new_UI;
+        my $template = $request->{_wire}->get('ui');
         return $template->render($request, 'setup/confirm_operation', $request);
     }
 
@@ -993,7 +1153,11 @@ coa_lc not set:  Select the coa location code
 
 sub select_coa {
     my ($request) = @_;
-    my $coa_data = LedgerSMB::Database::Config->new->charts_of_accounts;
+    my $hdr = $request->{_req}->header( 'Accept-Language' );
+    my $lang = $request->{_wire}->get( 'default_locale' )->from_header( $hdr );
+    my $coa_data = LedgerSMB::Database::Config
+        ->new( language =>  $lang )
+        ->charts_of_accounts;
 
     if ($request->{coa_lc}) {
         my $coa_lc = $request->{coa_lc};
@@ -1001,7 +1165,7 @@ sub select_coa {
             die $request->{_locale}->text('Invalid request');
         }
 
-        for my $coa_type (qw( chart gifi sic )) {
+        for my $coa_type (qw( chart sic )) {
             if ($request->{$coa_type}) {
                 if (! grep { $_ eq $request->{$coa_type} }
                     @{$coa_data->{$coa_lc}->{$coa_type}}) {
@@ -1009,25 +1173,37 @@ sub select_coa {
                 }
             }
         }
-    }
 
-    if ($request->{coa_lc}){
-        if ($request->{chart}){
+        if ($request->{chart}) {
+            if (my $csrf = $request->verify_csrf) {
+                return $csrf;
+            }
             my ($reauth, $database) = _get_database($request);
             return $reauth if $reauth;
 
-            $database->load_coa(
+            my $c = LedgerSMB::Company->new(
+                dbh => $database->connect(),
+                )->configuration;
+            my $fn = File::Spec->catdir('.', 'locale', 'coa',
+                                        $request->{coa_lc}, $request->{chart});
+            open my $fh, '<:encoding(UTF-8)', $fn
+                or die "Failed to open $fn: $!";
+            $c->from_xml($fh);
+            $c->dbh->commit;
+            $c->dbh->disconnect;
+            close $fh
+                or warn "Error closing $fn: $!";
+
+            $database->load_sic(
                 {
                     country => $request->{coa_lc},
-                    chart => $request->{chart},
-                    gifi => $request->{gifi},
                     sic => $request->{sic}
                 });
 
             # successful completion returns 'undef'
             return _dispatch_upgrade_workflow($request, '_select_coa');
         } else {
-            for my $select (qw(chart gifi sic)) {
+            for my $select (qw( chart sic )) {
                 $request->{"${select}s"} =
                     [ map { +{ name => $_ } }
                       @{$coa_data->{$request->{coa_lc}}->{$select}} ];
@@ -1038,7 +1214,7 @@ sub select_coa {
                                 values %$coa_data ];
     }
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/select_coa', $request);
 }
 
@@ -1075,9 +1251,7 @@ sub _render_user {
         funcname => 'person__list_salutations'
     );
 
-    @{$request->{countries}} = $request->call_procedure(
-        funcname => 'location_list_country'
-    );
+    $request->{countries} = $request->enabled_countries;
     my $locale = $request->{_locale};
 
     @{$request->{perm_sets}} = (
@@ -1086,7 +1260,7 @@ sub _render_user {
         {id => '-1', label => $locale->text('No changes')},
         );
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/new_user',
                              { %$request, save_action => $entrypoint });
 }
@@ -1144,30 +1318,33 @@ sub _save_user {
     $request->{dbh}->{AutoCommit} = 0;
 
     $request->{control_code} = $request->{employeenumber};
-    $request->{dob} = LedgerSMB::PGDate->from_input($request->{dob});
-    my $emp = LedgerSMB::Entity::Person::Employee->new(%$request);
+    local $LedgerSMB::App_State::User = {};
+    my $emp = LedgerSMB::Entity::Person::Employee->new(
+        %$request,
+        dob => LedgerSMB::PGDate->from_input($request->{dob})
+        );
     $emp->save;
     $request->{entity_id} = $emp->entity_id;
     my $user = LedgerSMB::Entity::User->new(%$request);
 
-    my $rerendered_user =
-        try { $user->create($request->{password}); 0 }
-    catch {
-        if ($_ =~ /duplicate user/i){
-           $request->{dbh}->rollback;
-           $request->{notice} = $request->{_locale}->text(
-                       'User already exists. Import?'
-            );
-           $request->{pls_import} = 1;
+    try {
+        $user->create($request->{password});
+    }
+    catch ($var) {
+        if ($var =~ /duplicate user/i){
+            $request->{dbh}->rollback;
+            $request->{notice} = $request->{_locale}->text(
+                'User already exists. Import?'
+                );
+            $request->{pls_import} = 1;
 
-           # return from the 'catch' block
-           return _render_user($request, $entrypoint);
-       } else {
-           die $_;
-       }
+            # return from the 'catch' block
+            return _render_user($request, $entrypoint);
+        }
+        else {
+            die $var;
+        }
     };
-    return $rerendered_user if $rerendered_user;
-
 
     if ($request->{perms} == 1){
          for my $role (
@@ -1228,7 +1405,6 @@ sub _post_migration_schema_upgrade {
                  FROM users WHERE username IN (select rolname from pg_roles)});
 
     $dbh->commit;
-    $dbh->disconnect;
 
     return;
 }
@@ -1248,6 +1424,9 @@ sub _create_initial_user {
 
 sub add_user {
     my ($request) = @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
 
     return (_create_initial_user($request)
             or login($request));
@@ -1265,32 +1444,36 @@ sub edit_user_roles {
         unless $request->{dbh};
     return $reauth if $reauth;
 
-    my $admin = LedgerSMB::DBObject::Admin->new();
-    $admin->set_dbh($request->{dbh});
-    my $all_roles = $admin->get_roles($request->{database});
-
-    my $user_obj = LedgerSMB::DBObject::User->new();
-    $user_obj->set_dbh($request->{dbh});
-    $user_obj->get($request->{id});
-
-    # LedgerSMB::DBObject::User doesn't retrieve the username
-    # field from the users table (nor any of the other values from it,
-    # really) and there's no stored procedure to do so.
-    # The name 'admin__get_user' is already taken, but takes the entity_id
-    # as its argument... So, we're going brute force here, for 1.4
-    my @user_rec = grep { $_->{id} == $request->{id} }
-          @{$user_obj->get_all_users};
-
-    $user_obj->{username} = $user_rec[0]->{username};
-
-    my $template = LedgerSMB::Template::UI->new_UI;
-    my $template_data = {
-                        request => $request,
-                           user => $user_obj,
-                          roles => $all_roles,
-            };
-
-    return $template->render($request, 'setup/edit_user', $template_data);
+    local $LedgerSMB::App_State::User = {};
+    my $template = $request->{_wire}->get('ui');
+    my ($user) = $request->call_procedure(
+        funcname => 'admin__get_user',
+        args     => [ $request->{id} ]);
+    return $template->render(
+        $request,
+        'setup/edit_user',
+        {
+            request => $request,
+            roles   => [
+                map {
+                    +{
+                        name => $_->{rolname},
+                        description => ($_->{rolname} =~ s/_/ /gr),
+                    }
+                }
+                $request->call_procedure(funcname => 'admin__get_roles')
+                ],
+            user    => {
+                roles    => [
+                    map { $_->{admin__get_roles_for_user} }
+                    $request->call_procedure(
+                        funcname => 'admin__get_roles_for_user',
+                        args     => [ $request->{id} ])
+                    ],
+                user_id  => $request->{id},
+                username => $user->{username},
+            }
+        });
 }
 
 =item save_user_roles
@@ -1300,16 +1483,37 @@ sub edit_user_roles {
 sub save_user_roles {
     my ($request) = @_;
 
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth) = _init_db($request);
     return $reauth if $reauth;
 
-    $request->{user_id} = $request->{id};
-    my $admin = LedgerSMB::DBObject::Admin->new(%$request);
-    my $roles = [];
-    for my $r (grep { $_ =~ m/lsmb_(.+)__/ } keys %$request) {
-        push @$roles, $r;
+    my ($user) = $request->call_procedure(
+        funcname => 'admin__get_user',
+        args => [ $request->{id} ] );
+    my %active_roles = map {
+        $_->{admin__get_roles_for_user} => 1
+    } $request->call_procedure(
+        funcname => 'admin__get_roles_for_user',
+        args     => [ $request->{id} ]);
+
+    for my $role (
+        map { $_->{rolname} }
+        $request->call_procedure( funcname => 'admin__get_roles' ) ) {
+        if ($active_roles{$role} and not $request->{$role}) {
+            # remove
+            $request->call_procedure(
+                funcname => 'admin__remove_user_from_role',
+                args     => [ $user->{username}, $role ] );
+        }
+        elsif ($request->{$role} and not $active_roles{$role}) {
+            # add
+            $request->call_procedure(
+                funcname => 'admin__add_user_to_role',
+                args     => [ $user->{username}, $role ] );
+        }
     }
-    $admin->save_roles($roles);
 
     return edit_user_roles($request);
 }
@@ -1322,6 +1526,9 @@ sub save_user_roles {
 sub reset_password {
     my ($request) = @_;
 
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth) = _init_db($request);
     return $reauth if $reauth;
 
@@ -1352,12 +1559,18 @@ Force work.  Forgets unmatching tests, applies a curing statement and move on.
 
 sub force {
     my ($request) = @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _init_db($request);
     return $reauth if $reauth;
 
+    my $hdr = $request->{_req}->header( 'Accept-Language' );
+    my $lang = $request->{_wire}->get( 'default_locale' )->from_header( $hdr );
     my $upgrade = LedgerSMB::Database::Upgrade->new(
         database => $database,
         type => '.../...',
+        language => $lang
         );
 
     my $test = $upgrade->applicable_test_by_name($request->{check});
@@ -1408,6 +1621,9 @@ sub _rebuild_modules {
 sub rebuild_modules {
     my ($request) = @_;
 
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     if (my $rv = _rebuild_modules($request, 'rebuild_modules')) {
         return $rv;
     }
@@ -1422,12 +1638,15 @@ Gets the statistics info and shows the complete screen.
 
 sub _complete {
     my ($request) = @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _init_db($request);
     return $reauth if $reauth;
 
     my $temp = $database->loader_log_filename();
     $request->{lsmb_info} = $database->stats();
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $request->{_wire}->get('ui');
     return $template->render($request, 'setup/complete', $request);
 }
 
@@ -1441,19 +1660,24 @@ Asks the various modules for system and version info, showing the result
 
 sub system_info {
     my ($request) = @_;
+    if (my $csrf = $request->verify_csrf) {
+        return $csrf;
+    }
     my ($reauth, $database) = _init_db($request);
     return $reauth if $reauth;
 
     # the intent here is to get a much more sophisticated system which
     # asks registered modules for their system and dependency info
+    my $settings = $request->{_wire}->get( 'setup_settings' );
+    my $auth_db = ($settings and $settings->{auth_db}) // 'postgres';
     my $info = {
-        db => $database->get_info->{system_info},
+        db     => $database->get_info($auth_db)->{system_info},
         system => LedgerSMB::system_info()->{system},
         environment => \%ENV,
         modules => \%INC,
     };
     $request->{info} = $info;
-    return LedgerSMB::Template::UI->new_UI
+    return $request->{_wire}->get('ui')
         ->render($request, 'setup/system_info', $request);
 }
 
@@ -1462,7 +1686,7 @@ sub system_info {
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2011-2018 The LedgerSMB Core Team
+Copyright (C) 2011-2024 The LedgerSMB Core Team
 
 This file is licensed under the GNU General Public License version 2, or at your
 option any later version.  A copy of the license should have been included with
