@@ -18,6 +18,8 @@ This module has a single function that returns upgrade tests.
 use strict;
 use warnings;
 
+use JSON::PP;
+use Log::Any qw($log);
 use Moose;
 use Moose::Util::TypeConstraints;
 use namespace::autoclean;
@@ -25,6 +27,8 @@ use List::Util qw( first );
 
 use LedgerSMB::Locale qw(marktext);
 
+my $json = JSON::PP->new;
+
 =head1 FUNCTIONS
 
 =over
@@ -224,18 +228,6 @@ has tooltips => (is => 'ro',
     }
 );
 
-=item skipable
-
-Can this test be skipped
-
-=cut
-
-has skipable => (is =>'ro', isa => 'Maybe[Bool]', lazy => 1,
-                 default =>  sub {
-                    return grep(/^Skip$/, @{$_[0]->{buttons}}) == 1;
-                 }
-);
-
 =back
 
 =head1 Methods
@@ -286,11 +278,11 @@ sub fix {
     if ($self->insert) {
         my $columns =
             join ', ',
-            map { $dbh->quote_identifier($_) } @{$self->columns};
+            map { $dbh->quote_identifier($_) } (@{$self->columns}, @{$self->id_columns});
         my $values =
-            join ', ', map { '?' } @{$self->columns};
+            join ', ', map { '?' } (@{$self->columns}, @{$self->id_columns});
         $query = qq{INSERT INTO $table ($columns) VALUES ($values)};
-        @bind_columns = @{$self->columns};
+        @bind_columns = (@{$self->columns}, @{$self->id_columns});
     }
     else {
         my $setters =
@@ -306,6 +298,7 @@ sub fix {
 
     my $sth = $dbh->prepare($query)
         or die "Failed to compile query ($query) to apply fixes: " . $dbh->errstr;
+    $log->debug( "Running fix-query: $query" . $json->encode( { columns => \@bind_columns, values => $fixes } ) );
     for my $row (@$fixes) {
         my $rv = $sth->execute(map { $row->{$_} } @bind_columns);
         if (not $rv) {
@@ -552,7 +545,7 @@ source database. Please either find professional help to migrate your
 database, or delete the offending rows through PgAdmin III or psql'),
       appname => 'ledgersmb',
   min_version => '1.2',
-  max_version => '1.4'
+  max_version => '1.3'
 );
 
 push @tests, __PACKAGE__->new(
@@ -560,8 +553,10 @@ push @tests, __PACKAGE__->new(
                      from entity_credit_account eca
                      join entity_class ec on eca.entity_class = ec.id
                      join entity e on eca.entity_id = e.id
-                   where meta_number in
-                       (select meta_number from entity_credit_account
+                   where exists
+                       (select meta_number from entity_credit_account eca2
+                         where eca.meta_number = eca2.meta_number
+                               and eca.entity_class = eca2.entity_class
                         group by meta_number having count(*) > 1)
                    order by meta_number',
  display_name => marktext('No duplicate meta_numbers'),
@@ -572,7 +567,7 @@ push @tests, __PACKAGE__->new(
  instructions => marktext('Make sure all meta numbers are unique.'),
       appname => 'ledgersmb',
   min_version => '1.3',
-  max_version => '1.4'
+  max_version => '1.3'
 );
 
 push @tests, __PACKAGE__->new(
@@ -648,6 +643,7 @@ push @tests, __PACKAGE__->new(
          name => 'missing_gifi_table_rows',
  display_cols => [ 'accno', 'description' ],
         table => 'gifi',
+       insert => 1,
       columns => ['description'],
    id_columns => ['accno'],
  instructions => marktext('Please add the missing GIFI accounts'),
@@ -667,16 +663,16 @@ push @tests, __PACKAGE__->new(
          name => 'missing_gifi_table_rows',
  display_cols => [ 'gifi_accno' ],
         table => 'account',
- instructions => marktext('Please use the 1.3/1.4 UI to add the GIFI accounts'),
+ instructions => marktext('Please use the 1.3 UI to add the GIFI accounts'),
       appname => 'ledgersmb',
   min_version => '1.3',
-  max_version => '1.4'
+  max_version => '1.3'
 );
 
 push @tests, __PACKAGE__->new(
    test_query => q{select chart_id, account
-                   from from cr_coa_to_account ccta
-                   where chart_id in (select crcoa.chart_id
+                     from cr_coa_to_account ccta
+                    where chart_id in (select crcoa.chart_id
                                         from cr_coa_to_account crcoa
                                        where ccta.chart_id = crcoa.chart_id
                                     group by crcoa.chart_id
@@ -688,13 +684,13 @@ push @tests, __PACKAGE__->new(
  instructions => marktext('Please use pgAdmin3 or psql to remove the duplicates'),
       appname => 'ledgersmb',
   min_version => '1.3',
-  max_version => '1.4'
+  max_version => '1.3'
 );
 
 push @tests, __PACKAGE__->new(
    test_query => q{select chart_id, account
-                   from from cr_coa_to_account ccta
-                   where not exists (select 1
+                     from cr_coa_to_account ccta
+                    where not exists (select 1
                                        from account
                                       where account.id = ccta.chart_id)},
  display_name => marktext('Accounts marked for recon exist'),
@@ -704,40 +700,49 @@ push @tests, __PACKAGE__->new(
  instructions => marktext(q(Please use pgAdmin3 or psql to look up the 'chart_id' value in the 'account' table and change it to an existing value)),
       appname => 'ledgersmb',
   min_version => '1.3',
-  max_version => '1.4'
+  max_version => '1.3'
 );
 
+push @tests, __PACKAGE__->new(
+   test_query => q{select id, name, contact from customer
+                   where arap_accno_id is null
+                   order by name},
+ instructions => marktext(
+                   'Select and assign the missing AR accounts'),
+selectable_values => {
+    arap_accno_id => q/SELECT description as text, id as value FROM chart WHERE (':' || link || ':') like '%:AR:%'/,
+    },
+ display_name => marktext('Empty AR account'),
+         name => 'no_null_ar_accounts',
+ display_cols => [ 'name', 'contact', 'arap_accno_id' ],
+      columns => [ 'arap_accno_id' ],
+      appname => 'sql-ledger',
+        table => 'customer',
+  min_version => '2.7',
+  max_version => '3.0'
+   );
 
-#=pod
-
-#  push @tests, __PACKAGE__->new(
-#     test_query => "select * from customer where arap_accno_id is null",
-#   display_name => marktext('Empty AR account'),
-#           name => 'no_null_ar_accounts',
-#   display_cols => [ 'name', 'contact' ],
-#   instructions => marktext("Please correct the empty AR accounts"),
-#        appname => 'sql-ledger',
-#    min_version => '2.7',
-#    max_version => '3.0'
-#     );
-
-#  push @tests, __PACKAGE__->new(
-#     test_query => "select * from vendor where arap_accno_id is null",
-#   display_name => marktext('Empty AP account'),
-#           name => 'no_null_ap_accounts',
-#   display_cols => [ 'name', 'contact' ],
-#   instructions => marktext("Please correct the empty AP accounts"),
-#        appname => 'sql-ledger',
-#    min_version => '2.7',
-#    max_version => '3.0'
-#     );
-#*/
-
-#=cut
-
+push @tests, __PACKAGE__->new(
+   test_query => q{select id, name, contact from vendor
+                   where arap_accno_id is null
+                   order by name},
+ instructions => marktext(
+                   'Select and assign the missing AP accounts'),
+selectable_values => {
+    arap_accno_id => q/SELECT description as text, id as value FROM chart WHERE (':' || link || ':') like '%:AP:%'/,
+    },
+ display_name => marktext('Empty AP account'),
+         name => 'no_null_ap_accounts',
+ display_cols => [ 'name', 'contact', 'arap_accno_id' ],
+      columns => [ 'arap_accno_id' ],
+      appname => 'sql-ledger',
+        table => 'vendor',
+  min_version => '2.7',
+  max_version => '3.0'
+   );
 
 push @tests,__PACKAGE__->new(
-    test_query => q{select category
+    test_query => q{ select category, accno, description
                     from chart
                    where charttype = 'A'
                      and category not in ('A', 'L', 'Q', 'I', 'E')},
@@ -754,6 +759,7 @@ push @tests,__PACKAGE__->new(
     max_version => '3.0'
     );
 
+
 push @tests,__PACKAGE__->new(
     test_query => q{select accno, description, link
                     from chart
@@ -894,109 +900,6 @@ selectable_values => { business_id => q{SELECT concat(description,' -- ',discoun
     max_version => '3.0'
     );
 
-push @tests, __PACKAGE__->new(
-    test_query => 'SELECT id, name, business_id
-                     FROM vendor
-                    WHERE business_id NOT IN (SELECT id
-                     FROM business)
-                      AND business_id <> 0
-                 ORDER BY name',
-    display_name => marktext('Vendor not in a business'),
-    name => 'no_business_for_vendor',
-    display_cols => ['name', 'business_id'],
-    columns => ['business_id'],
- instructions => marktext(
-                   'LedgerSMB vendors must be assigned to a valid business.<br>
-Please review the selection or select the proper business from the list'),
-selectable_values => { business_id => q{SELECT concat(description,' -- ',discount) AS text, id as value
-                                        FROM business
-                                        ORDER BY id}},
-    table => 'vendor',
-    appname => 'sql-ledger',
-    min_version => '2.7',
-    max_version => '3.0'
-    );
-
-push @tests, __PACKAGE__->new(
-    test_query => 'SELECT id, name, business_id
-                     FROM customer
-                    WHERE business_id NOT IN (SELECT id
-                                              FROM business)
-                      AND business_id <> 0
-                 ORDER BY name',
-    display_name => marktext('Customer not in a business'),
-    name => 'no_business_for_customer',
-    display_cols => ['name', 'business_id'],
-    columns => ['business_id'],
- instructions => marktext(
-                   'LedgerSMB customers must be assigned to a valid business.<br>
-Please review the selection or select the proper business from the list'),
-selectable_values => { business_id => q{SELECT concat(description,' -- ',discount) AS text, id as value
-                                        FROM business
-                                        ORDER BY id}},
-    table => 'customer',
-    appname => 'sql-ledger',
-    min_version => '2.7',
-    max_version => '3.0'
-    );
-
-push @tests,__PACKAGE__->new(
-    test_query => q{ select category, accno, description
-                    from chart
-                   where charttype = 'A'
-                     and category not in ('A', 'L', 'Q', 'I', 'E')},
-    display_name => marktext('Unsupported account categories'),
-    name => 'unsupported_account_types',
-    display_cols => ['category', 'accno', 'description'],
- instructions => marktext(
-                   'Please make sure all accounts have a category of
-(A)sset, (L)iability, e(Q)uity, (I)ncome or (E)xpense.'),
-   columns => ['category'],
-    table => 'chart',
-    appname => 'sql-ledger',
-    min_version => '2.7',
-    max_version => '3.0'
-    );
-
-# push @tests,__PACKAGE__->new(
-#     test_query => "select accno, description, link
-#                     from chart
-#                    where charttype = 'A'
-#                      and link ~ ':?\\(AR|AP|IC\\)\\(:|$\\)'",
-#     display_name => marktext('Unsupported account link combinations'),
-#     name => 'unsupported_account_links',
-#     display_cols => ['accno', 'description', 'link'],
-#  instructions =>
-#          marktext( 'An account can either be a summary account (which have a
-# link of "AR", "AP" or "IC" value) or be linked to dropdowns (having any
-# number of "AR_*", "AP_*" and/or "IC_*" links concatenated by colons (:).'),
-#    columns => ['category'],
-#     table => 'chart',
-#     appname => 'sql-ledger',
-#     min_version => '2.7',
-#     max_version => '3.0'
-#     );
-
-push @tests,__PACKAGE__->new(
-    test_query => q{select accno, description, link
-                    from chart c
-                   where charttype = 'A'
-                     and 0 = (select count(*)
-                            from chart cn
-                           where cn.charttype = 'H'
-                             and cn.accno < c.accno)},
-    display_name => marktext('Accounts without heading'),
-    name => 'no_header_accounts',
-    display_cols => ['accno', 'description', 'link'],
- instructions => marktext(
-                   'Please go into the SQL-Ledger UI and create/rename a
-heading which sorts alphanumerically before the first account by accno'),
-    table => 'chart',
-    appname => 'sql-ledger',
-    min_version => '2.7',
-    max_version => '3.0'
-    );
-
 push @tests,__PACKAGE__->new(
     test_query => 'select id, customernumber, name
                     from customer
@@ -1133,7 +1036,7 @@ push @tests, __PACKAGE__->new(
     );
 
 push @tests, __PACKAGE__->new(
-   test_query => 'select partnumber, description, sellprice
+   test_query => 'select id, partnumber, description, sellprice
                   from parts where obsolete is not true
                   and partnumber in
                   (select partnumber from parts
@@ -1337,25 +1240,6 @@ Void the clearing date in the dialog shown or go back to SQL-Ledger if you feel
        max_version => '3.0'
 );
 
-
-### On the vendor side, SL doesn't use pricegroups
-# push @tests, __PACKAGE__->new(
-#     test_query => "select *
-#                      from partsvendor
-#                     where not exists (select 1
-#                                         from pricegroup
-#                                        where id = pricegroup_id)",
-#     display_name => marktext('Non-existing vendor pricegroups in partsvendor'),
-#     name => 'partsvendor_pricegroups_exist',
-#     display_cols => ['parts_id', 'credit_id', 'pricegroup_id'],
-#  instructions =>
-#         marktext('Please fix the pricegroup data in your partsvendor table (no UI available)'),
-#     table => 'partsvendor',
-#     appname => 'sql-ledger',
-#     min_version => '2.7',
-#     max_version => '3.0'
-#     );
-
     return @tests;
 }
 
