@@ -48,14 +48,11 @@
 package lsmb_legacy;
 use LedgerSMB::GL;
 use LedgerSMB::PE;
-use LedgerSMB::Template::UI;
 use LedgerSMB::Setting::Sequence;
-use LedgerSMB::Company_Config;
 use LedgerSMB::Legacy_Util;
-use LedgerSMB::DBObject::Draft;
-use LedgerSMB::DBObject::TransTemplate;
+use LedgerSMB::Num2text;
 
-require "old/bin/arap.pl";
+require "old/bin/arap.pl"; # for: Schedule action
 
 # end of main
 
@@ -64,7 +61,7 @@ require "old/bin/arap.pl";
 # $locale->text('February')
 # $locale->text('March')
 # $locale->text('April')
-# $locale->text('May ')
+# $locale->text('May')
 # $locale->text('June')
 # $locale->text('July')
 # $locale->text('August')
@@ -90,15 +87,39 @@ require "old/bin/arap.pl";
 
 sub edit_and_save {
     check_balanced($form);
-    my $draft = LedgerSMB::DBObject::Draft->new(%$form);
-    $draft->delete();
+    $form->call_procedure(funcname=>'draft_delete', args => [ $form->{id} ]);
     GL->post_transaction( \%myconfig, \%$form, $locale);
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
+    edit();
+}
+
+sub save_info {
+    GL->save_notes( \%myconfig, \%$form, $locale);
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
     edit();
 }
 
 sub approve {
-    my $draft = LedgerSMB::DBObject::Draft->new(%$form);
-    $draft->approve();
+    $form->call_procedure(funcname=>'draft_approve', args => [ $form->{id} ]);
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
     if ($form->{callback}){
         print "Location: $form->{callback}\n";
         print "Status: 302 Found\n\n";
@@ -108,36 +129,49 @@ sub approve {
                 . qq|here</a>.</body></html>|;
 
     } else {
-        $form->info($locale->text('Draft Posted'));
+        new();
     }
 }
 
-
 sub new {
-     for my $row (0 .. $form->{rowcount}){
-         for my $fld(qw(accno projectnumber acc debit credit source memo)){
+    for my $row (0 .. $form->{rowcount}){
+        for my $fld (
+            qw(accno projectnumber acc debit credit source memo )
+            ) {
             delete $form->{"${fld}_${row}"};
-         }
-     }
-     delete $form->{description};
-     delete $form->{reference};
-     delete $form->{rowcount};
-     delete $form->{id};
-     add();
+        }
+    }
+    for my $fld (
+        qw(description reference rowcount id workflow_id
+           notes reversing reversing_reference )
+        ) {
+        delete $form->{$fld};
+    }
+    add();
 }
 
 sub copy_to_new {
-     delete $form->{reference};
-     delete $form->{id};
-     delete $form->{approved};
-     update();
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
+    delete $form->{reference};
+    delete $form->{id};
+    delete $form->{approved};
+    delete $form->{workflow_id};
+    update();
 }
 
 sub add {
 
     $form->{title} = "Add";
 
-    $form->{callback} = "$form->{script}?action=add&transfer=$form->{transfer}"
+    my $transfer
+        = ($form->{transfer}) ? "&transfer=$form->{transfer}" : '';
+    $form->{callback} = "$form->{script}?__action=add$transfer"
       unless $form->{callback};
 
     if (!$form->{rowcount}){
@@ -151,10 +185,116 @@ sub add {
 }
 
 
+sub _reverse_amounts {
+    # swap debits and credits
+    for my $rownum (0 .. $form->{rowcount}) {
+        my $credit = $form->{"credit_$rownum"};
+        my $credit_fx = $form->{"credit_fx_$rownum"};
+        $form->{"credit_$rownum"} = $form->{"debit_$rownum"};
+        $form->{"credit_fx_$rownum"} = $form->{"debit_fx_$rownum"};
+        $form->{"debit_$rownum"} = $credit;
+        $form->{"debit_fx_$rownum"} = $credit_fx;
+    }
+}
+
+sub reverse {
+    $form->{title}     = "Reverse";
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
+
+    &create_links; # runs GL->transaction()
+    _reverse_amounts();
+
+    $form->{reversing} = delete $form->{id};
+    $form->{reversing_reference} = $form->{reference};
+    delete $form->{approved};
+    delete $form->{workflow_id};
+
+    display_form();
+}
+
+
+sub post_reversing {
+    # we should save only the reference, sequence, transdate, description and notes.
+
+    $form->error(
+        $locale->text('Cannot post transaction for a closed period!') )
+        if ( $transdate and $form->is_closed( $transdate ) );
+    if (not $form->{id}) {
+        do {
+            local $form->{id} = $form->{reversing};
+
+            # save data we want to use from the posted form,
+            # not from the reversed transaction.
+            local $form->{reversing};
+            local $form->{notes};
+            local $form->{description};
+            local $form->{reference};
+            local $form->{reversing_reference} = $form->{reference};
+            local $form->{approved};
+            local $form->{workflow_id};
+
+            &create_links; # create_links overwrites 'reversing'
+        };
+        my $wf = $form->{_wire}->get('workflows')
+            ->create_workflow( 'GL',
+                               Workflow::Context->new(
+                                   'transdate' => $form->{transdate},
+                                   'batch-id' => $form->{batch_id},
+                                   'table_name' => 'gl',
+                                   'reversing' => $form->{reversing}
+                               ) );
+        $form->{workflow_id} = $wf->id;
+        $wf->execute_action( $form->{__action} );
+
+        # Why do I not need _reverse_amounts here???
+        # _reverse_amounts();
+        GL->post_transaction( \%myconfig, \%$form, $locale);
+        $form->call_procedure( funcname=>'draft_approve',
+                               args => [ $form->{id} ]);
+        $form->{approved} = 1;
+
+        my $query = q{UPDATE transactions SET reversing = ? WHERE id = ?};
+        $form->{dbh}->do(
+            $query,
+            {},
+            $form->{reversing},
+            $form->{id})
+            or $form->dberror($query);
+    }
+    else {
+        my $query = <<~'QUERY';
+        UPDATE gl
+           SET reference = ?,
+               description = ?,
+               transdate = ?,
+               notes = ?
+         WHERE id = ?
+        QUERY
+
+        $form->{dbh}->do(
+            $query,
+            {},
+            $form->{reference},
+            $form->{description},
+            $form->{transdate},
+            $form->{notes},
+
+            $form->{id})
+            or $form->dberror($query);
+    }
+
+    display_form();
+}
+
 sub display_form
 {
-    $form->{separate_duties}
-        = $LedgerSMB::Company_Config::settings->{separate_duties};
+    $form->{separate_duties} = $form->get_setting('separate_duties');
     #Add General Ledger Transaction
 
     # filter out '' transdates
@@ -166,12 +306,9 @@ sub display_form
     $form->open_form;
     my ($init) = @_;
     # Form header part begins -------------------------------------------
-    if (@{$form->{all_department}}){
+    if ($form->{all_department}){
         unshift @{ $form->{all_department} }, {};
     }
-    if (@{$form->{all_project}}){
-       unshift @{ $form->{all_project} }, {};
-    }
     $title = $locale->maketext($form->{title});
     if ( $form->{transfer} ) {
         $form->{title} = $locale->text("[_1] Cash Transfer Transaction", $title);
@@ -180,10 +317,6 @@ sub display_form
         $form->{title} = $locale->text("[_1] General Ledger Transaction", $title);
     }
 
-    for (qw(reference description notes)) {
-        $form->{$_} = $form->quote( $form->{$_} );
-    }
-
     if ( ( $rows = $form->numtextrows( $form->{description}, 50 ) ) > 1 ) {
          $form->{rowsdesc}=$rows; $form->{colsdesc}=50;
          $form->{colrowdesc}=1;
@@ -200,10 +333,23 @@ sub display_form
                $form->{colrownotes}=0;
     }
 
-    if (!defined $form->{approved}){
-        $form->{approved} = '1';
+    my $wf;
+    if($form->{workflow_id}) {
+        $wf = $form->{_wire}->get('workflows')
+            ->fetch_workflow( 'GL', $form->{workflow_id} );
     }
-
+    else {
+        $wf = $form->{_wire}->get('workflows')
+            ->create_workflow( 'GL',
+                               Workflow::Context->new(
+                                   'batch-id' => $form->{batch_id},
+                                   'table_name' => 'gl',
+                                   'reversing' => $form->{reversing}
+                               ) );
+        $form->{workflow_id} = $wf->id;
+    }
+    $wf->context->param( transdate => $form->{transdate} );
+    $form->{status} = $wf->state;
     $focus = ( $form->{focus} ) ? $form->{focus} : "debit_$form->{rowcount}";
     our %hiddens = (
         'direction' => $form->{direction},
@@ -211,8 +357,6 @@ sub display_form
         'batch_id' => $form->{batch_id},
         'id' => $form->{id},
         'transfer' => $form->{transfer},
-        'closedto' => $form->{closedto},
-        'locked' => $form->{locked},
         'oldtransdate' => $form->{oldtransdate},
         'recurring' => $form->{recurring},
         'title' => $title,
@@ -220,6 +364,9 @@ sub display_form
         'callback' => $form->{callback},
         'form_id' => $form->{form_id},
         'separate_duties' => $form->{separate_duties},
+        'reversing' => $form->{reversing},
+        'reversing_reference' => $form->{reversing_reference},
+        'workflow_id' => $form->{workflow_id}
     );
 
 
@@ -230,70 +377,48 @@ sub display_form
 
     #Form footer  Begins------------------------------------------
 
-  for (qw(totaldebit totalcredit)) {
-      $form->{$_} =
-    $form->format_amount( \%myconfig, $form->{$_}, 2, "0" );
-  }
+    $form->{_setting_decimal_places} //= LedgerSMB::Setting->new(%$form)->get('decimal_places');
+    for (qw(totaldebit totalcredit)) {
+        $form->{$_} =
+            $form->format_amount( \%myconfig, $form->{$_}, $form->{_setting_decimal_places}, "0" );
+    }
 
   $transdate = $form->datetonum( \%myconfig, $form->{transdate} );
-  $closedto  = $form->datetonum( \%myconfig, $form->{closedto} );
   my @buttons;
   if ( !$form->{readonly} ) {
-      my $i=1;
-      @buttons = (
-          { action => 'update', key => 'U', value => $locale->text('Update') },
-          { action => 'post', key => 'O', value =>
-                ($form->{separate_duties}
-                 ? $locale->text('Save') : $locale->text('Post')),
-            class => 'post' },
-          { action => 'approve', key => 'S', value => $locale->text('Post'),
-            class => 'post' },
-          { action => 'edit_and_save', key => 'V',
-            value => $locale->text('Save Draft') },
-          { action => 'save_temp', key   => 'T',
-            value => $locale->text('Save Template') },
-          { action => 'save_as_new',
-            key => 'N', value => $locale->text('Save as new') },
-          { action => 'schedule',
-            key => 'H', value => $locale->text('Schedule') },
-          { action => 'new',
-            key => 'N', value => $locale->text('New') },
-          { action => 'copy_to_new',
-            key => 'C', value => $locale->text('Copy to New') },
+      %button_types = (
+          print => 'lsmb/PrintButton'
           );
+      for my $action_name ( $wf->get_current_actions ) {
+          my $action = $wf->get_action( $action_name );
 
-      %a = ();
-      $a{'save_temp'} = 1;
-
-      if ( $form->{id}) {
-          for ( 'new', 'save_as_new', 'schedule', 'copy_to_new' ) {
-              $a{$_} = 1;
-          }
-          if (!$form->{approved} && !$form->{batch_id}) {
-              $a{approve} = 1;
-              $a{edit_and_save} = 1;
-              $a{update} = 1;
-          }
-      } else {
-          $a{'update'} = 1;
-          if ( ! $closedto or ($transdate > $closedto ) ) {
-              for ( 'post', 'schedule' ) { $a{$_} = 1 }
-          }
+          next if ($action->ui // '') eq 'none';
+          push @buttons, {
+              ndx   => $action->order,
+              name  => $action->name,
+              text => $locale->maketext($action->text),
+              doing => ($action->doing ? $locale->maketext($action->doing) : ''),
+              done  => ($action->done ? $locale->maketext($action->done) : ''),
+              type  => $button_types{$action->ui},
+              tooltip => ($action->short_help ? $locale->maketext($action->short_help) : '')
+          };
       }
 
-      my $i=1;
       @buttons = map {
           {
-              name => 'action',
-              value => $_->{action},
-              text => $_->{value},
+              name => '__action',
+              value => $_->{name},
+              text => $_->{text},
               type => 'submit',
               class => $_->{class} // 'submit',
-              accesskey => $_->{key},
-              order => $i++
+              order => $_->{ndx},
+              'data-lsmb-doing' => $_->{doing},
+              'data-lsmb-done' => $_->{done},
+              'data-dojo-type' => $_->{type} // 'dijit/form/Button',
+              'data-dojo-props' => $_->{type} ? 'minimalGET: false' : '',
           }
       }
-      grep { $a{$_->{action}} } @buttons;
+      sort { $a->{ndx} <=> $b->{ndx} } @buttons;
   }
 
   $form->{recurringset}=0;
@@ -301,8 +426,9 @@ sub display_form
       $form->{recurringset}=1;
   }
 
-    my $template = LedgerSMB::Template::UI->new_UI;
+    my $template = $form->{_wire}->get('ui');
     LedgerSMB::Legacy_Util::render_psgi(
+        $form,
         $template->render($form, 'journal/journal_entry',
                           {
                               form => $form,
@@ -316,17 +442,14 @@ sub display_form
 sub save_temp {
     my ($department_name, $department_id) = split/--/, $form->{department};
 
-    my $data = {
-        dbh => $form->{dbh},
-        department_id => $department_id,
-        reference => $form->{reference},
-        description => $form->{description},
-        department_id => $department_id,
-        post_date => $form->{transdate},
-        type => 'gl',
-        journal_lines => [],
-    };
-
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
+    my @lines;
     for my $iter (0 .. $form->{rowcount}){
         if ($form->{"accno_$iter"} and
             (($form->{"credit_$iter"} != 0) or
@@ -338,18 +461,43 @@ sub save_temp {
                  ( $form->{"debit_$iter"} * -1 );
              my $amount_fx = $form->{"credit_fx_$iter"} ||
                  ( $form->{"debit_fx_$iter"} * -1 );
-             push @{$data->{journal_lines}},
-                  {accno => $acc_id,
-                   amount => $amount,
-                   amount_fx => $amount_fx,
-                   curr => $form->{"curr_$iter"},
-                   cleared => false,
-                  };
+             push @lines, {
+                 accno => $acc_id,
+                 amount => $form->parse_amount( \%my_config, $amount),
+                 amount_fx => $form->parse_amount( \%my_config, $amount_fx),
+                 curr => $form->{"curr_$iter"},
+             };
         }
     }
 
-    $template = LedgerSMB::DBObject::TransTemplate->new(%$data);
-    $template->save;
+    my ($journal) = $form->call_procedure(
+        funcname => 'journal__add',
+        args     => [
+            $form->{reference},
+            $form->{description},
+            1, # GJ journal
+            $form->{transdate},
+            0,
+            1,
+            $form->{curr_0} ###BUG: Sets a currency that doesn't exist on the GL screen!
+        ]);
+    for my $line (@lines) {
+        my ($acc) = $form->call_procedure(
+            funcname => 'account__get_from_accno',
+            args     => [ $line->{accno} ]);
+        $form->call_procedure(
+            funcname => 'journal__add_line',
+            args     => [
+                $acc->{id},
+                $journal->{id},
+                $line->{amount},
+                $line->{amount_fx},
+                $line->{curr},
+                'false',
+                undef, ###BUG: discards the 'memo' text!
+                undef ###BUG: discards selected reporting units!
+            ]);
+    }
     $form->redirect( $locale->text('Template Saved!') );
 }
 
@@ -359,6 +507,7 @@ sub display_row {
   $form->{totaldebit}  = 0;
   $form->{totalcredit} = 0;
 
+    $form->{_setting_decimal_places} //= LedgerSMB::Setting->new(%$form)->get('decimal_places');
     for my $i ( 0 .. $form->{rowcount} ) {
 
         my $temphash1;
@@ -371,7 +520,6 @@ sub display_row {
         $temphash1->{fx_transactionset} = 1;
         if (!defined $form->{"accno_$i"} || ! $form->{"accno_$i"}) {
                   $temphash1->{accnoset}=0;   #use  @{ $form->{all_accno} }
-                  $temphash1->{projectset}=0; #use  @{ $form->{all_project} }
                   $temphash1->{fx_transactionset}=0;    #use checkbox and value=1 if transfer=1
 
         }
@@ -380,12 +528,12 @@ sub display_row {
                 LedgerSMB::PGNumber->from_input($form->{"debit_$i"});
             $form->{"credit_$i"} =
                 LedgerSMB::PGNumber->from_input($form->{"credit_$i"});
-            $form->{totaldebit}  += $form->{"debit_$i"};
-            $form->{totalcredit} += $form->{"credit_$i"};
+            $form->{totaldebit}  += ($form->{"debit_$i"} // 0);
+            $form->{totalcredit} += ($form->{"credit_$i"} // 0);
 
             for (qw(debit debit_fx credit credit_fx)) {
                 $form->{"${_}_$i"} = ($form->{"${_}_$i"})
-                    ? $form->format_amount( \%myconfig, $form->{"${_}_$i"}, 2 )
+                    ? $form->format_amount( \%myconfig, $form->{"${_}_$i"}, $form->{_setting_decimal_places} )
                     : "";
                 $temphash1->{$_} = $form->{"${_}_$i"};
             }
@@ -425,15 +573,11 @@ sub edit {
 
     $form->all_business_units($form->{transdate}, undef, 'GL');
 
-    $form->{locked} =
-      ( $form->{revtrans} )
-      ? '1'
-      : ( $form->datetonum( \%myconfig, $form->{transdate} ) <=
-          $form->datetonum( \%myconfig, $form->{closedto} ) );
     # readonly
     if ( !$form->{readonly} ) {
         $form->{readonly} = 1
-          if $myconfig{acs} =~ /General Ledger--Add Transaction/;
+            if ($myconfig{acs}
+                and $myconfig{acs} =~ /General Ledger--Add Transaction/);
     }
     $form->{title} = "Edit";
     if ($form->{department_id}) {
@@ -446,7 +590,9 @@ sub edit {
 
     foreach my $ref (@{ $form->{GL} }) {
         $form->{"accno_$i"} = "$ref->{accno}--$ref->{description}";
-        $form->{"projectnumber_$i"} = "$ref->{projectnumber}--$ref->{project_id}";
+        $form->{"projectnumber_$i"} =
+            "$ref->{projectnumber}--$ref->{project_id}"
+            if $ref->{projectnumber};
         for (qw(curr source memo)) { $form->{"${_}_$i"} = $ref->{$_} }
         if ( $ref->{amount_bc} < 0 ) {
             $form->{totaldebit} -= $ref->{amount_bc};
@@ -479,62 +625,15 @@ sub create_links {
 
 }
 
-sub gl_subtotal_tt {
-
-    my %column_data;
-    $subtotaldebit =
-      $form->format_amount( \%myconfig, $subtotaldebit, 2, " " );
-    $subtotalcredit =
-      $form->format_amount( \%myconfig, $subtotalcredit, 2, " " );
-
-    for (@column_index) { $column_data{$_} = " " }
-    $column_data{class} = 'subtotal';
-
-    $column_data{debit} = $subtotaldebit;
-    $column_data{credit} = $subtotalcredit;
-
-    $subtotaldebit  = 0;
-    $subtotalcredit = 0;
-
-    $sameitem = $ref->{ $form->{sort} };
-
-    return \%column_data;
-}
-
-sub gl_subtotal {
-    $subtotaldebit =
-      $form->format_amount( \%myconfig, $subtotaldebit, 2, "&nbsp;" );
-    $subtotalcredit =
-      $form->format_amount( \%myconfig, $subtotalcredit, 2, "&nbsp;" );
-
-    for (@column_index) { $column_data{$_} = "<td>&nbsp;</td>" }
-
-    $column_data{debit} =
-      "<th align=right class=listsubtotal>$subtotaldebit</td>";
-    $column_data{credit} =
-      "<th align=right class=listsubtotal>$subtotalcredit</td>";
-
-    print "<tr class=listsubtotal>";
-    for (@column_index) { print "$column_data{$_}\n" }
-    print "</tr>";
-
-    $subtotaldebit  = 0;
-    $subtotalcredit = 0;
-
-    $sameitem = $ref->{ $form->{sort} };
-
-}
-
-
 sub update {
     &create_links;
-     my $min_lines = $LedgerSMB::Company_Config::settings->{min_empty};
-     $form->open_form unless $form->check_form;
+    my $min_lines = $form->get_setting('min_empty');
+    $form->open_form unless $form->check_form;
 
-     $form->{transdate} = LedgerSMB::PGDate->from_input($form->{transdate})->to_output();
-     if ( $form->{transdate} ne $form->{oldtransdate} ) {
-         $form->{oldtransdate} = $form->{transdate};
-     }
+    $form->{transdate} = LedgerSMB::PGDate->from_input($form->{transdate})->to_output();
+    if ( $form->{transdate} ne $form->{oldtransdate} ) {
+        $form->{oldtransdate} = $form->{transdate};
+    }
 
     $form->all_business_units($form->{transdate}, undef, 'GL');
     GL->get_all_acc_dep_pro( \%myconfig, \%$form );
@@ -604,7 +703,10 @@ sub update {
 }
 
 
-
+sub post_and_approve {
+    post();
+    $form->call_procedure(funcname=>'draft_approve', args => [ $form->{id} ]);
+}
 
 sub post {
     if ($form->{id}){
@@ -615,13 +717,19 @@ sub post {
         $form->finalize_request();
     };
     $form->isblank( "transdate", $locale->text('Transaction Date missing!') );
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
 
     $transdate = $form->datetonum( \%myconfig, $form->{transdate} );
-    $closedto  = $form->datetonum( \%myconfig, $form->{closedto} );
 
     $form->error(
         $locale->text('Cannot post transaction for a closed period!') )
-      if ( $transdate <= $closedto );
+        if ( $transdate and $form->is_closed( $transdate ) );
 
     check_balanced($form);
     if ( !$form->{repost} ) {
@@ -636,14 +744,20 @@ sub post {
 
 }
 
-sub delete {
+sub del {
     $form->error($locale->text('Cannot delete posted transaction'))
        if ($form->{approved});
-    my $draft = LedgerSMB::DBObject::Draft->new(%$form);
-    $draft->delete();
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
+    $form->call_procedure(funcname=>'draft_delete', args => [ $form->{id} ]);
     delete $form->{id};
     delete $form->{reference};
-    add();
+    new();
 }
 
 
@@ -673,9 +787,91 @@ sub check_balanced {
 }
 
 sub save_as_new {
-    for (qw(id printed emailed queued)) { delete $form->{$_} }
-    $form->{approved} = 0;
+    for (qw(id printed emailed)) { delete $form->{$_} }
+    if ($form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')->fetch_workflow(
+            'GL', $form->{workflow_id}
+            );
+        $wf->context->param( transdate => $form->{transdate} );
+        $wf->execute_action( $form->{__action} );
+    }
+    delete $form->{approved};
     &post;
 }
 
+sub print {
+    # Get all the business units required to render
+    $form->all_business_units($form->{transdate}, undef, 'GL');
+    # Get the actual je transaction
+    &display_row(@_);
+
+    my $templateName = "print_journal_entry";
+    my $filename = "Journal_Entry_" . $form->{id} . ".html";
+    $form->{print_title} = "Journal Entry Transaction";
+
+    if($form->{transfer}) {
+        $templateName = "print_cash_transfer";
+        $filename = "Cash_Transfer_" . $form->{id} . ".html";
+        $form->{print_title} = "Transfer Voucher";
+
+        my $amount = $form->{"debit_0"} eq "" ? $form->{"credit_0"} : $form->{"debit_0"};
+        my $amount_fx = $form->{"debit_fx_0"} eq "" ? $form->{"credit_fx_0"} : $form->{"debit_fx_0"};
+
+        $form->{exchange_rate} = LedgerSMB::PGNumber->from_input($amount) / LedgerSMB::PGNumber->from_input($amount_fx);
+        $form->{exchange_rate} = $form->format_amount( \%myconfig, $form->{exchange_rate}, LedgerSMB::Setting->new(%$form)->get('decimal_places') );
+
+        $form->{curr} = $form->{"curr_0"};;
+
+        $form->{amount} = $form->{totaldebit};
+        my $fmt = LedgerSMB::Num2text->new($form->{_locale});
+        $form->{text_amount} = $fmt->num2text(LedgerSMB::PGNumber->from_input($form->{amount}));
+    } else {
+        # render the code--description for all business unit instead of id
+        for my $drow (@displayrows) {
+            for my $cls (@{$form->{bu_class}}) {
+                if(scalar @{$form->{b_units}->{$cls->{id}}}) {
+                    for my $bu (@{$form->{b_units}->{"$cls->{id}"}}) {
+                        if ($drow->{"b_unit_$cls->{id}"} eq $bu->{id}) {
+                            $drow->{"b_unit_$cls->{id}"} = $bu->{control_code} . '--' . $bu->{description};
+                            last;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    my %copy_settings = (
+        email => 'company_email',
+        company => 'company_name',
+        businessnumber => 'businessnumber',
+        address => 'company_address',
+        tel => 'company_phone',
+        fax => 'company_fax',
+    );
+    while (my ($key, $setting) = each %copy_settings ) {
+        $form->{$key} = $form->get_setting($setting);
+    }
+
+    my %output_options = (
+        filename => $filename
+    );
+    my $template = LedgerSMB::Template->new(
+        user => \%myconfig,
+        template => $templateName,
+        dbh => $form->{dbh},
+        path => 'DB',
+        locale => $locale,
+        output_options => \%output_options,
+        format_plugin => $form->{_wire}->get( 'output_formatter' )->get( 'HTML' ),
+    );
+    $template->render(
+        {
+            form => $form,
+            displayrows => \@displayrows
+        }
+    );
+    LedgerSMB::Legacy_Util::output_template($template, $form);
+}
+
 1;
