@@ -43,14 +43,14 @@ package lsmb_legacy;
 
 use List::Util qw(max min);
 use LedgerSMB::OE;
+use LedgerSMB::IIAA;
 use LedgerSMB::IR;
 use LedgerSMB::IS;
 use LedgerSMB::PE;
 use LedgerSMB::Setting;
 use LedgerSMB::Tax;
-use LedgerSMB::Template::UI;
 use LedgerSMB::Legacy_Util;
-use LedgerSMB::Locale;
+
 
 require "old/bin/arap.pl";
 require "old/bin/io.pl";
@@ -76,7 +76,7 @@ sub add {
         $form->{vc}    = 'customer';
     }
 
-    $form->{callback} = "$form->{script}?action=add&type=$form->{type}&vc=$form->{vc}"
+    $form->{callback} = "$form->{script}?__action=add&type=$form->{type}&vc=$form->{vc}"
       unless $form->{callback};
 
     $form->{rowcount} = 0;
@@ -88,6 +88,13 @@ sub add {
 }
 
 sub edit {
+    if (not $form->{id} and $form->{workflow_id}) {
+        my $wf = $form->{_wire}->get('workflows')
+            ->fetch_workflow( 'Order/Quote', $form->{workflow_id} );
+        $form->{id} = $wf->context->param( '_extra' )->{id};
+        delete $form->{workflow_id};
+    }
+
     OE->get_type($form);
     if ( $form->{type} =~ /(purchase_order|bin_list)/ ) {
         $form->{title} = $locale->text('Edit Purchase Order');
@@ -124,16 +131,6 @@ sub order_links {
              billing => 0,
              job => 1 );
 
-    # get projects, departments, languages
-    $form->get_regular_metadata(
-        \%myconfig,
-        $form->{vc},
-        $form->{transdate},
-        1,
-    );
-
-    $form->generate_selects;
-
     # retrieve order/quotation
     OE->retrieve( \%myconfig, \%$form );
 
@@ -169,21 +166,22 @@ sub order_links {
 }
 
 sub prepare_order {
-
-    $form->{format}   = "postscript" if $myconfig{printer};
-    $form->{media}    = $myconfig{printer};
-    $form->{formname} = $form->{type};
+    my %args          = @_;
+    $form->{format}   //= "postscript" if $myconfig{printer};
+    $form->{media}    //= $myconfig{printer};
+    $form->{formname} //= $form->{type};
     $form->{sortby} ||= "runningnumber";
     $form->{currency} =~ s/ //g;
     $form->{oldcurrency} = $form->{currency};
 
     if ( $form->{id} ) {
 
-        for(
-          qw(ordnumber quonumber shippingpoint shipvia notes intnotes shiptoname shiptoaddress1 shiptoaddress2 shiptocity shiptostate shiptozipcode shiptocountry shiptocontact)
-          )
-        {
-            $form->{$_} = $form->quote( $form->{$_} );
+        unless ($args{unquoted}) {
+            for(qw(ordnumber quonumber shippingpoint shipvia notes intnotes
+                   shiptoname shiptoaddress1 shiptoaddress2 shiptocity
+                   shiptostate shiptozipcode shiptocountry shiptocontact)) {
+                $form->{$_} = $form->quote( $form->{$_} );
+            }
         }
 
         my $i;
@@ -224,8 +222,10 @@ sub prepare_order {
               $form->format_amount( \%myconfig, $form->{"qty_$i"} );
             $form->{"oldqty_$i"} = $form->{"qty_$i"};
 
-            for (qw(partnumber sku description unit)) {
-                $form->{"${_}_$i"} = $form->quote( $form->{"${_}_$i"} );
+            unless ($args{unquoted}) {
+                for (qw(partnumber sku description unit)) {
+                    $form->{"${_}_$i"} = $form->quote( $form->{"${_}_$i"} );
+                }
             }
             $form->{rowcount} = $i;
         }
@@ -351,8 +351,8 @@ sub form_header {
             $exchangerate .=
                 qq|<th align=right>|
               . $locale->text('Exchange Rate')
-              . qq|</th><td>$form->{exchangerate}</td>
-      <input type=hidden name=exchangerate value=$form->{exchangerate}>
+              . qq|</th><td>$form->{exchangerate}
+      <input type=hidden name=exchangerate value=$form->{exchangerate}></td>
 |;
         }
         else {
@@ -399,8 +399,8 @@ sub form_header {
           <tr class="ordnumber-row">
         <th width=70% align=right nowrap>| . $locale->text('Order Number') . qq|</th>
                 <td><input data-dojo-type="dijit/form/TextBox" id=ordnumber name=ordnumber size=20 value="$form->{ordnumber}">
-                     $sequences</td>
-        <input type=hidden name=quonumber value="$form->{quonumber}">
+                     $sequences
+        <input type=hidden name=quonumber value="$form->{quonumber}"></td>
           </tr>
           <tr class="transdate-row">
         <th align=right nowrap>| . $locale->text('Order Date') . qq|</th>
@@ -474,8 +474,8 @@ sub form_header {
               . $locale->text('Quotation Number')
               . qq|</th>
         <td><input data-dojo-type="dijit/form/TextBox" id=quonumber name=quonumber size=20 value="$form->{quonumber}">
-                    $sequences</td>
-        <input type=hidden name=ordnumber value="$form->{ordnumber}">
+                    $sequences
+        <input type=hidden name=ordnumber value="$form->{ordnumber}"></td>
           </tr>
 |;
         }
@@ -484,8 +484,8 @@ sub form_header {
           <tr class="rfqnumber-row">
         <th width=70% align=right nowrap>| . $locale->text('RFQ Number') . qq|</th>
         <td><input data-dojo-type="dijit/form/TextBox" id=quonumber name=quonumber size=20 value="$form->{quonumber}">
-                    $sequences</td>
-        <input type=hidden name=ordnumber value="$form->{ordnumber}">
+                    $sequences
+        <input type=hidden name=ordnumber value="$form->{ordnumber}"></td>
           </tr>
 |;
 
@@ -509,7 +509,7 @@ sub form_header {
 <input type=hidden name=oldtransdate value=$form->{oldtransdate}>|;
 
     if ( $form->{"select$form->{vc}"} ) {
-        $vc = qq|<select data-dojo-type="dijit/form/Select" name=$form->{vc} id=$form->{vc}>$form->{"select$form->{vc}"}</select>|;
+        $vc = qq|<select data-dojo-type="lsmb/FilteringSelect" name="$form->{vc}" id="$form->{vc}"><option></option>$form->{"select$form->{vc}"}</select>|;
     }
     else {
         if ($form->{vc} eq 'vendor'){
@@ -519,7 +519,7 @@ sub form_header {
         }
         $vc = qq|<input data-dojo-type="dijit/form/TextBox" id=$form->{vc} name=$form->{vc} value="$form->{$form->{vc}}" size=35>
              <a id="new-contact" target="_blank"
-                 href="erp.pl?action=root#contact.pl?action=add&entity_class=$eclass">
+                 href="erp.pl?__action=root#contact.pl?__action=add&entity_class=$eclass">
                  [| . $locale->text('New') . qq|]</a>|;
     }
 
@@ -557,25 +557,26 @@ sub form_header {
     }
 
     $i     = $form->{rowcount} + 1;
-    $focus = "partnumber_$i";
+    $focus = $form->{barcode} ? "barcode" : "partnumber_$i";
 
     $form->header;
 
     print qq|
-<body class="lsmb $form->{dojo_theme}" onLoad="document.forms[0].${focus}.focus()" />
+<body>
 | . $form->open_status_div($status_div_id) . qq|
 <form method="post"
       id="invoice"
       data-dojo-type="lsmb/Invoice"
+      data-lsmb-focus="${focus}"
       action="$form->{script}" >
 |;
 
     if ($form->{notice}){
          print qq|$form->{notice}<br/>|;
     }
-    $form->hide_form(qw(entity_control_code meta_number tax_id address city));
+    $form->hide_form(qw(entity_control_code meta_number tax_id address city zipcode state country));
     $form->hide_form(
-        qw(id type printed emailed queued vc title discount creditlimit creditremaining tradediscount business recurring form_id nextsub
+        qw(id type printed emailed vc title discount creditlimit creditremaining tradediscount business recurring form_id nextsub
    lock_description)
     );
 
@@ -593,9 +594,9 @@ sub form_header {
         <table width=100%>
           <tr>
         <th align=right>$vclabel</th>
-        <td colspan=3>$vc</td>
+        <td colspan=3>$vc
         <input type=hidden name=$form->{vc}_id value=$form->{"$form->{vc}_id"}>
-        <input type=hidden name="old$form->{vc}" value="$form->{"old$form->{vc}"}">
+        <input type=hidden name="old$form->{vc}" value="$form->{"old$form->{vc}"}"></td>
           </tr>
           $creditremaining
           $business
@@ -640,12 +641,23 @@ sub form_header {
     }
     if ( !$form->{readonly} ) {
         %button = (
-            'update' =>
-              { ndx => 1, key => 'U', value => $locale->text('Update') },
+            'update' => {
+                ndx => 1,
+                key => 'U',
+                value => $locale->text('Update'),
+                doing => $locale->text('Updating...'),
+                done => $locale->text('Updated')
+            },
             'print' =>
               { ndx => 2, key => 'P', value => $locale->text('Print'),
                 type => 'lsmb/PrintButton' },
-            'save' => { ndx => 3, key => 'S', value => $locale->text('Save') },
+            'save' => {
+                ndx => 3,
+                key => 'S',
+                value => $locale->text('Save'),
+                doing => $locale->text('Saving...'),
+                done => $locale->text('Saved')
+            },
             'ship_to' =>
               { ndx => 4, key => 'T', value => $locale->text('Ship to') },
             'e_mail' =>
@@ -681,15 +693,15 @@ sub form_header {
                 value => $locale->text('Purchase Order')
             },
             'rfq' => { ndx => 14, key => 'Q', value => $locale->text('RFQ') },
-            'schedule' =>
-              { ndx => 15, key => 'H', value => $locale->text('Schedule') },
+            # Commented out; see github isue #5295
+#            'schedule' =>
+#              { ndx => 15, key => 'H', value => $locale->text('Schedule') },
             'delete' =>
               { ndx => 16, key => 'D', value => $locale->text('Delete') },
         );
 
         %a = ();
-        for ( "update", "ship_to", "save" ) { $a{$_} = 1 }
-        $a{'print_and_save'} = 1 if ${LedgerSMB::Sysconfig::latex};
+        for ( "update", "ship_to", "save", "print_and_save" ) { $a{$_} = 1 }
 
         if ( $form->{id} ) {
 
@@ -697,7 +709,7 @@ sub form_header {
             $a{'delete'}                = 1;
             $a{'print'}                 = 1;
             $a{'save_as_new'}           = 1;
-            $a{'print_and_save_as_new'} = 1 if ${LedgerSMB::Sysconfig::latex};
+            $a{'print_and_save_as_new'} = 1;
 
             if ( $form->{type} =~ /sales_/ ) {
                 if ( $myconfig{acs} !~ /AR--Sales Invoice/ ) {
@@ -735,9 +747,10 @@ sub form_header {
             }
         }
 
-        if ( $form->{type} =~ /_order/ ) {
-            $a{'schedule'} = 1;
-        }
+        # Commented out; see github isue #5295
+#        if ( $form->{type} =~ /_order/ ) {
+#            $a{'schedule'} = 1;
+#        }
 
     }
     for ( keys %button ) { delete $button{$_} if !$a{$_} }
@@ -745,8 +758,8 @@ sub form_header {
     for ( sort { $button{$a}->{ndx} <=> $button{$b}->{ndx} } keys %button ) {
         $form->print_button( \%button, $_ );
     }
+    $form->hide_form(qw(defaultcurrency workflow_id));
     print "</td></tr>";
-    $form->hide_form(qw(defaultcurrency));
 }
 
 sub form_footer {
@@ -780,6 +793,7 @@ qq|<textarea data-dojo-type="dijit/form/Textarea" id=intnotes name=intnotes rows
 |;
     }
 
+    $form->{_setting_decimal_places} = LedgerSMB::Setting->new(%$form)->get('decimal_places');
     if ( !$form->{taxincluded} ) {
         foreach my $item (keys %{$form->{taxes}}) {
             my $taccno = $item;
@@ -787,7 +801,7 @@ qq|<textarea data-dojo-type="dijit/form/Textarea" id=intnotes name=intnotes rows
             $form->{"${taccno}_total"} = $form->format_amount(
                 \%myconfig,
                 $form->round_amount( $form->{taxes}{$item}, 2 ),
-                2
+                $form->{_setting_decimal_places}
             );
             next if !$form->{"${taccno}_total"};
             $tax .= qq|
@@ -798,7 +812,7 @@ qq|<textarea data-dojo-type="dijit/form/Textarea" id=intnotes name=intnotes rows
         }
 
         $form->{invsubtotal} =
-          $form->format_amount( \%myconfig, $form->{invsubtotal}, 2, 0 );
+          $form->format_amount( \%myconfig, $form->{invsubtotal}, $form->{_setting_decimal_places}, 0 );
 
         $subtotal = qq|
           <tr>
@@ -810,9 +824,13 @@ qq|<textarea data-dojo-type="dijit/form/Textarea" id=intnotes name=intnotes rows
     }
     $form->{oldinvtotal} = $form->{invtotal};
     $form->{invtotal} =
-      $form->format_amount( \%myconfig, $form->{invtotal}, 2, 0 );
+      $form->format_amount( \%myconfig, $form->{invtotal}, $form->{_setting_decimal_places}, 0 );
 
+    my $display_barcode = $form->get_setting('have_barcodes') ? "initial" : "none";
     print qq|
+  <tr style="display:$display_barcode">
+   <td colspan="5"><b><label for="barcode">Barcode</label></b>: <input data-dojo-type="dijit/form/TextBox" id=barcode name=barcode></td>
+  </tr>
   <tr>
     <td>
       <table width=100%>
@@ -842,10 +860,19 @@ qq|<textarea data-dojo-type="dijit/form/Textarea" id=intnotes name=intnotes rows
       </td>
     </tr>
       </table>
-    </td>
-  </tr>
 <input type=hidden name=oldinvtotal value=$form->{oldinvtotal}>
 <input type=hidden name=oldtotalpaid value=$totalpaid>
+    </td>
+  </tr>
+  <tr>
+    <td>
+|;
+
+    IIAA->print_wf_history_table($form, 'Order/Quote');
+
+    print qq|
+    </td>
+  </tr>
   <tr>
     <td><hr size=3 noshade></td>
   </tr>
@@ -901,7 +928,7 @@ qq|<textarea data-dojo-type="dijit/form/Textarea" id=intnotes name=intnotes rows
         foreach my $file (@{$form->{files}}){
               print qq|
 <tr>
-<td><a href="file.pl?action=get&file_class=2&ref_key=$form->{id}&id=$file->{id}&type=sales_quotation&additional=type"
+<td><a href="file.pl?__action=get&file_class=2&ref_key=$form->{id}&id=$file->{id}&type=sales_quotation&additional=type"
        target="_download">$file->{file_name}</a></td>
 <td>$file->{mime_type}</td>
 <td>|.$file->{uploaded_at}.qq|</td>
@@ -938,9 +965,9 @@ qq|<textarea data-dojo-type="dijit/form/Textarea" id=intnotes name=intnotes rows
        }
        print qq|
 </table>|;
-       $callback = $form->escape("oe.pl?action=edit&id=".$form->{id});
+       $callback = $form->escape("oe.pl?__action=edit&id=".$form->{id});
        print qq|
-<a href="file.pl?action=show_attachment_screen&ref_key=$form->{id}&file_class=2&callback=$callback"
+<a href="file.pl?__action=show_attachment_screen&ref_key=$form->{id}&file_class=2&callback=$callback"
    >[| . $locale->text('Attach') . qq|]</a>|;
     }
 
@@ -1044,11 +1071,15 @@ sub update {
                       && ( $form->{"description_$i"} eq "" )
                       && ( $form->{"partsgroup_$i"}  eq "" ) );
     }
+    if ($form->{barcode}) {
+        $non_empty_rows++;
+        IIAA->process_form_barcode(\%myconfig, $form, $non_empty_rows, $form->{barcode});
+    }
 
     my $current_empties = $form->{rowcount} - $non_empty_rows;
     my $new_empties =
         max(0,
-            max($LedgerSMB::Company_Config::settings->{min_empty}, 1)
+            max($form->get_setting('min_empty'),1)
             - $current_empties);
 
 
@@ -1171,25 +1202,6 @@ sub update {
                 }
 
             }
-            else {
-
-            # ok, so this is a new part
-            # ask if it is a part or service item
-
-                if (   $form->{"partsgroup_$i"}
-                    && ( $form->{"partsnumber_$i"} eq "" )
-                    && ( $form->{"description_$i"} eq "" ) )
-                {
-                    $form->{rowcount}--;
-                    &display_form;
-                }
-                else {
-
-                    $form->{"id_$i"}   = 0;
-                    $form->{"unit_$i"} = $locale->text('ea');
-                    &new_item;
-                }
-            }
         }
     }
     $form->all_vc(\%myconfig, $form->{vc}, $form->{transdate}, 1) if ! @{$form->{"all_$form->{vc}"}};
@@ -1201,6 +1213,10 @@ sub update {
 }
 
 sub save {
+    &_save;
+}
+
+sub _save {
     delete $form->{display_form};
 
 
@@ -1270,8 +1286,9 @@ sub save {
 
     if ( !$form->{repost}  && $form->{id}) {
         $form->{repost} = 1;
-        my $template = LedgerSMB::Template::UI->new_UI;
+        my $template = $form->{_wire}->get('ui');
         return LedgerSMB::Legacy_Util::render_psgi(
+            $form,
             $template->render($form, 'oe-save-warn',
                               {
                                   hiddens => $form,
@@ -1286,6 +1303,21 @@ sub save {
     }
 
     if ( OE->save( \%myconfig, \%$form ) ) {
+       # the old workflow is being saved-as. the new workflow has its id
+       # set in workflow_id, because the form was saved already...
+       my $id  = $form->{old_workflow_id} // $form->{workflow_id};
+       my $wf  = $form->{_wire}->get('workflows')
+           ->fetch_workflow( 'Order/Quote', $id );
+       my $ctx = $wf->context;
+       $ctx->param( spawned_type => 'Order/Quote' );
+       $ctx->param( spawned_id   => $form->{workflow_id} );
+
+       # m/save_as/ matches both print_and_save_as_new as well as save_as_new
+       $wf->execute_action( ($form->{__action} =~ m/save_as/
+                             ? 'save_as_new' : 'save') );
+
+       delete $form->{old_workflow_id};
+
        edit();
     }
     else {
@@ -1296,6 +1328,14 @@ sub save {
 
 sub print_and_save {
 
+    my $wf = $form->{_wire}->get('workflows')
+        ->fetch_workflow( 'Order/Quote', $form->{workflow_id} );
+    $wf->execute_action( 'print_and_save' );
+
+    &_print_and_save;
+}
+
+sub _print_and_save {
     $form->error( $locale->text('Select postscript or PDF!') )
       if $form->{format} !~ /(postscript|pdf)/;
     $form->error( $locale->text('Select a Printer!') )
@@ -1306,13 +1346,25 @@ sub print_and_save {
     for ( keys %$form ) { $old_form->{$_} = $form->{$_} }
     $old_form->{rowcount}++;
 
+    my $wf =
+        $form->{_wire}->get('workflows')
+        ->fetch_workflow( 'Order/Quote', $form->{workflow_id} );
+    $wf->execute_action( 'print' );
     &print_form($old_form);
 
 }
 
 sub delete {
 
-    $form->header;
+    # The actual deletion in executed in the "yes" function below;
+    # if we execute the "delete" action here, we'll land the order/quote
+    # in limbo if the "yes" action isn't performed on the UI side (the
+    # workflow has a DELETED state whereas the quote still exists...)
+    #
+    # my $wf = $form->{_wire}->get('workflows')
+    #     ->fetch_workflow( 'Order/Quote', $form->{workflow_id} );
+    # $wf->execute_action( 'delete' );
+    # $form->header;
 
     if ( $form->{type} =~ /_order$/ ) {
         $msg = $locale->text('Are you sure you want to delete Order Number');
@@ -1327,10 +1379,10 @@ sub delete {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 |;
 
-    $form->{action} = "yes";
+    $form->{__action} = "yes";
     $form->hide_form;
 
     print qq|
@@ -1338,7 +1390,7 @@ sub delete {
 
 <h4>$msg $form->{$ordnumber}</h4>
 <p>
-<button data-dojo-type="dijit/form/Button" id="action-yes" name="action" class="submit" type="submit" value="yes">|
+<button data-dojo-type="dijit/form/Button" id="action-yes" name="__action" class="submit" type="submit" value="yes">|
       . $locale->text('Yes')
       . qq|</button>
 </form>
@@ -1351,6 +1403,11 @@ sub delete {
 
 sub yes {
 
+    my $wf = $form->{_wire}->get('workflows')
+        ->fetch_workflow( 'Order/Quote', $form->{workflow_id} );
+    $wf->execute_action( 'delete' );
+    $form->header;
+
     if ( $form->{type} =~ /_order$/ ) {
         $msg = $locale->text('Order deleted!');
         $err = $locale->text('Cannot delete order!');
@@ -1360,7 +1417,7 @@ sub yes {
         $err = $locale->text('Cannot delete quotation!');
     }
 
-    if ( OE->delete( \%myconfig, \%$form, ${LedgerSMB::Sysconfig::spool} ) ) {
+    if ( OE->delete( \%myconfig, \%$form ) ) {
         $form->redirect($msg);
     }
     else {
@@ -1410,10 +1467,12 @@ sub invoice {
     $form->{closed} = 1;
 
     OE->save( \%myconfig, \%$form );
+    my $wf = $form->{_wire}->get('workflows')
+        ->fetch_workflow( 'Order/Quote', $form->{workflow_id} );
+    my $action;
 
-    $form->{duedate} =
-      $form->current_date( \%myconfig, $form->{transdate}, $form->{terms} * 1 );
-
+    $form->{transdate} = '';
+    $form->{duedate} = '';
     $form->{id}     = '';
     $form->{closed} = 0;
     $form->{rowcount}--;
@@ -1422,7 +1481,11 @@ sub invoice {
     if ( $form->{type} =~ /_order$/ ) {
         $form->{exchangerate} = $exchangerate;
         &create_backorder;
+        $form->{transdate} = '';
+        $form->{duedate} = '';
+        $form->{crdate} = '';
     }
+    $form->{workflow_id} = '';
 
     if (   $form->{type} eq 'purchase_order'
         || $form->{type} eq 'request_quotation' )
@@ -1432,6 +1495,7 @@ sub invoice {
 
         $script  = "ir";
         $buysell = 'sell';
+        $action  = 'vendor_invoice';
     }
     if ( $form->{type} eq 'sales_order' || $form->{type} eq 'sales_quotation' )
     {
@@ -1439,9 +1503,11 @@ sub invoice {
         $form->{script} = 'is.pl';
         $script         = "is";
         $buysell        = 'buy';
+        $action         = 'sales_invoice';
     }
+    my $lib = uc($script);
 
-    for (qw(id subject message printed emailed queued)) { delete $form->{$_} }
+    for (qw(id subject message printed emailed)) { delete $form->{$_} }
     $form->{ $form->{vc} } =~ s/--.*//g;
     $form->{type} = "invoice";
 
@@ -1481,13 +1547,33 @@ sub invoice {
               $form->format_amount( \%myconfig, $form->{"${_}_$i"} );
         }
     }
+    $form->{duedate} = $form->current_date( \%myconfig, $form->{transdate},
+                                            $form->{terms} * 1 );
 
-    for (qw(id subject message printed emailed queued audittrail)) {
+    for (qw(id subject message printed emailed audittrail)) {
         delete $form->{$_};
     }
 
-    &display_form;
+    {
+        # force the invoice to be saved, not posted!
+        local $form->{separate_duties} = 1;
+        local $form->{approved} = undef;
 
+        my $wf = $form->{_wire}->get('workflows')
+            ->create_workflow( 'AR/AP' );
+        $form->{workflow_id} = $wf->id;
+
+        $wf->execute_action( 'post' );
+        # $lib contains either 'IS' or 'IR': the sales and purchase invoice libs
+        $lib->post_invoice(\%myconfig, $form);
+        $lib->retrieve_invoice(\%myconfig, $form);
+        &prepare_invoice;
+    }
+    $wf->context->param( 'spawned_id'   => $form->{workflow_id} );
+    $wf->context->param( 'spawned_type' => 'AR/AP' );
+    $wf->execute_action( $action );
+
+    &display_form;
 }
 
 sub backorder_exchangerate {
@@ -1498,11 +1584,11 @@ sub backorder_exchangerate {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 |;
 
     # delete action variable
-    for (qw(action nextsub exchangerate)) { delete $form->{$_} }
+    for (qw(nextsub exchangerate)) { delete $form->{$_} }
 
     $form->hide_form;
 
@@ -1540,7 +1626,7 @@ sub backorder_exchangerate {
 
 <br>
 
-<button data-dojo-type="dijit/form/Button" id="action-continue" name="action" class="submit" type="submit" value="continue">|
+<button data-dojo-type="dijit/form/Button" id="action-continue" name="__action" class="submit" type="submit" value="continue">|
       . $locale->text('Continue')
       . qq|</button>
 
@@ -1597,7 +1683,7 @@ sub create_backorder {
     }
 
     @flds =
-      qw(partnumber description qty ship unit sellprice discount oldqty orderitems_id id bin weight listprice lastcost taxaccounts pricematrix sku onhand deliverydate reqdate projectnumber partsgroup assembly);
+      qw(partnumber description qty ship unit sellprice discount oldqty orderitems_id id bin weight listprice lastcost taxaccounts pricematrix sku onhand deliverydate reqdate projectnumber partsgroup assembly notes serialnumber);
 
     foreach my $i ( 1 .. $form->{rowcount} ) {
         for (qw(qty sellprice discount)) {
@@ -1610,7 +1696,7 @@ sub create_backorder {
     }
 
     # clear flags
-    for (qw(id subject message cc bcc printed emailed queued audittrail)) {
+    for (qw(id subject message cc bcc printed emailed audittrail)) {
         delete $form->{$_};
     }
 
@@ -1641,15 +1727,24 @@ sub create_backorder {
 
 sub save_as_new {
 
-    for (qw(closed id printed emailed queued)) { delete $form->{$_} }
-    &save;
-
+    # orders don't have a quonumber
+    # quotes don't have an ordnumber
+    $form->{old_workflow_id} = $form->{workflow_id};
+    for (qw(closed id printed emailed ordnumber quonumber workflow_id)) {
+        delete $form->{$_}
+    }
+    &_save;
 }
 
 sub print_and_save_as_new {
 
-    for (qw(closed id printed emailed queued)) { delete $form->{$_} }
-    &print_and_save;
+    # orders don't have a quonumber
+    # quotes don't have an ordnumber
+    $form->{old_workflow_id} = $form->{workflow_id};
+    for (qw(closed id printed emailed ordnumber quonumber workflow_id)) {
+        delete $form->{$_}
+    }
+    &_print_and_save;
 
 }
 
@@ -1740,12 +1835,12 @@ sub display_ship_receive {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 
 <input type=hidden name=display_form value=display_ship_receive>
 |;
 
-    $form->hide_form(qw(id type media format printed emailed queued vc));
+    $form->hide_form(qw(id type media format printed emailed vc));
 
     print qq|
 <input type=hidden name="old$form->{vc}" value="$form->{"old$form->{vc}"}">
@@ -1763,9 +1858,9 @@ sub display_ship_receive {
         <table width=100%>
           <tr>
         <th align=right>$vclabel</th>
-        <td colspan=3>$form->{$form->{vc}}</td>
+        <td colspan=3>$form->{$form->{vc}}
         <input type=hidden name=$form->{vc} value="$form->{$form->{vc}}">
-        <input type=hidden name="$form->{vc}_id" value=$form->{"$form->{vc}_id"}>
+        <input type=hidden name="$form->{vc}_id" value=$form->{"$form->{vc}_id"}></td>
           </tr>
           $department
           <tr>
@@ -1786,19 +1881,19 @@ sub display_ship_receive {
           $employee
           <tr>
         <th align=right nowrap>| . $locale->text('Order Number') . qq|</th>
-        <td>$form->{ordnumber}</td>
-        <input type=hidden name=ordnumber value="$form->{ordnumber}">
+        <td>$form->{ordnumber}
+        <input type=hidden name=ordnumber value="$form->{ordnumber}"></td>
           </tr>
           <tr>
         <th align=right nowrap>| . $locale->text('Order Date') . qq|</th>
-        <td>$form->{transdate}</td>
-        <input type=hidden name=transdate value=$form->{transdate}>
+        <td>$form->{transdate}
+        <input type=hidden name=transdate value=$form->{transdate}></td>
           </tr>
           <tr>
         <th align=right nowrap>| . ($form->{type} =~ /purchase_/ ?
             $locale->text('SO Number') : $locale->text('PO Number')) . qq|</th>
-        <td>$form->{ponumber}</td>
-        <input type=hidden name=ponumber value="$form->{ponumber}">
+        <td>$form->{ponumber}
+        <input type=hidden name=ponumber value="$form->{ponumber}"></td>
           </tr>
           <tr>
         <th align=right nowrap>$shipped</th>
@@ -1903,11 +1998,13 @@ qq|<td><input data-dojo-type="dijit/form/TextBox" id="serialnumber_$i" name="ser
 
         for (@column_index) { print "\n$column_data{$_}" }
 
-        print qq|
-        </tr>
+        print q|
+        <td style="display:none">
 |;
         $form->hide_form( "orderitems_id_$i", "id_$i", "partsgroup_$i" );
-
+        print q|
+        </tr>
+|;
     }
 
     print qq|
@@ -2014,7 +2111,7 @@ sub search_transfer {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 
 <table width=100%>
   <tr>
@@ -2055,7 +2152,7 @@ sub search_transfer {
 <br>
 <input type=hidden name=nextsub value=list_transfer>
 
-<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-continue" name="action" value="continue">|
+<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-continue" name="__action" value="continue">|
       . $locale->text('Continue')
       . qq|</button>|;
 
@@ -2080,7 +2177,7 @@ sub list_transfer {
     OE->get_inventory( \%myconfig, \%$form );
 
     # construct href
-    $href = "$form->{script}?action=list_transfer";
+    $href = "$form->{script}?__action=list_transfer";
     for (qw(direction oldsort path login sessionid)) {
         $href .= "&$_=$form->{$_}";
     }
@@ -2091,7 +2188,7 @@ sub list_transfer {
     $form->sort_order();
 
     # construct callback
-    $callback = "$form->{script}?action=list_transfer";
+    $callback = "$form->{script}?__action=list_transfer";
     for (qw(direction oldsort path login sessionid)) {
         $callback .= "&$_=$form->{$_}";
     }
@@ -2161,7 +2258,7 @@ sub list_transfer {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 
 <input type=hidden name=warehouse_id value=$warehouse_id>
 
@@ -2236,11 +2333,11 @@ qq|<td><input type=hidden name="warehouse_id_$i" value="$ref->{warehouse_id}">$r
 <input type=hidden name=rowcount value=$i>
 |;
 
-    $form->{action} = "transfer";
-    $form->hide_form(qw(path login sessionid action));
+    $form->{__action} = "transfer";
+    $form->hide_form(qw(path login sessionid));
 
     print qq|
-<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-transfer" name="action" value="transfer">|
+<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-transfer" name="__action" value="transfer">|
       . $locale->text('Transfer')
       . qq|</button>|;
 
@@ -2289,6 +2386,7 @@ sub generate_purchase_orders {
 
     # flatten array
     $i = 0;
+    $form->{_setting_decimal_places} = LedgerSMB::Setting->new(%$form)->get('decimal_places');
     foreach my $parts_id (
         sort {
             $form->{orderitems}{$a}{partnumber}
@@ -2311,7 +2409,7 @@ sub generate_purchase_orders {
 
         $form->{"lastcost_$i"} =
           $form->format_amount( \%myconfig,
-            $form->{orderitems}{$parts_id}{lastcost}, 2 );
+            $form->{orderitems}{$parts_id}{lastcost}, $form->{_setting_decimal_places} );
 
         $form->{"qty_$i"} = $required;
 
@@ -2344,8 +2442,8 @@ sub generate_purchase_orders {
                 $form->{"lastcost_$i"} = $form->format_amount(
                     \%myconfig,
                     $form->{orderitems}{$parts_id}{"parts$form->{vc}"}{$id}
-                      {lastcost},
-                    2
+                      {lastcost}, $form->{_setting_decimal_places}
+
                 );
                 $form->{"leadtime_$i"} = $form->format_amount( \%myconfig,
                     $form->{orderitems}{$parts_id}{"parts$form->{vc}"}{$id}
@@ -2356,8 +2454,8 @@ sub generate_purchase_orders {
                       {lastcost} * $form->{
                         $form->{orderitems}{$parts_id}{"parts$form->{vc}"}{$id}
                           {curr}
-                      },
-                    2
+                    },
+                    $form->{_setting_decimal_places}
                 );
 
                 $form->{"id_$i"} = $parts_id;
@@ -2410,7 +2508,7 @@ sub po_orderitems {
     print qq|
 <body class="lsmb">
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 
 <table width=100%>
   <tr>
@@ -2497,12 +2595,12 @@ qq|<td><input name="ndx_$i" id="ndx_$i" class=checkbox type=checkbox data-dojo-t
     );
 
     print qq|
-<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-generate-orders" name="action" value="generate_orders">|
+<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-generate-orders" name="__action" value="generate_orders">|
       . $locale->text('Generate Orders')
       . qq|</button>|;
 
     print qq|
-<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-select-vendor" name="action" value="select_vendor">|
+<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-select-vendor" name="__action" value="select_vendor">|
       . $locale->text('Select Vendor')
       . qq|</button>|;
 
@@ -2539,19 +2637,19 @@ sub select_vendor {
     print qq|
 <body class="lsmb $form->{dojo_theme}" onload="document.forms[0].vendor.focus()" />
 
-<form method="post" data-dojo-type="lsmb/Form" action=$form->{script}>
+<form method="post" data-dojo-type="lsmb/Form" action="$form->{script}">
 
 <b>| . $locale->text('Vendor') . qq|</b> <input data-dojo-type="dijit/form/TextBox" id=vendor name=vendor size=40>
 
 |;
 
     $form->{nextsub} = "vendor_selected";
-    $form->{action}  = "vendor_selected";
+    $form->{__action}  = "vendor_selected";
 
     $form->hide_form;
 
     print qq|
-<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-continue" name="action" value="continue">|
+<button data-dojo-type="dijit/form/Button" class="submit" type="submit" id="action-continue" name="__action" value="continue">|
       . $locale->text('Continue')
       . qq|</button>
 
