@@ -27,13 +27,12 @@ replacement is available.
 
 
 package AA;
-use LedgerSMB::Sysconfig;
-use Log::Log4perl;
+use Log::Any;
 use LedgerSMB::File;
 use LedgerSMB::PGNumber;
 use LedgerSMB::Setting;
 
-my $logger = Log::Log4perl->get_logger("AA");
+my $logger = Log::Any->get_logger(category => "AA");
 
 =pod
 
@@ -66,9 +65,6 @@ sub post_transaction {
     my $batch_class;
     my %paid;
     my $paidamount;
-    if ($form->{separate_duties}){
-        $form->{approved} = '0';
-    }
     for (1 .. $form->{rowcount}){
         $form->{"amount_$_"} = $form->parse_amount(
                $myconfig, $form->{"amount_$_"}
@@ -78,7 +74,6 @@ sub post_transaction {
 
     $form->{crdate} ||= 'now';
 
-    # connect to database
     my $dbh = $form->{dbh};
 
     my $query;
@@ -139,11 +134,12 @@ sub post_transaction {
 
             push @{ $form->{acc_trans}{taxes} },
               {
-                accno          => $accno,
-            amount_bc      => $tax{amount}{$accno},
-            amount_tc      => $tax{fxamount}{$accno},
-            curr           => $form->{currency},
-                project_id     => undef,
+                  accno          => $accno,
+                  source         => $form->{"taxsource_$accno"},
+                  amount_bc      => $tax{amount}{$accno},
+                  amount_tc      => $tax{fxamount}{$accno},
+                  curr           => $form->{currency},
+                  project_id     => undef,
               };
 
     }
@@ -268,13 +264,9 @@ sub post_transaction {
       ? $invamount
       : $form->round_amount( $paid * $form->{exchangerate}, 2 );
 
-    $query = q|
-        SELECT (SELECT value FROM defaults
-                 WHERE setting_key = 'fxgain_accno_id'),
-               (SELECT value FROM defaults
-                 WHERE setting_key = 'fxloss_accno_id')|;
-
-    my ( $fxgain_accno_id, $fxloss_accno_id ) = $dbh->selectrow_array($query);
+    my $setting = LedgerSMB::Setting->new(%$form);
+    $form->{$_} = $setting->get($_)
+        for (qw/ fxgain_accno_id fxloss_accno_id /);
 
     #tshvr4 trunk svn-revison 6589,$form->login seems to contain id instead of name or '',so person_id not found,thus reports with join on person_id not working,quick fix,use employee_name
     #( $null, $form->{employee_id} ) = split /--/, $form->{employee};
@@ -330,8 +322,7 @@ sub post_transaction {
          person_id, entity_credit_account, approved,
          setting_sequence
         )
-      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?,
-              (SELECT u.entity_id FROM users u WHERE u.username = ?), ?, ?, ?)
+      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
       RETURNING id
     |;
    }
@@ -343,8 +334,7 @@ sub post_transaction {
          notes, intnotes, ponumber, crdate, reverse,
          person_id, entity_credit_account, approved
         )
-      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?,
-              (SELECT u.entity_id FROM users u WHERE u.username = ?), ?, ?)
+      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
       RETURNING id
     |;
    }
@@ -352,6 +342,8 @@ sub post_transaction {
     $form->{invnumber} = undef if $form->{invnumber} eq '';
     $form->{datepaid} = $form->{transdate} unless $form->{datepaid};
     my $datepaid = ($paid) ? qq|'$form->{datepaid}'| : undef;
+    my $approved = 1;
+    $approved = 0 if $form->get_setting('separate_duties');
 
     my @queryargs = (
         $form->{invnumber},        $form->{description},
@@ -363,16 +355,21 @@ sub post_transaction {
         $form->{duedate},
         $form->{notes},            $form->{intnotes},
         $form->{ponumber},         $form->{crdate},
-        $form->{reverse},          $form->{employee_name},
-        $form->{"$form->{vc}_id"}, $form->{approved}
+        $form->{reverse},          $form->{employee_id},
+        $form->{"$form->{vc}_id"},
+        $approved
         );
-   if ($table eq 'ar') {
-       push @queryargs, $form->{setting_sequence}
-   }
+    if ($table eq 'ar') {
+        push @queryargs, $form->{setting_sequence}
+    }
 
-   $sth = $dbh->prepare($query) or $form->dberror($query);
-   $sth->execute(@queryargs) or $form->dberror($query);
-   ($form->{id}) = $sth->fetchrow_array() or $form->dberror($query);
+    $sth = $dbh->prepare($query) or $form->dberror($query);
+    $sth->execute(@queryargs) or $form->dberror($query);
+    ($form->{id}) = $sth->fetchrow_array() or $form->dberror($query);
+    $query = q|UPDATE transactions SET workflow_id = ?, reversing = ? WHERE id = ? AND workflow_id IS NULL|;
+    $sth   = $dbh->prepare($query);
+    $sth->execute( $form->{workflow_id}, $form->{reversing}, $form->{id} )
+        || $form->dberror($query);
 
     if (defined $form->{approved}) {
         if (!$form->{approved} && $form->{batch_id}){
@@ -414,16 +411,17 @@ sub post_transaction {
             $query = qq|
                 INSERT INTO acc_trans
                         (trans_id, chart_id, amount_bc, curr, amount_tc,
-                        transdate, memo, cleared)
+                        transdate, approved, memo, cleared)
                 VALUES  (?, (SELECT id FROM account
                                   WHERE accno = ?),
-                         ?, ?, ?, ?, ?, ?)|;
+                         ?, ?, ?, ?, ?, ?, ?)|;
 
             @queryargs = (
                 $form->{id},            $ref->{accno},
                 $ref->{amount_bc} * $ml, $ref->{curr},
                 $ref->{amount_tc} * $ml,
                 $form->{transdate},
+                $approved,
                 $ref->{description},
                 $ref->{cleared}
             );
@@ -460,15 +458,17 @@ sub post_transaction {
             $query = qq|
                 INSERT INTO acc_trans
                         (trans_id, chart_id, amount_bc, curr, amount_tc,
-                            transdate)
+                            transdate, approved, source)
                      VALUES (?, (SELECT id FROM account
                               WHERE accno = ?),
-                        ?, ?, ?, ?)|;
+                        ?, ?, ?, ?, ?, ?)|;
 
             @queryargs = (
                 $form->{id}, $ref->{accno}, $ref->{amount_bc} * $ml,
                 $form->{currency}, $ref->{amount_tc} * $ml,
-                $form->{transdate}
+                $form->{transdate},
+                $approved,
+                $ref->{source}
             );
             $dbh->prepare($query)->execute(@queryargs)
               || $form->dberror($query);
@@ -482,15 +482,18 @@ sub post_transaction {
         ($accno) = split /--/, $form->{$ARAP};
         $query = qq|
             INSERT INTO acc_trans
-                     (trans_id, chart_id, amount_bc, curr, amount_tc, transdate)
+                     (trans_id, chart_id, amount_bc, curr, amount_tc,
+                      transdate, approved)
               VALUES (?, (SELECT id FROM account
                               WHERE accno = ?),
-                           ?, ?, ?, ?)|;
+                           ?, ?, ?, ?, ?)|;
         @queryargs =
             ( $form->{id}, $accno,
               $invamount * -1 * $ml, $form->{currency},
               $invamount * -1 * $ml / $form->{exchangerate},
-            $form->{transdate} );
+              $form->{transdate},
+              $approved,
+            );
 
         $dbh->prepare($query)->execute(@queryargs)
           || $form->dberror($query);
@@ -509,9 +512,9 @@ sub post_transaction {
 
     IIAA->process_form_payments($myconfig, $form);
 
-    # save printed and queued
+    # save printed
     $form->save_status($dbh);
-    return 1 unless $DBI::errstr;
+    return 1 unless $dbh->errstr;
 }
 
 =item get_files
@@ -579,7 +582,6 @@ sub get_name {
         $form->{vc} = 'customer';
     }
 
-    # grab the db connection
     my $dbh = $form->{dbh};
 
     my $dateformat = $myconfig->{dateformat};
@@ -589,11 +591,12 @@ sub get_name {
         $dateformat .= "yy" if ( length $a[2] > 2 );
     }
 
-    if ( $form->{transdate} !~ /\W/ ) {
+    if ( defined $from->{transdate} and $form->{transdate} !~ /\W/ ) {
         $dateformat = 'yyyymmdd';
     }
 
-    if ( $form->{transdate} =~ m/\d\d\d\d-\d\d-\d\d/ ) {
+    if ( defined $form->{transdate}
+         and $form->{transdate} =~ m/\d\d\d\d-\d\d-\d\d/ ) {
         $dateformat = 'yyyy-mm-dd';
     }
 
@@ -622,26 +625,35 @@ sub get_name {
               entity.control_code AS entity_control_code,
                           co.tax_id AS tax_id,
               c.meta_number, ctf.default_reportable,
+              aa.accno || '--' || aa.description as arap_accno,
                           c.cash_account_id, ca.accno as cash_accno,
                           c.id as eca_id,
                           coalesce(ecl.address, el.address) as address,
-                          coalesce(ecl.city, el.city) as city
+                          coalesce(ecl.city, el.city) as city,
+                          coalesce(ecl.zipcode, el.zipcode) as zipcode,
+                          coalesce(ecl.state, el.state) as state,
+                          coalesce(ecl.country, el.country) as country
              FROM entity_credit_account c
              JOIN entity ON (entity.id = c.entity_id)
                 LEFT JOIN account ca ON c.cash_account_id = ca.id
+                LEFT JOIN account aa ON c.ar_ap_account_id = aa.id
         LEFT JOIN business b ON (b.id = c.business_id)
                 LEFT JOIN country_tax_form ctf ON ctf.id = c.taxform_id
                 LEFT JOIN company co ON co.entity_id = c.entity_id
                 LEFT JOIN (SELECT coalesce(line_one, '')
                                || ' ' || coalesce(line_two, '') as address,
-                               l.city, etl.credit_id
+                               l.city, etl.credit_id, mail_code as zipcode,
+                               state, (select short_name from country
+                                        where id=l.country_id) as country
                           FROM eca_to_location etl
                           JOIN location l ON etl.location_id = l.id
                           WHERE etl.location_class = 1) ecl
                         ON (c.id = ecl.credit_id)
                 LEFT JOIN (SELECT coalesce(line_one, '')
                                || ' ' || coalesce(line_two, '') as address,
-                               l.city, etl.entity_id
+                               l.city, etl.entity_id, mail_code as zipcode,
+                               state, (select short_name from country
+                                        where id=l.country_id) as country
                           FROM entity_to_location etl
                           JOIN location l ON etl.location_id = l.id
                           WHERE etl.location_class = 1) el
@@ -650,9 +662,10 @@ sub get_name {
 
     @queryargs = ( $form->{"$form->{vc}_id"} );
     my $sth = $dbh->prepare($query);
-
     $sth->execute(@queryargs) || $form->dberror($query);
     $ref = $sth->fetchrow_hashref(NAME_lc);
+    $form->{$form->{ARAP}} = $ref->{arap_accno};
+    delete $ref->{arap_accno};
     $form->db_parse_numeric(sth => $sth, hashref => $ref);
     if ( $form->{id} ) {
         for (qw(currency employee employee_id intnotes)) {
@@ -921,6 +934,28 @@ sub save_intnotes {
     $sth->execute($form->{intnotes}, $form->{id});
 }
 
+=item save_employee($form)
+
+Saves the $form->{employee} into the ar/ap.person_id field.
+
+=cut
+
+sub save_employee {
+    my ($self,$form) = @_;
+    my $table;
+    if ($form->{arap} eq 'ar') {
+        $table = 'ar';
+    } elsif ($form->{arap} eq 'ap') {
+        $table = 'ap';
+    } else {
+        $form->error('Bad arap in save_employee');
+    }
+    my $sth = $form->{dbh}->prepare("UPDATE $table SET person_id = ? " .
+                                    "where id = ?");
+    my ($name, $person_id) = split(/--/, $form->{employee}, 2);
+    $sth->execute($person_id, $form->{id});
+}
+
 =back
 
 =head1 COPTYRIGHT
