@@ -40,13 +40,14 @@ LedgerSMB::IR - Inventory received module
 package IR;
 use LedgerSMB::Tax;
 use LedgerSMB::PriceMatrix;
-use LedgerSMB::Sysconfig;
 use LedgerSMB::Setting;
 use LedgerSMB::PGNumber;
 use LedgerSMB::IIAA;
 
 use LedgerSMB::Magic qw(BC_VENDOR_INVOICE);
 
+use Workflow::Context;
+
 =over
 
 =item get_files
@@ -102,7 +103,6 @@ sub post_invoice {
     my $sth;
     my $ref;
     my $null;
-    my $project_id;
     my $exchangerate = 0;
     my $taxrate;
     my $diff = 0;
@@ -110,6 +110,9 @@ sub post_invoice {
     my $invoice_id;
     my $fxdiff = 0;
 
+    $form->{acc_trans} = ();
+
+
     ( $null, $form->{employee_id} ) = split /--/, $form->{employee}
         if $form->{employee};
 
@@ -152,8 +155,10 @@ sub post_invoice {
         $sth->execute || $form->dberror($query);
 
         ( $form->{id} ) = $sth->fetchrow_array;
-        $sth->finish;
-
+        $query = q|UPDATE transactions SET workflow_id = ?, reversing = ? WHERE id = ? AND workflow_id IS NULL|;
+        $sth   = $dbh->prepare($query);
+        $sth->execute( $form->{workflow_id}, $form->{reversing}, $form->{id} )
+            || $form->dberror($query);
     }
 
     my $amount;
@@ -195,11 +200,6 @@ sub post_invoice {
             }
             $pth->finish;
 
-            if ( ($form->{"projectnumber_$i"} // "") ne "" ) {
-                ( $null, $project_id ) =
-                  split /--/, $form->{"projectnumber_$i"};
-            }
-
             # undo discount formatting
             $form->{"discount_$i"} =
               $form->parse_amount( $myconfig, $form->{"discount_$i"} ) / 100;
@@ -355,7 +355,6 @@ sub post_invoice {
                     fxgrossamount => $fxlinetotal +
                       $form->round_amount( $fxtax, 2 ),
                     grossamount => $grossamount,
-                    project_id  => $project_id,
                     invoice_id  => $invoice_id
                   };
 
@@ -380,7 +379,6 @@ sub post_invoice {
                     fxgrossamount => $fxlinetotal +
                       $form->round_amount( $fxtax, 2 ),
                     grossamount => $grossamount,
-                    project_id  => $project_id,
                     invoice_id  => $invoice_id
                   };
 
@@ -438,6 +436,10 @@ sub post_invoice {
           $form->round_amount( $form->{paid} * $form->{exchangerate}, 2 );
     }
 
+    my $approved = 1;
+    $approved = 0 if $form->get_setting('separate_duties');
+
+
     foreach my $ref ( sort { $b->{amount} <=> $a->{amount} }
         @{ $form->{acc_trans}{lineitems} } )
     {
@@ -446,13 +448,13 @@ sub post_invoice {
         $fxlinetotal = $ref->{fxlinetotal} + $diff/$form->{exchangerate};
         $query  = qq|
          INSERT INTO acc_trans (trans_id, chart_id, amount_bc, curr, amount_tc,
-                     transdate, invoice_id)
-                     VALUES (?, ?, ?, ?, ?, ?, ?)|;
+                     transdate, approved, invoice_id)
+                     VALUES (?, ?, ?, ?, ?, ?, ?, ?)|;
         $sth = $dbh->prepare($query);
         $sth->execute(
             $form->{id},        $ref->{chart_id},  $amount * -1,
             $form->{currency},  $fxlinetotal * -1, $form->{transdate},
-            $ref->{invoice_id}
+            $approved, $ref->{invoice_id}
         ) || $form->dberror($query);
 
         $diff   = 0;
@@ -481,19 +483,19 @@ sub post_invoice {
 
         $query = qq|
           INSERT INTO acc_trans (trans_id, chart_id,
-                                amount_bc, curr, amount_tc, transdate)
+                                amount_bc, curr, amount_tc, transdate, approved)
                        VALUES (?, (SELECT id FROM account WHERE accno = ?),
-                              ?, ?, ?, ?)|;
+                              ?, ?, ?, ?, ?)|;
         $sth = $dbh->prepare($query)
             or $form->dberror($dbh->errstr);
          $sth->execute( $form->{id}, $accno,
                        $form->{payables}, $form->{currency},
                     $form->{payables}/$form->{exchangerate},
-                       $form->{transdate})
+                       $form->{transdate}, $approved)
           || $form->dberror($query);
     }
 
-    # post taxes, if !$form->{manual} (see above)    
+    # post taxes, if !$form->{manual} (see above)
     foreach my $trans_id ( keys %{ $form->{acc_trans} } ) {
         foreach my $accno ( keys %{ $form->{acc_trans}{$trans_id} } ) {
             $amount =
@@ -504,15 +506,15 @@ sub post_invoice {
                 $query = qq|
                     INSERT INTO acc_trans
                            (trans_id, chart_id, amount_bc, curr, amount_tc,
-                                transdate)
+                                transdate, approved)
                             VALUES (?, (SELECT id FROM account
                                 WHERE accno = ?),
-                           ?, ?, ?, ?)|;
+                           ?, ?, ?, ?, ?)|;
                 $sth = $dbh->prepare($query)
                     || $form->dberror($dbh->errstr);
                 $sth->execute( $trans_id, $accno,
                                $amount, $form->{defaultcurrency}, $amount,
-                               $form->{transdate} )
+                               $form->{transdate}, $approved )
                   || $form->dberror($query);
             }
         }
@@ -531,9 +533,6 @@ sub post_invoice {
     $form->{taxincluded} //= 0;
     $form->{taxincluded} *= 1;
 
-    my $approved = 1;
-    $approved = 0 if $form->{separate_duties};
-
     # save AP record
     $query = qq|
         UPDATE ap
@@ -588,7 +587,7 @@ sub post_invoice {
         if $form->{vendor} && $form->{vendor_id};
     $form->add_shipto($form->{id});
 
-    if (!$form->{separate_duties}){
+    if ($form->get_setting('separate_duties')){
         $self->add_cogs($form);
     }
 
@@ -608,7 +607,6 @@ sub post_invoice {
 sub retrieve_invoice {
     my ( $self, $myconfig, $form ) = @_;
 
-    # connect to database
     my $dbh = $form->{dbh};
 
     my $query;
@@ -623,90 +621,25 @@ sub retrieve_invoice {
                         JOIN account c ON c.id = ac.chart_id
                        WHERE ac.trans_id = ?|);
         $tax_sth->execute($form->{id});
+        my $reverse = $form->{reverse} ? -1 : 1;
         while (my $taxref = $tax_sth->fetchrow_hashref('NAME_lc')){
               $form->db_parse_numeric(sth=>$tax_sth,hashref=>$taxref);
               $form->{manual_tax} = 1;
               my $taccno = $taxref->{accno};
-              $form->{"mt_amount_$taccno"} = LedgerSMB::PGNumber->new($taxref->{amount} * -1);
+              $form->{"mt_amount_$taccno"} =
+                  LedgerSMB::PGNumber->new($taxref->{amount} * -1 * $reverse);
               $form->{"mt_rate_$taccno"}  = $taxref->{rate};
-              $form->{"mt_basis_$taccno"} = LedgerSMB::PGNumber->new($taxref->{tax_basis} * -1);
+              $form->{"mt_basis_$taccno"} =
+                  LedgerSMB::PGNumber->new($taxref->{tax_basis} * -1 * $reverse);
               $form->{"mt_memo_$taccno"}  = $taxref->{memo};
               $form->{"mt_ref_$taccno"}  = $taxref->{source};
         }
-
-        # get default accounts and last invoice number
-        $query = qq|
-            SELECT (select c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'inventory_accno_id'))
-                   AS inventory_accno,
-
-                   (SELECT c.accno FROM account c
-                 WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'income_accno_id'))
-                   AS income_accno,
-
-                   (SELECT c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'expense_accno_id'))
-                   AS expense_accno,
-
-                   (SELECT c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'fxgain_accno_id'))
-                   AS fxgain_accno,
-
-                   (SELECT c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'fxloss_accno_id'))
-                AS fxloss_accno|;
     }
-    else {
-        $query = qq|
-            SELECT (select c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'inventory_accno_id'))
-                   AS inventory_accno,
 
-                   (SELECT c.accno FROM account c
-                 WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'income_accno_id'))
-                   AS income_accno,
-
-                   (SELECT c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'expense_accno_id'))
-                   AS expense_accno,
-
-                   (SELECT c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'fxgain_accno_id'))
-                   AS fxgain_accno,
-
-                   (SELECT c.accno FROM account c
-                     WHERE c.id = (SELECT value::int FROM defaults
-                                    WHERE setting_key =
-                                          'fxloss_accno_id'))
-                   AS fxloss_accno
-|;
-    }
-    my $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-
-    my $ref = $sth->fetchrow_hashref(NAME_lc);
-    for ( keys %$ref ) {
-        $form->{$_} = $ref->{$_};
-    }
-    $sth->finish;
+    my $setting = LedgerSMB::Setting->new(%$form);
+    $form->{$_} = $setting->get($_)
+        for (qw/ inventory_accno_id income_accno_id
+                 fxgain_accno_id fxloss_accno_id /);
     @{$form->{currencies}} =
         (LedgerSMB::Setting->new(%$form))->get_currencies;
 
@@ -716,9 +649,11 @@ sub retrieve_invoice {
             SELECT a.invnumber, a.transdate, a.duedate,
                    a.ordnumber, a.quonumber, a.taxincluded,
                    a.notes, a.intnotes, a.curr AS currency,
-                   a.entity_credit_account as vendor_id, a.language_code, a.ponumber, a.crdate,
-                   a.on_hold, a.reverse, a.description
+                   a.entity_credit_account as vendor_id, a.language_code,
+                   a.ponumber, a.crdate, a.on_hold, a.reverse, a.description,
+                   tran.workflow_id
               FROM ap a
+              JOIN transactions tran USING (id)
              WHERE id = ?|;
         $sth = $dbh->prepare($query);
         $sth->execute( $form->{id} ) || $form->dberror($query);
@@ -759,7 +694,7 @@ sub retrieve_invoice {
                  FROM invoice i
                  JOIN parts p ON (i.parts_id = p.id)
             LEFT JOIN partsgroup pg ON (pg.id = p.partsgroup_id)
-            LEFT JOIN translation t
+            LEFT JOIN partsgroup_translation t
                       ON (t.trans_id = p.partsgroup_id
                       AND t.language_code = ?)
                 WHERE i.trans_id = ?
@@ -875,9 +810,9 @@ sub retrieve_item {
         LEFT JOIN partsgroup pg ON (pg.id = p.partsgroup_id)
                 LEFT JOIN partsvendor pv ON (pv.parts_id = p.id
                                            AND pv.credit_id = ?)
-        LEFT JOIN translation t1
+        LEFT JOIN parts_translation t1
                   ON (t1.trans_id = p.id AND t1.language_code = ?)
-        LEFT JOIN translation t2
+        LEFT JOIN partsgroup_translation t2
                   ON (t2.trans_id = p.partsgroup_id
                   AND t2.language_code = ?)
              $where
@@ -943,7 +878,6 @@ sub retrieve_item {
 sub vendor_details {
     my ( $self, $myconfig, $form ) = @_;
 
-    # connect to database
     my $dbh = $form->{dbh};
 
     # get rest for the vendor
@@ -991,11 +925,10 @@ sub vendor_details {
 sub item_links {
     my ( $self, $myconfig, $form ) = @_;
 
-    # connect to database
     my $dbh = $form->{dbh};
 
     my $query = qq|
-           SELECT accno, description, as_array(l.description) as link
+           SELECT accno, description, array_agg(l.description) as link
              FROM account a
              JOIN account_link l ON a.id = l.account-id
             WHERE l.description like 'IC%'
