@@ -1,6 +1,5 @@
 =head1 NAME
 
-            ($form->{"$partnumber_$i"}) = split(/--/, $form->{"$partnumber_$i"});
 LedgerSMB::IS - Inventory Invoicing
 
 =cut
@@ -40,16 +39,16 @@ LedgerSMB::IS - Inventory Invoicing
 package IS;
 use LedgerSMB::Tax;
 use LedgerSMB::PriceMatrix;
-use LedgerSMB::Sysconfig;
 use LedgerSMB::Setting;
 use LedgerSMB::Num2text;
 use LedgerSMB::IIAA;
-use Log::Log4perl;
+use Log::Any;
 
 use LedgerSMB::Magic qw(BC_SALES_INVOICE);
 
+use Workflow::Context;
 
-my $logger = Log::Log4perl->get_logger('LedgerSMB::IS');
+my $logger = Log::Any->get_logger(category => 'LedgerSMB::IS');
 
 
 sub add_cogs {
@@ -88,22 +87,20 @@ sub invoice_details {
 
     $form->{duedate} = $form->{transdate} unless ( $form->{duedate} );
 
-    # connect to database
 
     my $dbh = $form->{dbh};
 
     my $query = qq|
         SELECT ?::date - ?::date
-                       AS terms, value
-          FROM defaults
-         WHERE setting_key = 'weightunit'|;
+                       AS terms|;
     my $sth = $dbh->prepare($query);
     $sth->execute( $form->{duedate}, $form->{transdate} )
       || $form->dberror($query);
 
-    ( $form->{terms}, $form->{weightunit} ) = $sth->fetchrow_array;
+    ( $form->{terms} ) = $sth->fetchrow_array;
     $sth->finish;
 
+    $form->{weightunit} = $form->get_setting( 'weightunit' );
     # this is for the template
     $form->{invdate} = $form->{transdate};
 
@@ -111,24 +108,11 @@ sub invoice_details {
     my $item;
     my $i;
     my @sortlist = ();
-    my $projectnumber;
-    my $projectdescription;
-    my $projectnumber_id;
-    my $translation;
     my @taxaccounts;
     my %taxaccounts;
     my $taxrate;
     my $taxamount;
 
-    $query = qq|
-           SELECT p.description, t.description
-             FROM project p
-        LEFT JOIN translation t
-                  ON (t.trans_id = p.id
-                  AND t.language_code = ?)
-            WHERE id = ?|;
-    my $prh = $dbh->prepare($query) || $form->dberror($query);
-
     $query = qq|
         SELECT inventory_accno_id, income_accno_id,
                expense_accno_id, assembly, weight FROM parts
@@ -147,55 +131,17 @@ sub invoice_details {
         for ( keys %$ref ) { $form->{"${_}_$i"} = $ref->{$_} }
         $pth->finish;
 
-        $projectnumber_id      = 0;
-        $projectnumber         = "";
         $form->{partsgroup}    = "";
         $form->{projectnumber} = "";
 
-        if ( $form->{groupprojectnumber} || $form->{grouppartsgroup} ) {
+        if ( $form->{grouppartsgroup} ) {
 
             $inventory_accno_id =
               ( $form->{"inventory_accno_id_$i"} || $form->{"assembly_$i"} )
               ? "1"
               : "";
 
-            if ( $form->{groupprojectnumber} ) {
-                ( $projectnumber, $projectnumber_id ) =
-                  split /--/, $form->{"projectnumber_$i"};
-            }
-            if ( $form->{grouppartsgroup} ) {
-                ( $form->{partsgroup} ) =
-                  split /--/, $form->{"partsgroup_$i"};
-            }
-
-            if ( $projectnumber_id && $form->{groupprojectnumber} ) {
-                if ( $translation{$projectnumber_id} ) {
-                    $form->{projectnumber} = $translation{$projectnumber_id};
-                }
-                else {
-
-                    # get project description
-                    $prh->execute( $projectnumber_id, $form->{language_code} );
-
-                    ( $projectdescription, $translation ) =
-                      $prh->fetchrow_array;
-
-                    $prh->finish;
-
-                    $form->{projectnumber} =
-                      ($translation)
-                      ? "$projectnumber, $translation"
-                      : "$projectnumber, " . "$projectdescription";
-
-                    $translation{$projectnumber_id} = $form->{projectnumber};
-                }
-            }
-
-            if ( $form->{grouppartsgroup} && $form->{partsgroup} ) {
-                $form->{projectnumber} .= " / "
-                  if $projectnumber_id;
-                $form->{projectnumber} .= $form->{partsgroup};
-            }
+            ( $form->{partsgroup} ) = split /--/, $form->{"partsgroup_$i"};
         }
 
         $sortby = qq|$projectnumber$form->{partsgroup}|;
@@ -596,7 +542,7 @@ sub invoice_details {
             );
 
             push( @{ $form->{taxdescription} },
-                $form->{"${item}_description"} );
+                $form->{_accno_descriptions}->{$item} );
 
             $form->{"${item}_taxrate"} =
               $form->format_amount( $myconfig, $form->{"${item}_rate"} * 100 );
@@ -646,7 +592,7 @@ sub invoice_details {
     for (qw(totalparts totalservices)) {
         $form->{$_} = $form->format_amount( $myconfig, $form->{$_}, 2 );
     }
-    for (qw(totalqty totalship totalweight)) {
+    for (qw(totalqty totalship totalweight totalweightship)) {
         $form->{$_} = $form->format_amount( $myconfig, $form->{$_} );
     }
     $form->{subtotal} = $form->format_amount( $myconfig, $form->{total}, 2 );
@@ -764,6 +710,7 @@ sub post_invoice {
     my $project_id;
     my $exchangerate = 0;
     my $keepcleared  = 0;
+    my $transdate = $form->{transdate} || 'today';
 
     $form->{acc_trans} = ();
 
@@ -779,14 +726,9 @@ sub post_invoice {
     ( $null, $form->{department_id} ) = split( /--/, $form->{department} );
     $form->{department_id} *= 1;
 
-    $query = qq|
-        SELECT (SELECT value FROM defaults
-                 WHERE setting_key = 'fxgain_accno_id')
-               AS fxgain_accno_id,
-               (SELECT value FROM defaults
-                 WHERE setting_key = 'fxloss_accno_id')
-               AS fxloss_accno_id|;
-    my ( $fxgain_accno_id, $fxloss_accno_id ) = $dbh->selectrow_array($query);
+    my $setting = LedgerSMB::Setting->new( %$form );
+    my $fxgain_accno_id = $setting->get('fxgain_accno_id');
+    my $fxloss_accno_id = $setting->get('fxloss_accno_id');
 
     $query = qq|
         SELECT p.assembly, p.inventory_accno_id,
@@ -816,8 +758,11 @@ sub post_invoice {
         $sth->execute || $form->dberror($query);
 
         ( $form->{id} ) = $sth->fetchrow_array;
-        $sth->finish;
 
+        $query = q|UPDATE transactions SET workflow_id = ?, reversing = ? WHERE id = ? AND workflow_id IS NULL|;
+        $sth   = $dbh->prepare($query);
+        $sth->execute( $form->{workflow_id}, $form->{reversing}, $form->{id} )
+            || $form->dberror($query);
     }
 
     if ( $form->{currency} eq $form->{defaultcurrency} ) {
@@ -1118,6 +1063,10 @@ sub post_invoice {
           VALUES (currval('acc_trans_entry_id_seq'), ?, ?)"
     );
 
+    my $approved = 1;
+    $approved = 0 if $form->get_setting('separate_duties');
+
+
     foreach my $ref ( sort { $b->{amount} <=> $a->{amount} }
         @{ $form->{acc_trans}{lineitems} } )
     {
@@ -1129,12 +1078,12 @@ sub post_invoice {
         $query  = qq|
             INSERT INTO acc_trans
                      (trans_id, chart_id, amount_bc, curr, amount_tc,
-                     transdate, invoice_id)
-              VALUES (?, ?, ?, ?, ?, ?, ?)|;
+                      transdate, approved, invoice_id)
+              VALUES (?, ?, ?, ?, ?, ?, ?, ?)|;
         $sth = $dbh->prepare($query);
         $sth->execute( $form->{id}, $ref->{chart_id}, $amount,
                        $form->{currency}, $amount - $ref->{fxdiff},
-                       $form->{transdate}, $ref->{invoice_id})
+                       $transdate, $approved, $ref->{invoice_id})
           || $form->dberror($query);
         $diff   = 0;
         $fxdiff = 0;
@@ -1163,16 +1112,16 @@ sub post_invoice {
         $query = qq|
             INSERT INTO acc_trans
                      (trans_id, chart_id,
-                      amount_bc, curr, amount_tc, transdate)
+                      amount_bc, curr, amount_tc, transdate, approved)
                  VALUES (?, (SELECT id FROM account WHERE accno = ?),
-                     ?, ?, ?, ?)|;
+                     ?, ?, ?, ?, ?)|;
 
         $sth = $dbh->prepare($query)
             or $form->dberror($dbh->errstr);
         $sth->execute( $form->{id}, $accno,
                        $form->{receivables}, $form->{currency},
                $form->{receivables} / $form->{exchangerate},
-                       $form->{transdate})
+                       $transdate, $approved)
             || $form->dberror($query);
     }
 
@@ -1187,16 +1136,17 @@ sub post_invoice {
                 $query = qq|
                     INSERT INTO acc_trans
                            (trans_id, chart_id, amount_bc, curr, amount_tc,
-                                transdate)
+                                transdate, approved, source)
                     VALUES (?, (SELECT id FROM account
                                  WHERE accno = ?),
-                      ?, ?, ?, ?)|;
+                      ?, ?, ?, ?, ?, ?)|;
 
                 $sth = $dbh->prepare($query)
                     || $form->dberror($dbh->errstr);
                 $sth->execute( $trans_id, $accno,
                                $amount, $form->{defaultcurrency}, $amount,
-                               $form->{transdate} )
+                               $transdate, $approved,
+                               $form->{acc_trans}{$trans_id}{$accno}{source} )
                   || $form->dberror($query);
             }
         }
@@ -1216,8 +1166,6 @@ sub post_invoice {
     $form->{taxincluded} *= 1;
 
 
-    my $approved = 1;
-    $approved = 0 if $form->{separate_duties};
     # save AR record
     $query = qq|
         UPDATE ar set
@@ -1241,7 +1189,6 @@ sub post_invoice {
                taxincluded = ?,
                curr = ?,
                person_id = ?,
-               till = ?,
                language_code = ?,
                ponumber = ?,
                        approved = ?,
@@ -1255,7 +1202,7 @@ sub post_invoice {
     $sth->execute(
         $form->{invnumber},     $form->{ordnumber},
         $form->{quonumber},     $form->{description},
-        $form->{transdate} || 'now',
+        $transdate,
         $form->{customer_id},   $invamount,
         $invamount/$form->{exchangerate},
         $invnetamount,          $invnetamount/$form->{exchangerate},
@@ -1264,7 +1211,7 @@ sub post_invoice {
         $form->{terms},         $form->{notes},
         $form->{intnotes},      $form->{taxincluded},
         $form->{currency},
-        $form->{employee_id},   $form->{till},
+        $form->{employee_id},
         $form->{language_code}, $form->{ponumber}, $approved,
         $form->{crdate} || 'today', $form->{reverse},
         $form->{is_return},     $form->{setting_sequence},
@@ -1276,10 +1223,10 @@ sub post_invoice {
     $form->{name} =~ s/--$form->{customer_id}//;
     $form->add_shipto($form->{id});
 
-    # save printed, emailed and queued
+    # save printed, emailed
     $form->save_status($dbh);
 
-    if (!$form->{separate_duties}){
+    if ($form->get_setting('separate_duties')){
         $self->add_cogs($form);
     }
 
@@ -1307,11 +1254,13 @@ sub retrieve_invoice {
                       a.shippingpoint, a.shipvia, a.terms, a.notes,
                       a.intnotes,
                       a.duedate, a.taxincluded, a.curr AS currency,
-                      a.person_id as employee_id, e.name AS employee, a.till,
+                      a.person_id as employee_id, e.name AS employee,
                       a.reverse, a.entity_credit_account as customer_id,
                       a.language_code, a.ponumber, a.crdate,
-                      a.on_hold, a.description, a.setting_sequence
+                      a.on_hold, a.description, a.setting_sequence,
+                      tran.workflow_id
                  FROM ar a
+                 JOIN transactions tran USING (id)
             LEFT JOIN entity_employee em ON (em.entity_id = a.person_id)
             LEFT JOIN entity e ON e.id = em.entity_id
                 WHERE a.id = ?|;
@@ -1333,12 +1282,13 @@ sub retrieve_invoice {
                         JOIN account c ON c.id = ac.chart_id
                        WHERE ac.trans_id = ?|);
         $tax_sth->execute($form->{id});
+        my $reverse = $form->{reverse} ? -1 : 1;
         while (my $taxref = $tax_sth->fetchrow_hashref('NAME_lc')){
               $form->{manual_tax} = 1;
               my $taccno = $taxref->{accno};
-              $form->{"mt_amount_$taccno"} = $taxref->{amount};
+              $form->{"mt_amount_$taccno"} = $taxref->{amount} * $reverse;
               $form->{"mt_rate_$taccno"}  = $taxref->{rate};
-              $form->{"mt_basis_$taccno"} = $taxref->{tax_basis};
+              $form->{"mt_basis_$taccno"} = $taxref->{tax_basis} * $reverse;
               $form->{"mt_memo_$taccno"}  = $taxref->{memo};
               $form->{"mt_ref_$taccno"}  = $taxref->{source};
         }
@@ -1372,7 +1322,7 @@ sub retrieve_invoice {
                  FROM invoice i
                      JOIN parts p ON (i.parts_id = p.id)
             LEFT JOIN partsgroup pg ON (p.partsgroup_id = pg.id)
-            LEFT JOIN translation t
+            LEFT JOIN partsgroup_translation t
                       ON (t.trans_id = p.partsgroup_id
                       AND t.language_code
                       = ?)
@@ -1496,9 +1446,9 @@ sub retrieve_item {
                 LEFT JOIN makemodel mm ON (mm.parts_id = p.id AND mm.barcode = |
                              . $dbh->quote($form->{"partnumber_$i"}) . qq|)
         LEFT JOIN partsgroup pg ON (pg.id = p.partsgroup_id)
-        LEFT JOIN translation t1
+        LEFT JOIN parts_translation t1
                   ON (t1.trans_id = p.id AND t1.language_code = ?)
-        LEFT JOIN translation t2
+        LEFT JOIN partsgroup_translation t2
                   ON (t2.trans_id = p.partsgroup_id
                   AND t2.language_code = ?)
              $where
@@ -1586,210 +1536,6 @@ sub toggle_on_hold {
 }
 
 
-sub list_locations_contacts
-{
-
-    my ( $self, $myconfig, $form ) = @_;
-
-
-
-    my $dbh = $form->{dbh};
-
-    # get rest for the customer
-    my $query = qq|
-                    WITH eca AS (select * from entity_credit_account
-                                  where id = ?
-                    )
-            select  id as locationid,line_one as shiptoaddress1,line_two as shiptoaddress2,line_three as shiptoaddress3,city as shiptocity,
-                state as shiptostate,mail_code as shiptozipcode,country as shiptocountry
-            from (
-                            select (eca__list_locations(id)).*
-                              FROM eca
-                             UNION
-                            SELECT (entity__list_locations(entity_id)).*
-                              FROM eca
-                        ) l
-                         WHERE location_class = 3;
-          |;
-
-
-
-    my $sth = $dbh->prepare($query);
-
-    $sth->execute( $form->{customer_id} ) || $form->dberror($query);
-
-    my $i=0;
-
-    while($ref = $sth->fetchrow_hashref(NAME_lc))
-    {
-           $i++;
-           for ( keys %$ref )
-       {
-         $form->{"$_\_$i"} = $ref->{$_};
-       }
-
-    }
-
-    $form->{totallocations}=$i;
-
-    $sth->finish();
-
-
-
-
-
-
-    $query = qq|
-            select c.class as shiptotype,ec.contact as shiptocontact,ec.description as shiptodescription
-            from eca_to_contact ec
-            join contact_class c on(c.id=ec.contact_class_id)
-            where ec.credit_id=?;
-          |;
-
-
-
-    $sth = $dbh->prepare($query);
-
-    $sth->execute( $form->{customer_id} ) || $form->dberror($query);
-
-    $i=0;
-
-    while($ref = $sth->fetchrow_hashref(NAME_lc))
-    {
-           $i++;
-           for ( keys %$ref )
-       {
-         $form->{"$_\_$i"} = $ref->{$_};
-       }
-
-    }
-
-    $form->{totalcontacts}=$i;
-
-    $sth->finish();
-
-    # for ( keys %$ref ) { $form->{$_} = $ref->{$_} }
-}
-
-
-sub construct_countrys
-{
-
-
-    my ( $self,$form ) = @_;
-
-    my $dbh = $form->{dbh};
-
-
-    my $query = qq|
-            select id,short_name  from location_list_country();
-          |;
-
-
-
-    my $sth = $dbh->prepare($query);
-
-    $sth->execute() || $form->dberror($query);
-
-    my $returnvalue=qq|<option value="">null</option>|;
-
-    while(my $ref = $sth->fetchrow_hashref(NAME_lc))
-    {
-
-      $returnvalue.=qq|<option value="$ref->{id} ">$ref->{short_name}</option>|;
-
-    }
-
-
-   $sth->finish();
-
-   return($returnvalue);
-
-}
-
-
-sub construct_types
-{
-
-
-    my ( $self,$form ) = @_;
-
-    my $dbh = $form->{dbh};
-
-
-    my $query = qq|
-            select id,class from contact_class;
-          |;
-
-
-
-    my $sth = $dbh->prepare($query);
-
-    $sth->execute() || $form->dberror($query);
-
-    my $returnvalue=qq|<option value="">null</option>|;
-
-    while(my $ref = $sth->fetchrow_hashref(NAME_lc))
-    {
-
-      $returnvalue.=qq|<option value="$ref->{id} ">|.substr($ref->{class},0,3).qq|</option>|;  ## no critic (ProhibitMagicNumbers) sniff
-
-    }
-
-
-   $sth->finish();
-
-   return($returnvalue);
-
-}
-
-
-sub createlocation
-{
-
-
-  my ( $self,$form ) = @_;
-
-  my $dbh=$form->{dbh};
-
-  my $query="select * from eca__location_save(?,?,?,?,?,?,?,?,?,?, null);";
-
-  my $sth=$dbh->prepare("$query");
-
-   $sth->execute($form->{"customer_id"},
-         undef,
-         3,  ## no critic (ProhibitMagicNumbers) sniff
-         $form->{"shiptoaddress1_new"},
-         $form->{"shiptoaddress2_new"},
-         $form->{"shiptoaddress3_new"},
-         $form->{"shiptocity_new"},
-         $form->{"shiptostate_new"},
-         $form->{"shiptozipcode_new"},
-         $form->{"shiptocountry_new"}
-            ) || $form->dberror($query);
-  my ($l_id) = $sth->fetchrow_array;
-  $sth->finish();
-  return $l_id;
-}
-
-
-
-sub createcontact
-{
-
-  my ( $self,$form ) = @_;
-
-  my $dbh=$form->{dbh};
-
-  my $query="select * from eca__save_contact(?,?,?,?,?,?);";
-
-  my $sth=$dbh->prepare("$query");
-
-  $sth->execute($form->{"customer_id"},$form->{"shiptotype_new"},$form->{"shiptodescription_new"},$form->{"shiptocontact_new"},$form->{"shiptocontact_new"},$form->{"shiptotype_new"}) || $form->dberror($query);
-
-  $sth->finish();
-
-}
 
 
 
