@@ -39,15 +39,15 @@ LedgerSMB::OE - Order Entry
 #======================================================================
 
 package OE;
-use LedgerSMB::Tax;
-use LedgerSMB::Setting;
-use LedgerSMB::Sysconfig;
-use LedgerSMB::Num2text;
-use Log::Log4perl;
 
 use LedgerSMB::Magic qw(OEC_QUOTATION OEC_RFQ);
+use LedgerSMB::Num2text;
+use LedgerSMB::Setting;
+use LedgerSMB::Tax;
 
-my $logger = Log::Log4perl->get_logger('OE');
+use Log::Any;
+
+my $logger = Log::Any->get_logger(category => 'OE');
 
 =over
 
@@ -192,6 +192,11 @@ sub save {
     }
     my $did_insert = 0;
     if ( !$form->{id} ) {
+        if (! $form->{workflow_id}) {
+            my $wf = $form->{_wire}->get('workflows')
+                ->create_workflow( 'Order/Quote' );
+            $form->{workflow_id} = $wf->id;
+        }
         $query = qq|SELECT nextval('oe_id_seq')|;
         $sth   = $dbh->prepare($query) || $form->dberror($query);
         $sth->execute || $form->dberror($query);
@@ -215,13 +220,13 @@ sub save {
                 reqdate, shippingpoint, shipvia,
                 notes, intnotes, curr, closed,
                 person_id, language_code, ponumber, terms,
-                quotation, oe_class_id, entity_credit_account)
+                quotation, oe_class_id, entity_credit_account, workflow_id)
             VALUES
                 (?, ?, ?,
                  ?, ?, ?,
                  ?, ?, ?, ?, ?,
                  ?, ?, ?, ?,
-                 ?, ?, ?)|;
+                 ?, ?, ?, ?)|;
         @queryargs = (
             $form->{id},
             $form->{ordnumber},     $form->{quonumber},
@@ -231,7 +236,8 @@ sub save {
             $form->{currency},      $form->{closed},
             $form->{person_id},
             $form->{language_code}, $form->{ponumber},
-            $form->{terms},         $quotation, $class_id, $form->{"$form->{vc}_id"}
+            $form->{terms},         $quotation, $class_id, $form->{"$form->{vc}_id"},
+            $form->{workflow_id}
         );
         $sth = $dbh->prepare($query);
         $sth->execute(@queryargs) || $form->dberror($query);
@@ -411,7 +417,7 @@ sub save {
             UPDATE oe SET
                 amount_tc = ?,
                 netamount_tc = ?,
-                taxincluded = ?
+                taxincluded = ?,
             WHERE id = ?|;
         @queryargs = ( $amount, $netamount, $form->{taxincluded}, $form->{id} );
     }
@@ -468,8 +474,7 @@ sub save {
 
     $form->add_shipto($form->{id}, 1);
 
-    # save printed, emailed, queued
-
+    # save printed, emailed
     $form->save_status($dbh);
 
     if ( $form->{type} =~ /_order$/ ) {
@@ -487,31 +492,14 @@ sub save {
         action   => 'saved',
         id       => $form->{id}
     );
-
+    return 1;
 }
 
 sub delete {
     my ( $self, $myconfig, $form ) = @_;
 
-    # connect to database
     my $dbh = $form->{dbh};
 
-    # delete spool files
-    my $query = qq|
-        SELECT spoolfile FROM status
-        WHERE trans_id = ?
-        AND spoolfile IS NOT NULL|;
-    $sth = $dbh->prepare($query);
-    $sth->execute( $form->{id} ) || $form->dberror($query);
-
-    my $spoolfile;
-    my @spoolfiles = ();
-
-    while ( ($spoolfile) = $sth->fetchrow_array ) {
-        push @spoolfiles, $spoolfile;
-    }
-    $sth->finish;
-
     $query = qq|
         SELECT o.parts_id, o.ship, p.inventory_accno_id, p.assembly
         FROM orderitems o
@@ -551,9 +539,6 @@ sub delete {
     $sth->execute( $form->{id} ) || $form->dberror($query);
     $sth->finish;
 
-    foreach my $spoolfile (@spoolfiles) {
-        unlink "${LedgerSMB::Sysconfig::spool}/$spoolfile" if $spoolfile;
-    }
     return 1;
 }
 
@@ -561,7 +546,6 @@ sub retrieve {
     use LedgerSMB::PriceMatrix;
     my ( $self, $myconfig, $form ) = @_;
 
-    # connect to database
     my $dbh = $form->{dbh};
 
     my $bu_sth = $dbh->prepare(
@@ -577,18 +561,14 @@ sub retrieve {
     @{$form->{currencies}} =
         (LedgerSMB::Setting->new(%$form))->get_currencies;
     $form->{defaultcurrency} = $form->{currencies}->[0];
-
-    $query = qq|
-        SELECT value FROM defaults
-         WHERE setting_key = 'lock_description'|;
-    ( $form->{lock_description}) = $dbh->selectrow_array($query);
+    $form->{lock_description} = $form->get_setting( 'lock_description' );
 
 
     if ( $form->{id} ) {
 
         # retrieve order
         $query = qq|
-            SELECT o.ordnumber, o.transdate, o.reqdate, o.terms,
+            SELECT o.ordnumber, o.transdate, o.reqdate, o.terms, o.workflow_id,
                         o.taxincluded, o.shippingpoint, o.shipvia,
                 o.notes, o.intnotes, o.curr AS currency,
                 pe.first_name \|\| ' ' \|\| pe.last_name AS employee,
@@ -627,9 +607,9 @@ sub retrieve {
         for ( keys %$ref ) { $form->{$_} = $ref->{$_} unless ( $_ eq "id") }
         $sth->finish;
 
-        # get printed, emailed and queued
+        # get printed, emailed
         $query = qq|
-            SELECT s.printed, s.emailed, s.spoolfile, s.formname
+            SELECT s.printed, s.emailed, s.formname
             FROM status s
             WHERE s.trans_id = ?|;
         $sth = $dbh->prepare($query);
@@ -640,11 +620,9 @@ sub retrieve {
               if $ref->{printed};
             $form->{emailed} .= "$ref->{formname} "
               if $ref->{emailed};
-            $form->{queued} .= "$ref->{formname} $ref->{spoolfile} "
-              if $ref->{spoolfile};
         }
         $sth->finish;
-        for (qw(printed emailed queued)) { $form->{$_} =~ s/ +$//g }
+        for (qw(printed emailed)) { $form->{$_} =~ s/ +$//g }
 
         # retrieve individual items
         $query = qq|
@@ -667,7 +645,7 @@ sub retrieve {
             LEFT JOIN partsgroup pg ON (p.partsgroup_id = pg.id)
             LEFT JOIN partsvendor pv ON (pv.parts_id = p.id
                                            AND pv.credit_id = ?)
-            LEFT JOIN translation t
+            LEFT JOIN partsgroup_translation t
                 ON (t.trans_id = p.partsgroup_id
                     AND t.language_code = ?)
             WHERE o.trans_id = ?
@@ -701,7 +679,7 @@ sub retrieve {
 
             $form->db_parse_numeric(sth=>$sth, hashref=>$ref);
 
-            $bu_sth->execute($ref->{invoice_id});
+            $bu_sth->execute($ref->{orderitems_id});
             while ( $buref = $bu_sth->fetchrow_hashref(NAME_lc) ) {
                 $ref->{"b_unit_$buref->{class_id}"} = $buref->{bu_id};
             }
@@ -747,16 +725,8 @@ sub retrieve {
         $form->get_recurring;
     }
     else {
-
-        # get last name used
-        $form->lastname_used($form->{vc})
-          unless $form->{"$form->{vc}_id"};
-
         delete $form->{notes};
-
     }
-
-
 }
 
 sub exchangerate_defaults {
@@ -769,7 +739,6 @@ sub exchangerate_defaults {
 sub order_details {
     my ( $self, $myconfig, $form ) = @_;
 
-    # connect to database
     my $dbh = $form->{dbh};
     my $query;
     my $sth;
@@ -777,10 +746,6 @@ sub order_details {
     my $item;
     my $i;
     my @sortlist = ();
-    my $projectnumber;
-    my $projectdescription;
-    my $projectnumber_id;
-    my $translation;
     my @queryargs;
 
     my @taxaccounts;
@@ -789,15 +754,6 @@ sub order_details {
     my $taxrate;
     my $taxamount;
 
-    my $language_code = $form->{dbh}->quote( $form->{language_code} );
-    $query = qq|
-        SELECT p.description, t.description
-        FROM project p
-        LEFT JOIN translation t ON (t.trans_id = p.id AND
-            t.language_code = $language_code)
-           WHERE id = ?|;
-    my $prh = $dbh->prepare($query) || $form->dberror($query);
-
     $query = qq|
         SELECT inventory_accno_id, income_accno_id,
         expense_accno_id, assembly FROM parts
@@ -817,13 +773,10 @@ sub order_details {
             for ( keys %$ref ) { $form->{"${_}_$i"} = $ref->{$_} }
             $pth->finish;
 
-            $projectnumber_id      = 0;
-            $projectnumber         = "";
             $form->{partsgroup}    = "";
             $form->{projectnumber} = "";
 
-            if (   $form->{groupprojectnumber}
-                || $form->{grouppartsgroup} )
+            if ( $form->{grouppartsgroup} )
             {
 
                 $inventory_accno_id =
@@ -831,50 +784,7 @@ sub order_details {
                   ? "1"
                   : "";
 
-                if ( $form->{groupprojectnumber} ) {
-                    ( $projectnumber, $projectnumber_id ) =
-                      split /--/, $form->{"projectnumber_$i"};
-                }
-                if ( $form->{grouppartsgroup} ) {
-                    ( $form->{partsgroup} ) = split /--/,
-                      $form->{"partsgroup_$i"};
-                }
-
-                if (   $projectnumber_id
-                    && $form->{groupprojectnumber} )
-                {
-                    if ( $translation{$projectnumber_id} ) {
-                        $form->{projectnumber} =
-                          $translation{$projectnumber_id};
-                    }
-                    else {
-
-                        # get project description
-                        $prh->execute($projectnumber_id);
-                        ( $projectdescription, $translation ) =
-                          $prh->fetchrow_array;
-                        $prh->finish;
-
-                        $form->{projectnumber} =
-                          ($translation)
-                          ? "$projectnumber, \n" . "$translation"
-                          : "$projectnumber, \n" . "$projectdescription";
-
-                        $translation{$projectnumber_id} =
-                          $form->{projectnumber};
-                    }
-                }
-
-                if (   $form->{grouppartsgroup}
-                    && $form->{partsgroup} )
-                {
-                    $form->{projectnumber} .= " / "
-                      if $projectnumber_id;
-                    $form->{projectnumber} .= $form->{partsgroup};
-                }
-
-                $form->format_string($form->{projectnumber});
-
+                ( $form->{partsgroup} ) = split /--/, $form->{"partsgroup_$i"};
             }
 
             $sortby = qq|$projectnumber$form->{partsgroup}|;
@@ -1286,7 +1196,7 @@ sub order_details {
             );
 
             push( @{ $form->{taxdescription} },
-                $form->{"${item}_description"} );
+                $form->{_accno_descriptions}->{$item} );
 
             $form->{"${item}_taxrate"} =
               $form->format_amount( $myconfig, $form->{"${item}_rate"} * 100 );
@@ -1323,7 +1233,7 @@ sub order_details {
     for (qw(totalparts totalservices)) {
         $form->{$_} = $form->format_amount( $myconfig, $form->{$_}, 2 );
     }
-    for (qw(totalqty totalship totalweight)) {
+    for (qw(totalqty totalship totalweight totalweightship)) {
         $form->{$_} = $form->format_amount( $myconfig, $form->{$_} );
     }
     $form->{subtotal} = $form->format_amount( $myconfig, $form->{ordtotal}, 2 );
@@ -1352,13 +1262,7 @@ sub order_details {
       $form->format_amount( $myconfig, $form->{ordtotal}, 2 );
 
     $form->format_string(qw(text_amount text_decimal));
-
-    $query = qq|
-        SELECT value FROM defaults
-         WHERE setting_key = 'weightunit'|;
-    ( $form->{weightunit} ) = $dbh->selectrow_array($query);
-
-
+    $form->{weightunit} = $form->get_setting( 'weightunit' );
 }
 
 sub assembly_details {
@@ -1938,7 +1842,6 @@ sub transfer {
 sub get_soparts {
     my ( $self, $myconfig, $form ) = @_;
 
-    # connect to database
     my $dbh = $form->{dbh};
 
     my $id;
@@ -2068,7 +1971,6 @@ sub generate_orders {
         }
     }
 
-    # connect to database
     my $dbh = $form->{dbh};
 
     # foreign exchange rates
