@@ -22,6 +22,8 @@ CREATE TYPE tax_form_report_item AS (
     entity_class integer,
     control_code text,
     meta_number character varying(32),
+    tax_id text,
+    sales_tax_id text,
     acc_sum numeric,
     invoice_sum numeric,
     total_sum numeric);
@@ -34,6 +36,8 @@ CREATE TYPE tax_form_report_detail_item AS (
     entity_class integer,
     control_code text,
     meta_number character varying(32),
+    tax_id text,
+    sales_tax_id text,
     acc_sum numeric,
     invoice_sum numeric,
     total_sum numeric,
@@ -41,12 +45,14 @@ CREATE TYPE tax_form_report_detail_item AS (
     duedate text,
     invoice_id int);
 
-CREATE OR REPLACE FUNCTION tax_form_summary_report(in_tax_form_id int, in_begin date, in_end date)
+drop function if exists tax_form_summary_report(int,date,date);
+CREATE OR REPLACE FUNCTION tax_form_summary_report(in_tax_form_id int, in_from_date date, in_to_date date)
 RETURNS SETOF tax_form_report_item AS $BODY$
               SELECT entity_credit_account.id,
                      company.legal_name, company.entity_id,
                      entity_credit_account.entity_class, entity.control_code,
                      entity_credit_account.meta_number,
+                     company.tax_id, company.sales_tax_id,
                      sum(CASE WHEN gl.amount_bc = 0 THEN 0
                               WHEN relation = 'acc_trans'
                           THEN ac.reportable_amount_bc * pmt.amount_bc
@@ -95,14 +101,14 @@ RETURNS SETOF tax_form_report_item AS $BODY$
                                   OR (gl.invoice is false
                                      and ac.relation='acc_trans')))
                 JOIN (SELECT ac.trans_id, sum(ac.amount_bc) as amount_bc,
-                             as_array(entry_id) as entry_ids,
-                             as_array(chart_id) as chart_ids,
+                             array_agg(entry_id) as entry_ids,
+                             array_agg(chart_id) as chart_ids,
                              count(*) as num
                         FROM acc_trans ac
                        where chart_id in (select account_id
                                             from account_link
                                            where description like '%paid')
-                          AND transdate BETWEEN in_begin AND in_end
+                          AND transdate BETWEEN in_from_date AND in_to_date
                      group by ac.trans_id
                      ) pmt ON  (pmt.trans_id = gl.id)
                 JOIN entity_credit_account
@@ -111,20 +117,22 @@ RETURNS SETOF tax_form_report_item AS $BODY$
                 JOIN company ON (entity.id = company.entity_id)
                 JOIN country_tax_form ON (entity_credit_account.taxform_id = country_tax_form.id)
                WHERE country_tax_form.id = in_tax_form_id
-             GROUP BY legal_name, meta_number, company.entity_id, entity_credit_account.entity_class, entity.control_code, entity_credit_account.id
+             GROUP BY legal_name, meta_number, company.tax_id, company.sales_tax_id, company.entity_id, entity_credit_account.entity_class, entity.control_code, entity_credit_account.id
 $BODY$ LANGUAGE SQL;
 
 COMMENT ON FUNCTION tax_form_summary_report
-(in_tax_form_id int, in_begin date, in_end date) IS
+(in_tax_form_id int, in_from_date date, in_to_date date) IS
 $$This provides the total reportable value per vendor.  As per 1099 forms, these
 are cash-basis documents and show amounts paid.$$;
 
-CREATE OR REPLACE FUNCTION tax_form_details_report(in_tax_form_id int, in_begin date, in_end date, in_meta_number text)
+drop function if exists tax_form_details_report(int, date, date, text);
+CREATE OR REPLACE FUNCTION tax_form_details_report(in_tax_form_id int, in_from_date date, in_to_date date, in_meta_number text)
 RETURNS SETOF tax_form_report_detail_item AS $BODY$
               SELECT entity_credit_account.id,
                      company.legal_name, company.entity_id,
                      entity_credit_account.entity_class, entity.control_code,
                      entity_credit_account.meta_number,
+                     company.tax_id, company.sales_tax_id,
                      sum(CASE WHEN gl.amount_bc = 0 then 0
                               when relation = 'acc_trans'
                           THEN ac.reportable_amount_bc * pmt.amount_bc
@@ -177,33 +185,35 @@ RETURNS SETOF tax_form_report_detail_item AS $BODY$
                 JOIN company ON (entity.id = company.entity_id)
                 JOIN country_tax_form ON (entity_credit_account.taxform_id = country_tax_form.id)
                 JOIN (SELECT ac.trans_id, sum(ac.amount_bc) as amount_bc,
-                             as_array(entry_id) as entry_ids,
-                             as_array(chart_id) as chart_ids,
+                             array_agg(entry_id) as entry_ids,
+                             array_agg(chart_id) as chart_ids,
                              count(*) as num
                         FROM acc_trans ac
                        where chart_id in (select account_id
                                             from account_link
                                            where description like '%paid')
-                          AND transdate BETWEEN in_begin AND in_end
+                          AND transdate BETWEEN in_from_date AND in_to_date
                      group by ac.trans_id
                      ) pmt ON  (pmt.trans_id = gl.id)
                 WHERE country_tax_form.id = in_tax_form_id AND meta_number = in_meta_number
-                GROUP BY legal_name, meta_number, company.entity_id, entity_credit_account.entity_class, entity.control_code, gl.invnumber, gl.duedate, gl.id, entity_credit_account.id
+                GROUP BY legal_name, meta_number, company.entity_id, company.tax_id, company.sales_tax_id, entity_credit_account.entity_class, entity.control_code, gl.invnumber, gl.duedate, gl.id, entity_credit_account.id
 $BODY$ LANGUAGE SQL;
 
 COMMENT ON FUNCTION tax_form_details_report
-(in_tax_form_id int, in_begin date, in_end date, in_meta_number text) IS
+(in_tax_form_id int, in_from_date date, in_to_date date, in_meta_number text) IS
 $$ This provides a list of invoices and transactions that a report hits.  This
 is intended to allow an organization to adjust what is reported on the 1099
 before printing them.$$;
 
+drop function if exists tax_form_summary_report_accrual(int, date, date);
 CREATE OR REPLACE FUNCTION tax_form_summary_report_accrual
-(in_tax_form_id int, in_begin date, in_end date)
+(in_tax_form_id int, in_from_date date, in_to_date date)
 RETURNS SETOF tax_form_report_item AS $BODY$
               SELECT entity_credit_account.id,
                      company.legal_name, company.entity_id,
                      entity_credit_account.entity_class, entity.control_code,
                      entity_credit_account.meta_number,
+                     company.tax_id, company.sales_tax_id,
                      sum(CASE WHEN gl.amount_bc = 0 THEN 0
                               WHEN relation = 'acc_trans'
                           THEN ac.reportable_amount_bc
@@ -221,11 +231,11 @@ RETURNS SETOF tax_form_report_item AS $BODY$
 
                 FROM (select id, transdate, entity_credit_account, invoice,
                              amount_bc, 'ar' as class FROM ar
-                       WHERE transdate BETWEEN in_begin AND in_end
+                       WHERE transdate BETWEEN in_from_date AND in_to_date
                        UNION
                       select id, transdate, entity_credit_account, invoice,
                               amount_bc, 'ap' as class from ap
-                       WHERE transdate BETWEEN in_begin AND in_end
+                       WHERE transdate BETWEEN in_from_date AND in_to_date
                      ) gl
                JOIN (select trans_id, 'acc_trans' as relation,
                              sum(amount_bc) as amount_bc,
@@ -256,21 +266,23 @@ RETURNS SETOF tax_form_report_item AS $BODY$
                 JOIN company ON (entity.id = company.entity_id)
                 JOIN country_tax_form ON (entity_credit_account.taxform_id = country_tax_form.id)
                WHERE country_tax_form.id = in_tax_form_id
-             GROUP BY legal_name, meta_number, company.entity_id, entity_credit_account.entity_class, entity.control_code, entity_credit_account.id
+             GROUP BY legal_name, meta_number, company.entity_id, company.tax_id, company.sales_tax_id, entity_credit_account.entity_class, entity.control_code, entity_credit_account.id
 $BODY$ LANGUAGE SQL;
 
 COMMENT ON FUNCTION tax_form_summary_report_accrual
-(in_tax_form_id int, in_begin date, in_end date) IS
+(in_tax_form_id int, in_from_date date, in_to_date date) IS
 $$This provides the total reportable value per vendor.  As per 1099 forms, these
 are cash-basis documents and show amounts paid.$$;
 
+drop function if exists tax_form_details_report_accrual(int, date, date, text);
 CREATE OR REPLACE FUNCTION tax_form_details_report_accrual
-(in_tax_form_id int, in_begin date, in_end date, in_meta_number text)
+(in_tax_form_id int, in_from_date date, in_to_date date, in_meta_number text)
 RETURNS SETOF tax_form_report_detail_item AS $BODY$
               SELECT entity_credit_account.id,
                      company.legal_name, company.entity_id,
                      entity_credit_account.entity_class, entity.control_code,
                      entity_credit_account.meta_number,
+                     company.tax_id, company.sales_tax_id,
                      sum(CASE WHEN gl.amount_bc = 0 then 0
                               when relation = 'acc_trans'
                           THEN ac.reportable_amount_bc
@@ -291,12 +303,12 @@ RETURNS SETOF tax_form_report_detail_item AS $BODY$
                 FROM (select id, entity_credit_account, invnumber, duedate,
                              amount_bc, transdate, 'ar' as class
                         FROM ar
-                       WHERE transdate BETWEEN in_begin AND in_end
+                       WHERE transdate BETWEEN in_from_date AND in_to_date
                        UNION
                       select id, entity_credit_account, invnumber, duedate,
                              amount_bc, transdate, 'ap' as class
                         FROM ap
-                       WHERE transdate BETWEEN in_begin AND in_end
+                       WHERE transdate BETWEEN in_from_date AND in_to_date
                      ) gl
                 JOIN (select trans_id, 'acc_trans' as relation,
                              sum(amount_bc) as amount_bc,
@@ -323,11 +335,11 @@ RETURNS SETOF tax_form_report_detail_item AS $BODY$
                 JOIN company ON (entity.id = company.entity_id)
                 JOIN country_tax_form ON (entity_credit_account.taxform_id = country_tax_form.id)
                 WHERE country_tax_form.id = in_tax_form_id AND meta_number = in_meta_number
-                GROUP BY legal_name, meta_number, company.entity_id, entity_credit_account.entity_class, entity.control_code, gl.invnumber, gl.duedate, gl.id, entity_credit_account.id
+                GROUP BY legal_name, meta_number, company.entity_id, company.tax_id, company.sales_tax_id, entity_credit_account.entity_class, entity.control_code, gl.invnumber, gl.duedate, gl.id, entity_credit_account.id
 $BODY$ LANGUAGE SQL;
 
 COMMENT ON FUNCTION tax_form_details_report_accrual
-(in_tax_form_id int, in_begin date, in_end date, in_meta_number text) IS
+(in_tax_form_id int, in_from_date date, in_to_date date, in_meta_number text) IS
 $$ This provides a list of invoices and transactions that a report hits.  This
 is intended to allow an organization to adjust what is reported on the 1099
 before printing them.$$;
