@@ -261,6 +261,7 @@ CREATE TYPE account_config AS (
   category CHAR(1),
   gifi_accno text,
   heading int,
+  heading_negative_balance int,
   contra bool,
   tax bool,
   obsolete bool,
@@ -270,13 +271,15 @@ CREATE TYPE account_config AS (
 DROP FUNCTION IF EXISTS account_get(int);
 CREATE OR REPLACE FUNCTION account_get (in_id int) RETURNS account_config AS
 $$
-select c.*, concat_colon(l.description) as link
+  select c.id, c.accno, c.description, c.is_temp, c.category, c.gifi_accno,
+  c.heading, c.heading_negative_balance, c.contra, c.tax, c.obsolete,
+  string_agg(l.description, ':') as link
   from account c
   left join account_link l
     ON (c.id = l.account_id)
   where  id = $1
 group by c.id, c.accno, c.description, c.category,
-         c.heading, c.gifi_accno, c.contra, c.tax;
+         c.heading, c.heading_negative_balance, c.gifi_accno, c.contra, c.tax;
 $$ LANGUAGE sql;
 
 COMMENT ON FUNCTION account_get(in_id int) IS
@@ -404,13 +407,17 @@ COMMENT ON FUNCTION account_has_transactions (in_id int) IS
 $$ Checks to see if any transactions use this account.  If so, returns true.
 If not, returns false.$$;
 
-CREATE OR REPLACE FUNCTION account__save
+DROP FUNCTION IF EXISTS account__save
 (in_id int, in_accno text, in_description text, in_category char(1),
 in_gifi_accno text, in_heading int, in_contra bool, in_tax bool,
+in_link text[], in_obsolete bool, in_is_temp bool);
+
+CREATE OR REPLACE FUNCTION account__save
+(in_id int, in_accno text, in_description text, in_category char(1),
+in_gifi_accno text, in_heading int, in_heading_negative_balance int, in_contra bool, in_tax bool,
 in_link text[], in_obsolete bool, in_is_temp bool)
 RETURNS int AS $$
 DECLARE
-        t_heading_id int;
         t_link record;
         t_id int;
         t_tax bool;
@@ -427,13 +434,6 @@ BEGIN
                         RAISE EXCEPTION 'Invalid link settings:  Summary';
                 END IF;
         END LOOP;
-        -- heading settings
-        IF in_heading IS NULL THEN
-                SELECT id INTO t_heading_id FROM account_heading
-                WHERE accno < in_accno order by accno desc limit 1;
-        ELSE
-                t_heading_id := in_heading;
-        END IF;
 
         -- Remove all links. Later we'll (re-)insert the ones we want.
         DELETE FROM account_link
@@ -441,14 +441,15 @@ BEGIN
 
         UPDATE account
         SET accno = in_accno,
-                description = in_description,
-                category = in_category,
-                gifi_accno = in_gifi_accno,
-                heading = t_heading_id,
-                contra = in_contra,
-                obsolete = coalesce(in_obsolete,'f'),
-                tax = t_tax,
-                is_temp = coalesce(in_is_temp,'f')
+            description = in_description,
+            category = in_category,
+            gifi_accno = in_gifi_accno,
+            heading = in_heading,
+            heading_negative_balance = in_heading_negative_balance,
+            contra = in_contra,
+            obsolete = coalesce(in_obsolete,'f'),
+            tax = t_tax,
+            is_temp = coalesce(in_is_temp,'f')
         WHERE id = in_id;
 
         IF FOUND THEN
@@ -457,9 +458,10 @@ BEGIN
                 -- can't obsolete on insert, but this can be changed if users
                 -- request it --CT
                 INSERT INTO account (accno, description, category, gifi_accno,
-                        heading, contra, tax, is_temp)
+                        heading, heading_negative_balance, contra, tax, is_temp)
                 VALUES (in_accno, in_description, in_category, in_gifi_accno,
-                        t_heading_id, in_contra, in_tax, coalesce(in_is_temp, 'f'));
+                        in_heading, in_heading_negative_balance, in_contra,
+                        in_tax, coalesce(in_is_temp, 'f'));
 
                 t_id := currval('account_id_seq');
         END IF;
@@ -480,8 +482,9 @@ $$ language plpgsql;
 
 COMMENT ON FUNCTION account__save
 (in_id int, in_accno text, in_description text, in_category char(1),
-in_gifi_accno text, in_heading int, in_contra bool, in_tax bool,
-in_link text[], in_obsolete bool, in_is_temp bool) IS
+in_gifi_accno text, in_heading int, in_heading_negative_balance int,
+in_contra bool, in_tax bool, in_link text[], in_obsolete bool,
+in_is_temp bool) IS
 $$ This deletes existing account_link entries, where the
 account_link.description is not designated as a custom one in the
 account_link_description table.
@@ -508,6 +511,7 @@ BEGIN
     AND debits = 0
     AND credits = 0;
 
+    DELETE FROM cr_coa_to_account WHERE chart_id = in_id;
     DELETE FROM tax WHERE chart_id = in_id;
     DELETE FROM account_link WHERE account_id = in_id;
     DELETE FROM account WHERE id = in_id;
@@ -716,6 +720,19 @@ eca(account_id) AS (
     SELECT DISTINCT cash_account_id
       FROM entity_credit_account
 ),
+p(account_id) AS (
+  SELECT DISTINCT inventory_accno_id
+    FROM parts
+           UNION
+  SELECT DISTINCT income_accno_id
+    FROM parts
+           UNION
+  SELECT DISTINCT expense_accno_id
+    FROM parts
+           UNION
+  SELECT DISTINCT returns_accno_id
+    FROM parts
+),
 ta(account_id) AS (
     SELECT chart_id
       FROM eca_tax
@@ -726,7 +743,7 @@ SELECT a.id, a.is_heading, a.accno, a.description, a.gifi_accno,
             ELSE null::numeric END,
        CASE WHEN sum(ac.amount_bc) > 0 THEN sum(amount_bc)
             ELSE null::numeric END,
-       count(ac.*)+count(hh.*)+count(ha.*)+count(eca.*)+count(ta.*), l.link
+       count(ac.*)+count(hh.*)+count(ha.*)+count(eca.*)+count(p.*)+count(ta.*), l.link
   FROM (SELECT id, heading, false as is_heading, accno, description, gifi_accno
           FROM account
          UNION
@@ -738,6 +755,7 @@ SELECT a.id, a.is_heading, a.accno, a.description, a.gifi_accno,
  LEFT JOIN hh ON hh.parent_id = a.id AND a.is_heading
  LEFT JOIN ha ON ha.heading = a.id AND a.is_heading
  LEFT JOIN eca ON eca.account_id = a.id AND NOT a.is_heading
+ LEFT JOIN p ON p.account_id = a.id AND NOT a.is_heading
  LEFT JOIN ta ON ta.account_id = a.id AND NOT a.is_heading
   GROUP BY a.id, a.is_heading, a.accno, a.description, a.gifi_accno, l.link
   ORDER BY a.accno;
@@ -843,8 +861,8 @@ PERFORM * from (
   SELECT *
     FROM account_heading ah
     JOIN account_headings at ON ah.parent_id = at.id
-   WHERE at.path || '||||' ||  ah.accno NOT IN
-          (select path from account_headings)
+   WHERE NOT EXISTS (SELECT 1 FROM account_headings
+                  WHERE path = at.path || '||||' || ah.accno)
 ) x;
 
 IF found then
