@@ -7,7 +7,6 @@ use warnings;
 use LedgerSMB::Batch;
 use LedgerSMB::IR;
 use LedgerSMB::Form;
-use LedgerSMB::DBObject::Account;
 use LedgerSMB::DBObject::Reconciliation;
 use LedgerSMB::Entity::Company;
 use LedgerSMB::Entity::Credit_Account;
@@ -27,7 +26,8 @@ Given qr/a (fresh )?standard test company(?: named "(.*)")?/, sub {
     S->{ext_lsmb}->ensure_template;
 
     if (! S->{"the company"} || $fresh_required) {
-        $company //= "standard-$$-" . $company_seq++;
+        my $job_name = $ENV{T2_HARNESS_JOB_NAME} // '';
+        $company //= "standard-$$-$job_name-" . $company_seq++;
         S->{ext_lsmb}->create_from_template($company);
     }
 };
@@ -313,6 +313,7 @@ Given qr/inventory has been built up for '(.*)' from these transactions:$/, sub
 
         my $form = Form->new();
         $form->{dbh} = S->{ext_lsmb}->admin_dbh;
+        $form->{_wire} = S->{ext_lsmb}->wire;
         $form->{rowcount} = 2;
         $form->{transdate} = $trans->{transdate};
         $form->{qty_1} = $trans->{amount};
@@ -357,7 +358,8 @@ Given qr/^(\d+) units inventory of ((?:a|the) part|part "(.*)") purchased at (\d
     $dbh->do(
         q{
         INSERT INTO gl (reference, description, transdate, person_id, approved)
-               VALUES ('INV-INIT', 'Initial setup', '2020-01-01', '1', true);
+               VALUES ('INV-INIT', 'Initial setup', '2020-01-01',
+                       person__get_my_id(), true);
         })
         or die $dbh->errstr;
 
@@ -414,8 +416,10 @@ Given qr/(a batch|batches) with these properties:$/, sub {
 
 Given qr/^(a reconciliation report|reconciliation reports) with these properties:$/, sub {
 
-    my $db = PGObject::Simple->new();
-    $db->set_dbh(S->{ext_lsmb}->admin_dbh);
+    my $db = PGObject::Simple->new(
+        dbh => S->{ext_lsmb}->admin_dbh,
+        _funcschema => 'xyz',
+        );
 
     foreach my $report_spec (@{C->data}) {
 
@@ -528,11 +532,8 @@ Then qr/^I expect "(.*)" to be selected for "(.*)"$/, sub {
     );
     ok($element, "found element labeled '$label_text'");
 
-    my $option = $element->find(
-        qq{//span[\@role="option" and \@aria-selected="true" and .="$option_text"]}
-    );
-
-    ok($option, "Found option '$option_text' of dropdown '$label_text'");
+    ok($element->get_text eq $option_text,
+        "Found option '$option_text' of dropdown '$label_text'");
 };
 
 Given qr/a customer named "(.*)"/, sub {
@@ -543,10 +544,13 @@ Given qr/a customer named "(.*)"/, sub {
     #  the environment contains PGUSER='postgres', but the username
     #  was set to 'test-user-admin' -- yet the postgres value is used
     my $dbh = LedgerSMB::Database->new(
-        dbname => S->{"the company"},
-        usermame => $ENV{PGUSER},     ###TODO: we had 'S->{"the admin"}
-        password => $ENV{PGPASSWORD}, ### but that didn't work
-        host => 'localhost')
+        connect_data => {
+            dbname => S->{"the company"},
+            user     => $ENV{PGUSER},     ###TODO: we had 'S->{"the admin"}
+            password => $ENV{PGPASSWORD}, ### but that didn't work
+            host     => 'localhost',
+        },
+        schema => 'xyz')
         ->connect({ PrintError => 0, RaiseError => 1, AutoCommit => 0 });
 
     my $company = LedgerSMB::Entity::Company->new(
@@ -795,8 +799,8 @@ Given qr/that standard payment terms apply for "(.+)"/, sub {
             q{SELECT 1 FROM account WHERE accno = 'DISC'})) {
         $dbh->do(q{
               SELECT account__save(NULL, 'DISC', 'Payment discounts',
-                 'E', NULL, (select id from account where accno='5600'),
-                 false, false, ARRAY['AR_discount','AP_discount']::text[],
+                 'E', NULL, (select id from account_heading where accno='5600'),
+                 null, false, false, ARRAY['AR_discount','AP_discount']::text[],
                  false, false);
             })
             or die $dbh->errstr;
